@@ -1,15 +1,20 @@
 """
 Check if NSModalSessions are properly wrapped.
 
-XXX: This is not a proper unittest, it requires human eyes to check if the
+NOTE: This is not a proper unittest, it requires human eyes to check if the
 output is correct.
 """
-from __future__ import print_function
-import objc
-from Foundation import *
-from AppKit import *
 import time
 
+from Cocoa import (
+    NSApplication,
+    NSGetInformationalAlertPanel,
+    NSReleaseAlertPanel,
+    NSObject,
+    NSApp,
+    NSWindow,
+)
+
 
 def doTest():
     alertPanel = None
