@@ -4,7 +4,6 @@ Helper code for implementing unittests.
 This module is unsupported and is primairily used in the PyObjC
 testsuite.
 """
-from __future__ import print_function
 
 import contextlib
 import gc as _gc
@@ -14,6 +13,7 @@ import re as _re
 import struct as _struct
 import sys as _sys
 import unittest as _unittest
+import subprocess as _subprocess
 from distutils.sysconfig import get_config_var as _get_config_var
 
 import objc
@@ -181,6 +181,11 @@ def os_release():
     else:
         pl = _pl.readPlist("/System/Library/CoreServices/SystemVersion.plist")
     v = pl["ProductVersion"]
+    if v.startswith("10.16"):
+        # We're in a python compiled with an pre-macOS 11 SDK, which causes
+        # the system to ly to us. Use sw_vers to get the actual version.
+        v = _subprocess.check_output(["sw_vers", "-productVersion"]).decode().strip()
+
     return ".".join(v.split("."))
 
 
@@ -275,6 +280,24 @@ def onlyOn64Bit(function):
     return onlyIf(not is32Bit(), "64-bit only")(function)
 
 
+def arch_only(arch):
+    """
+    Usage::
+        class Tests (unittest.TestCase):
+
+            @arch_only("arm64")
+            def testArm64(self):
+                pass
+
+    The test runs only when the specified architecture matches
+    """
+
+    def decorator(function):
+        return onlyIf(objc.arch == arch, f"{arch} only")(function)
+
+    return decorator
+
+
 def min_python_release(version):
     """
     Usage::
@@ -286,9 +309,7 @@ def min_python_release(version):
                 pass
     """
     parts = tuple(map(int, version.split(".")))
-    return onlyIf(
-        _sys.version_info[:2] >= parts, "Requires Python %s or later" % (version,)
-    )
+    return onlyIf(_sys.version_info[:2] >= parts, f"Requires Python {version} or later")
 
 
 def _sort_key(version):
@@ -297,7 +318,7 @@ def _sort_key(version):
         parts.append("0")
 
     if len(parts) != 3:
-        raise ValueError("Invalid version: %r" % (version,))
+        raise ValueError(f"Invalid version: {version!r}")
 
     return tuple(int(x) for x in parts)
 
@@ -320,7 +341,7 @@ def min_sdk_level(release):
     """
     v = (objc.PyObjC_BUILD_RELEASE // 100, objc.PyObjC_BUILD_RELEASE % 100, 0)
     return onlyIf(
-        v >= os_level_key(release), "Requires build with SDK %s or later" % (release,)
+        v >= os_level_key(release), f"Requires build with SDK {release} or later"
     )
 
 
@@ -335,7 +356,7 @@ def max_sdk_level(release):
     """
     v = (objc.PyObjC_BUILD_RELEASE // 100, objc.PyObjC_BUILD_RELEASE % 100, 0)
     return onlyIf(
-        v <= os_level_key(release), "Requires build with SDK %s or later" % (release,)
+        v <= os_level_key(release), f"Requires build with SDK {release} or later"
     )
 
 
@@ -351,7 +372,7 @@ def min_os_level(release):
     """
     return onlyIf(
         os_level_key(os_release()) >= os_level_key(release),
-        "Requires OSX %s or later" % (release,),
+        f"Requires OSX {release} or later",
     )
 
 
@@ -367,7 +388,7 @@ def max_os_level(release):
     """
     return onlyIf(
         os_level_key(os_release()) <= os_level_key(release),
-        "Requires OSX upto %s" % (release,),
+        f"Requires OSX up to {release}",
     )
 
 
@@ -385,7 +406,7 @@ def os_level_between(min_release, max_release):
         os_level_key(min_release)
         <= os_level_key(os_release())
         <= os_level_key(max_release),
-        "Requires OSX %s upto %s" % (min_release, max_release),
+        f"Requires OSX {min_release} up to {max_release}",
     )
 
 
@@ -691,16 +712,16 @@ class TestCase(_unittest.TestCase):
 
     def assertIsCFType(self, tp, message=None):
         if not isinstance(tp, objc.objc_class):
-            self.fail(message or "%r is not a CFTypeRef type" % (tp,))
+            self.fail(message or f"{tp!r} is not a CFTypeRef type")
 
         if any(x is tp for x in _nscftype):
-            self.fail(message or "%r is not a unique CFTypeRef type" % (tp,))
+            self.fail(message or f"{tp!r} is not a unique CFTypeRef type")
 
         for cls in tp.__bases__:
             if "NSCFType" in cls.__name__:
                 return
 
-        self.fail(message or "%r is not a CFTypeRef type" % (tp,))
+        self.fail(message or f"{tp!r} is not a CFTypeRef type")
 
         # NOTE: Don't test if this is a subclass of one of the known
         #       CF roots, this tests is mostly used to ensure that the
@@ -714,17 +735,15 @@ class TestCase(_unittest.TestCase):
 
     def assertIsOpaquePointer(self, tp, message=None):
         if not hasattr(tp, "__pointer__"):
-            self.fail(message or "%r is not an opaque-pointer" % (tp,))
+            self.fail(message or f"{tp!r} is not an opaque-pointer")
 
         if not hasattr(tp, "__typestr__"):
-            self.fail(message or "%r is not an opaque-pointer" % (tp,))
+            self.fail(message or f"{tp!r} is not an opaque-pointer")
 
     def assertResultIsNullTerminated(self, method, message=None):
         info = method.__metadata__()
         if not info.get("retval", {}).get("c_array_delimited_by_null"):
-            self.fail(
-                message or "result of %r is not a null-terminated array" % (method,)
-            )
+            self.fail(message or f"result of {method!r} is not a null-terminated array")
 
     def assertIsNullTerminated(self, method, message=None):
         info = method.__metadata__()
@@ -776,9 +795,7 @@ class TestCase(_unittest.TestCase):
     def assertResultIsVariableSize(self, method, message=None):
         info = method.__metadata__()
         if not info.get("retval", {}).get("c_array_of_variable_length", False):
-            self.fail(
-                message or "result of %r is not a variable sized array" % (method,)
-            )
+            self.fail(message or f"result of {method!r} is not a variable sized array")
 
     def assertArgSizeInResult(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -806,7 +823,7 @@ class TestCase(_unittest.TestCase):
             offset = 0
         info = method.__metadata__()
         if not info.get("variadic"):
-            self.fail(message or "%r is not a variadic function" % (method,))
+            self.fail(message or f"{method!r} is not a variadic function")
 
         try:
             if not info["arguments"][argno + offset].get("printf_format"):
@@ -829,9 +846,9 @@ class TestCase(_unittest.TestCase):
 
         try:
             if not info["arguments"][argno + offset]["already_cfretained"]:
-                self.fail(message or "%r is not cfretained" % (method,))
+                self.fail(message or f"{method!r} is not cfretained")
         except (KeyError, IndexError):
-            self.fail(message or "%r is not cfretained" % (method,))
+            self.fail(message or f"{method!r} is not cfretained")
 
     def assertArgIsNotCFRetained(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -841,7 +858,7 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
         try:
             if info["arguments"][argno + offset]["already_cfretained"]:
-                self.fail(message or "%r is cfretained" % (method,))
+                self.fail(message or f"{method!r} is cfretained")
         except (KeyError, IndexError):
             pass
 
@@ -849,12 +866,12 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
 
         if not info.get("retval", {}).get("already_cfretained", False):
-            self.fail(message or "%r is not cfretained" % (method,))
+            self.fail(message or f"{method!r} is not cfretained")
 
     def assertResultIsNotCFRetained(self, method, message=None):
         info = method.__metadata__()
         if info.get("retval", {}).get("already_cfretained", False):
-            self.fail(message or "%r is cfretained" % (method,))
+            self.fail(message or f"{method!r} is cfretained")
 
     def assertArgIsRetained(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -865,9 +882,9 @@ class TestCase(_unittest.TestCase):
 
         try:
             if not info["arguments"][argno + offset]["already_retained"]:
-                self.fail(message or "%r is not retained" % (method,))
+                self.fail(message or f"{method!r} is not retained")
         except (KeyError, IndexError):
-            self.fail(message or "%r is not retained" % (method,))
+            self.fail(message or f"{method!r} is not retained")
 
     def assertArgIsNotRetained(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -877,19 +894,19 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
         try:
             if info["arguments"][argno + offset]["already_retained"]:
-                self.fail(message or "%r is retained" % (method,))
+                self.fail(message or f"{method!r} is retained")
         except (KeyError, IndexError):
             pass
 
     def assertResultIsRetained(self, method, message=None):
         info = method.__metadata__()
         if not info.get("retval", {}).get("already_retained", False):
-            self.fail(message or "%r is not retained" % (method,))
+            self.fail(message or f"{method!r} is not retained")
 
     def assertResultIsNotRetained(self, method, message=None):
         info = method.__metadata__()
         if info.get("retval", {}).get("already_retained", False):
-            self.fail(message or "%r is retained" % (method,))
+            self.fail(message or f"{method!r} is retained")
 
     def assertResultHasType(self, method, tp, message=None):
         info = method.__metadata__()
@@ -901,7 +918,7 @@ class TestCase(_unittest.TestCase):
         ):
             self.fail(
                 message
-                or "result of %r is not of type %r, but %r" % (method, tp, typestr)
+                or f"result of {method!r} is not of type {tp!r}, but {typestr!r}"
             )
 
     def assertArgHasType(self, method, argno, tp, message=None):
@@ -997,22 +1014,18 @@ class TestCase(_unittest.TestCase):
             i = info["retval"]
         except (KeyError, IndexError):
             self.fail(
-                message or "result of %s has no metadata (or doesn't exist)" % (method,)
+                message or f"result of {method} has no metadata (or doesn't exist)"
             )
 
         else:
             typestr = i.get("type", b"@")
 
         if typestr != b"^?":
-            self.fail(
-                message or "result of %s is not of type function_pointer" % (method,)
-            )
+            self.fail(message or f"result of {method} is not of type function_pointer")
 
         st = i.get("callable")
         if st is None:
-            self.fail(
-                message or "result of %s is not of type function_pointer" % (method,)
-            )
+            self.fail(message or f"result of {method} is not of type function_pointer")
 
         try:
             iface = st["retval"]["type"]
@@ -1087,12 +1100,11 @@ class TestCase(_unittest.TestCase):
             typestr = info["retval"]["type"]
             if typestr != b"@?":
                 self.fail(
-                    message
-                    or "result of %s is not of type block: %s" % (method, typestr)
+                    message or f"result of {method} is not of type block: {typestr}"
                 )
         except KeyError:
             self.fail(
-                message or "result of %s is not of type block: %s" % (method, b"v")
+                message or "result of {} is not of type block: {}".format(method, b"v")
             )
 
         st = info["retval"].get("callable")
@@ -1132,12 +1144,12 @@ class TestCase(_unittest.TestCase):
             i = info["retval"]
         except (KeyError, IndexError):
             self.fail(
-                message or "result of %s has no metadata (or doesn't exist)" % (method,)
+                message or f"result of {method} has no metadata (or doesn't exist)"
             )
 
         typestr = i.get("type", b"@")
         if typestr != objc._C_SEL:
-            self.fail(message or "result of %s is not of type SEL" % (method,))
+            self.fail(message or f"result of {method} is not of type SEL")
 
         st = i.get("sel_of_type")
         if st != sel_type and _typemap(st) != _typemap(sel_type):
@@ -1178,8 +1190,7 @@ class TestCase(_unittest.TestCase):
         typestr = info["retval"]["type"]
         if typestr != objc._C_NSBOOL:
             self.fail(
-                message
-                or "result of %s is not of type BOOL, but %r" % (method, typestr)
+                message or f"result of {method} is not of type BOOL, but {typestr!r}"
             )
 
     def assertArgIsBOOL(self, method, argno, message=None):
@@ -1366,7 +1377,7 @@ class TestCase(_unittest.TestCase):
 
         def assertStartswith(self, value, test, message=None):  # pragma: no cover
             if not value.startswith(test):
-                self.fail(message or "%r does not start with %r" % (value, test))
+                self.fail(message or f"{value!r} does not start with {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertIs"):  # pragma: no cover
 
@@ -1382,7 +1393,7 @@ class TestCase(_unittest.TestCase):
 
         def assertIsNot(self, value, test, message=None):
             if value is test:
-                self.fail(message or "%r is %r" % (value, test))
+                self.fail(message or f"{value!r} is {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertIsNone"):  # pragma: no cover
 
@@ -1393,27 +1404,27 @@ class TestCase(_unittest.TestCase):
 
         def assertIsNotNone(self, value, message=None):
             if value is None:
-                self.fail(message, "%r is not None" % (value,))
+                self.fail(message, f"{value!r} is not None")
 
     if not hasattr(_unittest.TestCase, "assertHasAttr"):  # pragma: no cover
 
         def assertHasAttr(self, value, key, message=None):
             if not hasattr(value, key):
-                self.fail(message or "%s is not an attribute of %r" % (key, value))
+                self.fail(message or f"{key} is not an attribute of {value!r}")
 
     if not hasattr(_unittest.TestCase, "assertNotHasAttr"):  # pragma: no cover
 
         def assertNotHasAttr(self, value, key, message=None):
             if hasattr(value, key):
-                self.fail(message or "%s is an attribute of %r" % (key, value))
+                self.fail(message or f"{key} is an attribute of {value!r}")
 
     def assertIsSubclass(self, value, types, message=None):
         if not issubclass(value, types):
-            self.fail(message or "%s is not a subclass of %r" % (value, types))
+            self.fail(message or f"{value} is not a subclass of {types!r}")
 
     def assertIsNotSubclass(self, value, types, message=None):
         if issubclass(value, types):
-            self.fail(message or "%s is a subclass of %r" % (value, types))
+            self.fail(message or f"{value} is a subclass of {types!r}")
 
     if not hasattr(_unittest.TestCase, "assertIsInstance"):  # pragma: no cover
 
@@ -1429,52 +1440,73 @@ class TestCase(_unittest.TestCase):
 
         def assertIsNotInstance(self, value, types, message=None):
             if isinstance(value, types):
-                self.fail(message or "%s is an instance of %r" % (value, types))
+                self.fail(message or f"{value} is an instance of {types!r}")
 
     if not hasattr(_unittest.TestCase, "assertIn"):  # pragma: no cover
 
         def assertIn(self, value, seq, message=None):
             if value not in seq:
-                self.fail(message or "%r is not in %r" % (value, seq))
+                self.fail(message or f"{value!r} is not in {seq!r}")
 
     if not hasattr(_unittest.TestCase, "assertNotIn"):  # pragma: no cover
 
         def assertNotIn(self, value, seq, message=None):
             if value in seq:
-                self.fail(message or "%r is in %r" % (value, seq))
+                self.fail(message or f"{value!r} is in {seq!r}")
 
     if not hasattr(_unittest.TestCase, "assertGreaterThan"):  # pragma: no cover
 
         def assertGreaterThan(self, val, test, message=None):
             if not (val > test):
-                self.fail(message or "%r <= %r" % (val, test))
+                self.fail(message or f"{val!r} <= {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertGreaterEqual"):  # pragma: no cover
 
         def assertGreaterEqual(self, val, test, message=None):
             if not (val >= test):
-                self.fail(message or "%r < %r" % (val, test))
+                self.fail(message or f"{val!r} < {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertLessThan"):  # pragma: no cover
 
         def assertLessThan(self, val, test, message=None):
             if not (val < test):
-                self.fail(message or "%r >= %r" % (val, test))
+                self.fail(message or f"{val!r} >= {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertLessEqual"):  # pragma: no cover
 
         def assertLessEqual(self, val, test, message=None):
             if not (val <= test):
-                self.fail(message or "%r > %r" % (val, test))
+                self.fail(message or f"{val!r} > {test!r}")
 
     if not hasattr(_unittest.TestCase, "assertAlmostEquals"):  # pragma: no cover
 
         def assertAlmostEquals(self, val1, val2, message=None):
-            self.failUnless(
+            self.assertTrue(
                 abs(val1 - val2) < 0.00001,
-                message or "abs(%r - %r) >= 0.00001" % (val1, val2),
+                message or f"abs({val1!r} - {val2!r}) >= 0.00001",
             )
 
+    def assertClassIsFinal(self, cls):
+        if not isinstance(cls, objc.objc_class):
+            self.fail(f"{cls} is not an Objective-C class")
+        elif not cls.__objc_final__:
+            self.fail(f"{cls} is not an final class")
+
+    def assertProtocolExists(self, name):
+        ok = True
+        try:
+            proto = objc.protocolNamed(name)
+
+        except objc.ProtocolError:
+            ok = False
+
+        if not ok:
+            self.fail(f"Protocol {name!r} does not exist")
+
+        if not isinstance(proto, objc.formal_protocol):
+            # Should never happen
+            self.fail(f"Protocol {name!r} is not a protocol, but {type(proto)}")
+
     def run(self, *args):
         """
         Run the test, same as unittest.TestCase.run, but every test is
