@@ -1,5 +1,7 @@
 """
-Backward compatibity with bridgesupport files
+Backward compatibility with bridgesupport files
+
+This functionality is deprecated and will be removed in PyObjC 9.
 """
 __all__ = ("initFrameworkWrapper", "parseBridgeSupport")
 
@@ -33,7 +35,7 @@ for method in (
 # XML files.
 
 # NOTE: This search path only contains system locations to
-# avoid accidently reiying on system-specific functionality.
+# avoid accidentally reiying on system-specific functionality.
 BRIDGESUPPORT_DIRECTORIES = ["/System/Library/BridgeSupport"]
 
 _SENTINEL = object()
@@ -68,7 +70,7 @@ def _as_string(value):
     return value
 
 
-class _BridgeSupportParser(object):
+class _BridgeSupportParser:
     """
     Parser for the bridge support file format.
 
@@ -100,7 +102,7 @@ class _BridgeSupportParser(object):
             raise objc.error("invalid root node in bridgesupport file")
 
         for node in root:
-            method = getattr(self, "do_%s" % (node.tag,), None)
+            method = getattr(self, f"do_{node.tag}", None)
             if method is None:
                 continue
 
@@ -315,7 +317,7 @@ class _BridgeSupportParser(object):
         typestr = self.typestr2typestr(typestr)
 
         if typestr.startswith(objc._C_STRUCT_B):
-            # Look for structs with embbeded function pointers
+            # Look for structs with embedded function pointers
             # and ignore those
 
             def has_embedded_function(typestr):
@@ -586,7 +588,7 @@ class _BridgeSupportParser(object):
                 value = value.encode("latin1")
             except UnicodeError as e:
                 warnings.warn(
-                    "Error parsing BridgeSupport data for constant %s: %s" % (name, e),
+                    f"Error parsing BridgeSupport data for constant {name}: {e}",
                     RuntimeWarning,
                 )
                 return
@@ -600,6 +602,11 @@ _libraries = []
 def parseBridgeSupport(
     xmldata, globals, frameworkName, dylib_path=None, inlineTab=None  # noqa: A002
 ):
+    warnings.warn(
+        "This function will be removed in PyObjC 9, switch to the modern metadata system",
+        DeprecationWarning,
+        stacklevel=2,
+    )
 
     if dylib_path:
         _get_ctypes()
@@ -662,7 +669,7 @@ def _parseBridgeSupport(data, globals, frameworkName, *args, **kwds):  # noqa: A
         import warnings
 
         warnings.warn(
-            "Error parsing BridgeSupport data for %s: %s" % (frameworkName, e),
+            f"Error parsing BridgeSupport data for {frameworkName}: {e}",
             RuntimeWarning,
         )
 
@@ -691,6 +698,11 @@ def initFrameworkWrapper(
     either one embedded in the framework or one in a BrigeSupport library
     directory).
     """
+    warnings.warn(
+        "This function will be removed in PyObjC 9, switch to the modern metadata system",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if frameworkResourceName is None:
         frameworkResourceName = frameworkName
 
@@ -817,7 +829,7 @@ def _structConvenience(structname, structencoding):
             return objc.ivar(name=name, type=structencoding)
 
     makevar.__name__ = structname
-    makevar.__doc__ = "Create *ivar* for type encoding %r" % (structencoding,)
+    makevar.__doc__ = f"Create *ivar* for type encoding {structencoding!r}"
     if hasattr(objc.ivar, "__qualname__"):
         makevar.__qualname__ = objc.ivar.__qualname__ + "." + structname
     _ivar_dict[structname] = classmethod(makevar)
