@@ -1,7 +1,7 @@
 """
 Helper module that will enable lazy imports of Cocoa wrapper items.
 
-This should improve startup times and memory usage, at the cost
+This improves startup times and memory usage, at the cost
 of not being able to use 'from Cocoa import *'
 """
 __all__ = ("ObjCLazyModule",)
@@ -55,7 +55,7 @@ def _loadBundle(frameworkName, frameworkIdentifier, frameworkPath):
     return bundle
 
 
-class GetAttrMap(object):
+class GetAttrMap:
     __slots__ = ("_container",)
 
     def __init__(self, container):
@@ -104,7 +104,7 @@ class ObjCLazyModule(ModuleType):
         initialdict=None,
         parents=(),
     ):
-        super(ObjCLazyModule, self).__init__(name)
+        super().__init__(name)
 
         if frameworkIdentifier is not None or frameworkPath is not None:
             self.__bundle = self.__dict__["__bundle__"] = _loadBundle(
@@ -326,8 +326,11 @@ class ObjCLazyModule(ModuleType):
             except AttributeError:
                 all_names.update(dir(p))
 
-        # Add all class names
-        all_names.update(cls.__name__ for cls in getClassList())
+        # Add all class names, ignoring names with a dot because
+        # those are not valid attribute names (and in general are private)
+        all_names.update(
+            cls.__name__ for cls in getClassList() if "." not in cls.__name__
+        )
 
         return [v for v in all_names if not v.startswith("_")]
 
@@ -349,7 +352,7 @@ class ObjCLazyModule(ModuleType):
     def __get_constant(self, name):
         if self.__varmap_dct:
             if name in self.__varmap_dct:
-                tp = self.__varmap_dct.pop(name)
+                tp = self.__varmap_dct[name]
                 if tp.startswith("=="):
                     tp = tp[2:]
                     magic = 2
@@ -359,13 +362,14 @@ class ObjCLazyModule(ModuleType):
                 else:
                     magic = 0
                 result = objc._loadConstant(name, tp, magic)
+                self.__varmap_dct.pop(name)
                 if name in self.__varmap_deprecated:
                     _check_deprecated(name, self.__varmap_deprecated[name])
 
                 return result
 
         if self.__varmap:
-            m = re.search(r"\$%s(@[^$]*)?\$" % (name,), self.__varmap)
+            m = re.search(fr"\${name}(@[^$]*)?\$", self.__varmap)
             if m is not None:
                 tp = m.group(1)
                 if not tp:
@@ -391,7 +395,7 @@ class ObjCLazyModule(ModuleType):
                 return result
 
         if self.__enummap:
-            m = re.search(r"\$%s@([^$]*)\$" % (name,), self.__enummap)
+            m = re.search(fr"\${name}@([^$]*)\$", self.__enummap)
             if m is not None:
                 result = self.__prs_enum(m.group(1))
                 if name in self.__enum_deprecated:
@@ -400,17 +404,14 @@ class ObjCLazyModule(ModuleType):
 
         if self.__funcmap:
             if name in self.__funcmap:
-                # NOTE: Remove 'name' from funcmap because
-                #       it won't be needed anymore (either the
-                #       function doesn't exist, or it is loaded)
-                #       Should use slightly less memory.
-                info = self.__funcmap.pop(name)
+                info = self.__funcmap[name]
 
                 func_list = [(name,) + info]
 
                 d = {}
                 objc.loadBundleFunctions(self.__bundle, d, func_list)
                 if name in d:
+                    self.__funcmap.pop(name)
                     return d[name]
 
                 if self.__inlinelist is not None:
@@ -425,15 +426,17 @@ class ObjCLazyModule(ModuleType):
                 # NOTE: 'name' is popped because it is no longer needed
                 #       in the metadata and popping should slightly reduce
                 #       memory usage.
-                info = self.__expressions.pop(name)
+                info = self.__expressions[name]
                 try:
-                    return eval(info, {}, self.__expressions_mapping)
+                    result = eval(info, {}, self.__expressions_mapping)
+                    self.__expressions.pop(name)
+                    return result
                 except:  # noqa: E722, B001. Ignore all errors in evaluation the expression.
                     pass
 
         if self.__aliases:
             if name in self.__aliases:
-                alias = self.__aliases.pop(name)
+                alias = self.__aliases[name]
                 if alias == "ULONG_MAX":
                     result = (sys.maxsize * 2) + 1
                 elif alias == "LONG_MAX":
@@ -455,6 +458,8 @@ class ObjCLazyModule(ModuleType):
                 else:
                     result = getattr(self, alias)
 
+                self.__aliases.pop(name)
+
                 if name in self.__aliases_deprecated:
                     _check_deprecated(name, self.__aliases_deprecated[name])
                 return result
