@@ -9,6 +9,8 @@
 #import "pyobjc.h"
 #import <Foundation/Foundation.h>
 
+NS_ASSUME_NONNULL_BEGIN
+
 /*!
  * @class OC_NSBundleHack
  * @abstract NSBundle hacks to support plugins
@@ -17,14 +19,14 @@
  *     if it does not do the right thing
  */
 
-@interface OC_NSBundleHack : NSBundle
-{
+@interface OC_NSBundleHack : NSBundle {
 }
-+(void)installBundleHack;
++ (void)installBundleHack;
 @end
 
-@interface OC_NSBundleHackCheck : NSObject
-{
+@interface OC_NSBundleHackCheck : NSObject {
 }
-+(NSBundle*)bundleForClass;
++ (NSBundle* _Nullable)bundleForClass;
 @end
+
+NS_ASSUME_NONNULL_END
