@@ -1,9 +1,10 @@
 #include "pyobjc.h"
-#import "OC_PythonDate.h"
+
+NS_ASSUME_NONNULL_BEGIN
 
 @implementation OC_PythonDate
 
-+ (instancetype)dateWithPythonObject:(PyObject*)v
++ (instancetype _Nullable)dateWithPythonObject:(PyObject*)v
 {
     OC_PythonDate* res;
 
@@ -11,10 +12,11 @@
     return [res autorelease];
 }
 
-- (id)initWithPythonObject:(PyObject*)v
+- (id _Nullable)initWithPythonObject:(PyObject*)v
 {
     self = [super init];
-    if (unlikely(self == nil)) return nil;
+    if (unlikely(self == nil))
+        return nil;
 
     oc_value = nil;
 
@@ -22,28 +24,30 @@
     return self;
 }
 
--(PyObject*)__pyobjc_PythonObject__
+- (PyObject*)__pyobjc_PythonObject__
 {
     Py_INCREF(value);
     return value;
 }
 
--(PyObject*)__pyobjc_PythonTransient__:(int*)cookie
+- (PyObject*)__pyobjc_PythonTransient__:(int*)cookie
 {
     *cookie = 0;
     Py_INCREF(value);
     return value;
 }
 
--(BOOL)supportsWeakPointers {
+- (BOOL)supportsWeakPointers
+{
     return YES;
 }
 
-+ (BOOL)supportsSecureCoding {
++ (BOOL)supportsSecureCoding
+{
     return NO;
 }
 
--(oneway void)release
+- (oneway void)release
 {
     /* See comment in OC_PythonUnicode */
     if (unlikely(!Py_IsInitialized())) {
@@ -57,7 +61,7 @@
     PyObjC_END_WITH_GIL
 }
 
--(void)dealloc
+- (void)dealloc
 {
     [oc_value release];
     oc_value = nil;
@@ -76,43 +80,45 @@
     [super dealloc];
 }
 
-
 - (void)encodeWithCoder:(NSCoder*)coder
 {
     PyObjC_encodeWithCoder(value, coder);
 }
 
-
 /*
  * Helper method for initWithCoder, needed to deal with
  * recursive objects (e.g. o.value = o)
  */
--(void)pyobjcSetValue:(NSObject*)other
+- (void)pyobjcSetValue:(NSObject*)other
 {
     PyObjC_BEGIN_WITH_GIL
-        PyObject* v = PyObjC_IdToPython(other);
+        PyObject* v = id_to_python(other);
 
         SET_FIELD(value, v);
 
     PyObjC_END_WITH_GIL
 }
 
--(id)initWithCoder:(NSCoder*)coder
+- (id _Nullable)initWithCoder:(NSCoder*)coder
 {
     value = NULL;
 
     if (PyObjC_Decoder != NULL) {
         PyObjC_BEGIN_WITH_GIL
-            PyObject* cdr = PyObjC_IdToPython(coder);
+            PyObject* cdr = id_to_python(coder);
             if (cdr == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
             }
 
             PyObject* setValue;
             PyObject* selfAsPython = PyObjCObject_New(self, 0, YES);
+            if (selfAsPython == NULL) {
+                PyObjC_GIL_FORWARD_EXC();
+            }
             setValue = PyObject_GetAttrString(selfAsPython, "pyobjcSetValue_");
 
-            PyObject* v = PyObject_CallFunction(PyObjC_Decoder, "OO", cdr, setValue);
+            PyObject* v = PyObjC_CallDecoder(cdr, setValue);
+
             Py_DECREF(cdr);
             Py_DECREF(setValue);
             Py_DECREF(selfAsPython);
@@ -130,28 +136,38 @@
 
     } else {
         [NSException raise:NSInvalidArgumentException
-                format:@"decoding Python objects is not supported"];
+                    format:@"decoding Python objects is not supported"];
         return nil;
-
     }
 }
 
--(NSDate*)_make_oc_value
+- (NSDate*)_make_oc_value
 {
     if (oc_value == nil) {
         PyObjC_BEGIN_WITH_GIL
             PyObject* v;
 
-            v = PyObject_CallMethod(value, "strftime", "s",
-                "%Y-%m-%d %H:%M:%S %z");
+            PyObject* args[3] = {NULL, value, PyObjCNM_date_format_string};
+            if (args[2] == NULL) {
+                PyObjC_GIL_FORWARD_EXC();
+            }
+            v = PyObject_VectorcallMethod(PyObjCNM_strftime, args + 1,
+                                          2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+
             if (v == NULL) {
                 /* Raise ObjC exception */
                 PyObjC_GIL_FORWARD_EXC();
             }
 
+            NSString* oc_v;
+            if (depythonify_python_object(v, &oc_v) == -1) {
+                Py_DECREF(v);
+                PyObjC_GIL_FORWARD_EXC();
+            }
+
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
-            oc_value = [NSDate dateWithString: PyObjC_PythonToId(v)];
+            oc_value = [NSDate dateWithString:oc_v];
 #pragma clang diagnostic pop
             [oc_value retain];
             Py_DECREF(v);
@@ -162,9 +178,9 @@
                  */
                 char buf[128];
 
-                NSTimeZone* zone = [NSTimeZone defaultTimeZone];
-                NSInteger offset = [zone secondsFromGMT];
-                char posneg;
+                NSTimeZone* zone   = [NSTimeZone defaultTimeZone];
+                NSInteger   offset = [zone secondsFromGMT];
+                char        posneg;
                 if (offset < 0) {
                     posneg = '-';
                     offset = -offset;
@@ -174,20 +190,30 @@
                 offset = offset / 60; /* Seconds to minutes */
 
                 NSInteger minutes = offset % 60;
-                NSInteger hours = offset / 60;
+                NSInteger hours   = offset / 60;
 
-
-
-                snprintf(buf, sizeof(buf), "%%Y-%%m-%%d %%H:%%M:%%S %c%02ld%02ld",
-                    posneg, (long)hours, (long)minutes);
-                v = PyObject_CallMethod(value, "strftime", "s", buf);
+                snprintf(buf, sizeof(buf), "%%Y-%%m-%%d %%H:%%M:%%S %c%02ld%02ld", posneg,
+                         (long)hours, (long)minutes);
+                args[2] = PyUnicode_FromString(buf);
+                if (args[2] == NULL) {
+                    PyObjC_GIL_FORWARD_EXC();
+                }
+                v = PyObject_VectorcallMethod(PyObjCNM_strftime, args + 1,
+                                              2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+                Py_DECREF(args[2]);
                 if (v == NULL) {
                     /* Raise ObjC exception */
+                    PyObjC_GIL_FORWARD_EXC();
                 }
 
+                NSString* oc_v;
+                if (depythonify_python_object(v, &oc_v) == -1) {
+                    Py_DECREF(v);
+                    PyObjC_GIL_FORWARD_EXC();
+                }
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
-                oc_value = [NSDate dateWithString: PyObjC_PythonToId(v)];
+                oc_value = [NSDate dateWithString:oc_v];
 #pragma clang diagnostic pop
                 [oc_value retain];
                 Py_DECREF(v);
@@ -198,12 +224,11 @@
     return oc_value;
 }
 
--(NSTimeInterval)timeIntervalSinceReferenceDate
+- (NSTimeInterval)timeIntervalSinceReferenceDate
 {
     return [[self _make_oc_value] timeIntervalSinceReferenceDate];
 }
 
-
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
 
@@ -214,12 +239,13 @@
 
 #pragma clang diagnostic pop
 
-- (NSComparisonResult)compare:(NSDate *)anotherDate
+- (NSComparisonResult)compare:(NSDate*)anotherDate
 {
     return [[self _make_oc_value] compare:anotherDate];
 }
 
-- (NSCalendarDate *)dateWithCalendarFormat:(NSString *)formatString timeZone:(NSTimeZone *)timeZone
+- (NSCalendarDate*)dateWithCalendarFormat:(NSString* _Nullable)formatString
+                                 timeZone:(NSTimeZone* _Nullable)timeZone
 {
     return [[self _make_oc_value] dateWithCalendarFormat:formatString timeZone:timeZone];
 }
@@ -229,17 +255,21 @@
     return [[self _make_oc_value] description];
 }
 
-- (NSString *)descriptionWithCalendarFormat:(NSString *)formatString timeZone:(NSTimeZone *)aTimeZone locale:(id)localeDictionary
+- (NSString* _Nullable)descriptionWithCalendarFormat:(NSString* _Nullable)formatString
+                                            timeZone:(NSTimeZone* _Nullable)aTimeZone
+                                              locale:(id _Nullable)localeDictionary
 {
-    return [[self _make_oc_value] descriptionWithCalendarFormat:formatString timeZone:aTimeZone locale:localeDictionary];
+    return [[self _make_oc_value] descriptionWithCalendarFormat:formatString
+                                                       timeZone:aTimeZone
+                                                         locale:localeDictionary];
 }
 
-- (NSString *)descriptionWithLocale:(id)localeDictionary
+- (NSString*)descriptionWithLocale:(id _Nullable)localeDictionary
 {
     return [[self _make_oc_value] descriptionWithLocale:localeDictionary];
 }
 
-- (NSDate *)earlierDate:(NSDate *)anotherDate
+- (NSDate*)earlierDate:(NSDate*)anotherDate
 {
     if ([[self _make_oc_value] earlierDate:anotherDate] == self) {
         return self;
@@ -248,14 +278,12 @@
     }
 }
 
-
-- (BOOL)isEqualToDate:(NSDate *)anotherDate
+- (BOOL)isEqualToDate:(NSDate*)anotherDate
 {
     return [[self _make_oc_value] isEqualToDate:anotherDate];
 }
 
-
-- (NSDate *)laterDate:(NSDate *)anotherDate
+- (NSDate*)laterDate:(NSDate*)anotherDate
 {
     if ([[self _make_oc_value] laterDate:anotherDate] == self) {
         return self;
@@ -269,7 +297,7 @@
     return [[self _make_oc_value] timeIntervalSince1970];
 }
 
-- (NSTimeInterval)timeIntervalSinceDate:(NSDate *)anotherDate
+- (NSTimeInterval)timeIntervalSinceDate:(NSDate*)anotherDate
 {
     return [[self _make_oc_value] timeIntervalSinceDate:anotherDate];
 }
@@ -279,5 +307,6 @@
     return [[self _make_oc_value] timeIntervalSinceNow];
 }
 
-
 @end /* implementation OC_PythonDate */
+
+NS_ASSUME_NONNULL_END
