@@ -1,16 +1,18 @@
 #include "pyobjc.h"
 
+NS_ASSUME_NONNULL_BEGIN
+
 @implementation OC_PythonEnumerator
 
-+(instancetype)enumeratorWithPythonObject:(PyObject*)object
++ (instancetype)enumeratorWithPythonObject:(PyObject*)object
 {
     return [[[self alloc] initWithPythonObject:object] autorelease];
 }
 
--(id)initWithPythonObject:(PyObject*)object
+- (id)initWithPythonObject:(PyObject*)object
 {
-    self = [super init];
-    if (self == nil) return nil;
+    /* -[NSObject init] is documented as return self */
+    self = (id _Nonnull)[super init];
 
     SET_FIELD_INCREF(value, object);
     valid = YES;
@@ -18,7 +20,7 @@
     return self;
 }
 
--(oneway void)release
+- (oneway void)release
 {
     /* See comment in OC_PythonUnicode */
     if (unlikely(!Py_IsInitialized())) {
@@ -32,7 +34,7 @@
     PyObjC_END_WITH_GIL
 }
 
--(void)dealloc
+- (void)dealloc
 {
     if (unlikely(!Py_IsInitialized())) {
         [super release];
@@ -46,13 +48,13 @@
     [super dealloc];
 }
 
--(id)nextObject
+- (id _Nullable)nextObject
 {
     if (!valid) {
         return nil;
     }
 
-    NSObject* result = nil;
+    NSObject* result;
 
     PyObjC_BEGIN_WITH_GIL
         PyObject* object = PyIter_Next(value);
@@ -64,31 +66,26 @@
             } else {
                 PyObjC_GIL_FORWARD_EXC();
             }
-
         }
 
         if (object == Py_None) {
             result = [NSNull null];
         } else {
-            result = PyObjC_PythonToId(object);
-        }
-        if (result == nil) {
-            if (PyErr_Occurred()) {
+            if (depythonify_python_object(object, &result) == -1) {
                 PyObjC_GIL_FORWARD_EXC();
-            } else {
-                PyObjC_GIL_RETURN([NSNull null]);
             }
         }
+        Py_DECREF(object);
 
     PyObjC_END_WITH_GIL
 
     return result;
 }
 
--(NSArray*)allObjects
+- (NSArray*)allObjects
 {
     NSMutableArray* array;
-    NSObject* cur;
+    NSObject*       cur;
 
     array = [NSMutableArray array];
     if (array == nil)
@@ -102,3 +99,5 @@
 }
 
 @end
+
+NS_ASSUME_NONNULL_END
