@@ -1,10 +1,11 @@
 #include "pyobjc.h"
 
+NS_ASSUME_NONNULL_BEGIN
+
 @implementation OC_PythonNumber
 
-+ (instancetype)numberWithPythonObject:(PyObject*)v
++ (instancetype _Nullable)numberWithPythonObject:(PyObject*)v
 {
-    OC_PythonNumber* res;
     if (PyLong_Check(v)) {
         unsigned long long lv = PyLong_AsUnsignedLongLong(v);
         if (PyErr_Occurred()) {
@@ -18,6 +19,16 @@
             /*
              * This should return an autoreleased value, but that causes
              * a crash in pyobjc-framework-Cocoa/test_regr.py
+             *
+             * XXX: Further investigate (in particular
+             *
+             *      This is caused by callers of this method assuming
+             *      that the result is an instance of an OC_ class that
+             *      will unregister itself in dealloc. That's not true
+             *      of instances like this.
+             *
+             *      I'm leaving the code like this for now and need
+             *      to perform more testing before fixing this.
              */
             return [[NSNumber alloc] initWithUnsignedLongLong:lv];
 #ifdef __clang__
@@ -26,12 +37,10 @@
         }
     }
 
-    res = [[self alloc] initWithPythonObject:v];
-    [res autorelease];
-    return res;
+    return [[[self alloc] initWithPythonObject:v] autorelease];
 }
 
-- (instancetype)initWithPythonObject:(PyObject*)v
+- (instancetype _Nullable)initWithPythonObject:(PyObject*)v
 {
     self = [super init];
     if (unlikely(self == nil))
@@ -126,7 +135,7 @@
 - (void)getValue:(void*)buffer
 {
     const char* encoded = [self objCType];
-    int r;
+    int         r;
     PyObjC_BEGIN_WITH_GIL
         r = depythonify_c_value(encoded, value, buffer);
         if (r == -1) {
@@ -159,7 +168,7 @@
 - (NSDecimal)decimalValue
 {
     NSDecimal result;
-    int r;
+    int       r;
 
     PyObjC_BEGIN_WITH_GIL
         r = PyObjC_number_to_decimal(value, &result);
@@ -271,7 +280,7 @@
                  * to undefined.
                  */
                 long long t = (long long)temp;
-                result = (unsigned long long)t;
+                result      = (unsigned long long)t;
             } else {
                 result = (unsigned long long)temp;
             }
@@ -292,7 +301,7 @@
 - (NSString*)stringValue
 {
     PyObject* repr;
-    NSObject* result = nil;
+    NSString* result = nil;
 
     PyObjC_BEGIN_WITH_GIL
         repr = PyObject_Repr(value);
@@ -300,14 +309,13 @@
             PyObjC_GIL_FORWARD_EXC();
         }
 
-        result = PyObjC_PythonToId(repr);
-        Py_DECREF(repr);
-        if (PyErr_Occurred()) {
+        if (depythonify_python_object(repr, &result) == -1) {
+            Py_DECREF(repr);
             PyObjC_GIL_FORWARD_EXC();
         }
-
+        Py_DECREF(repr);
     PyObjC_END_WITH_GIL
-    return (NSString*)result;
+    return result;
 }
 
 /* NSCoding support */
@@ -357,17 +365,17 @@
 - (void)pyobjcSetValue:(NSObject*)other
 {
     PyObjC_BEGIN_WITH_GIL
-        PyObject* v = PyObjC_IdToPython(other);
+        PyObject* v = id_to_python(other);
 
         SET_FIELD(value, v);
     PyObjC_END_WITH_GIL
 }
 
-- (id)initWithCoder:(NSCoder*)coder
+- (id _Nullable)initWithCoder:(NSCoder*)coder
 {
     if (PyObjC_Decoder != NULL) {
         PyObjC_BEGIN_WITH_GIL
-            PyObject* cdr = PyObjC_IdToPython(coder);
+            PyObject* cdr = id_to_python(coder);
             PyObject* setValue;
             PyObject* selfAsPython;
             PyObject* v;
@@ -377,9 +385,12 @@
             }
 
             selfAsPython = PyObjCObject_New(self, 0, YES);
+            if (selfAsPython == NULL) {
+                PyObjC_GIL_FORWARD_EXC();
+            }
             setValue = PyObject_GetAttrString(selfAsPython, "pyobjcSetValue_");
 
-            v = PyObject_CallFunction(PyObjC_Decoder, "OO", cdr, setValue);
+            v = PyObjC_CallDecoder(cdr, setValue);
             Py_DECREF(cdr);
             Py_DECREF(setValue);
             Py_DECREF(selfAsPython);
@@ -409,7 +420,7 @@
     return [self compare:(NSNumber*)other] == NSOrderedSame;
 }
 
-- (NSComparisonResult)compare:(NSNumber*)aNumber
+- (NSComparisonResult)compare:(NSNumber*)number
 {
     /* Rely on -[NSNumber compare:] when the other value
      * is a number and we're not a python int that doesn't
@@ -417,8 +428,8 @@
      *
      * In all other cases use Python's comparison semantics.
      */
-    if ([aNumber isKindOfClass:[NSNumber class]] &&
-        ![aNumber isKindOfClass:[OC_PythonNumber class]]) {
+    if ([number isKindOfClass:[NSNumber class]]
+        && ![number isKindOfClass:[OC_PythonNumber class]]) {
         int use_super = 0;
 
         PyObjC_BEGIN_WITH_GIL
@@ -435,13 +446,13 @@
         PyObjC_END_WITH_GIL;
 
         if (use_super) {
-            return [super compare:aNumber];
+            return [super compare:number];
         }
     }
 
     PyObjC_BEGIN_WITH_GIL
-        PyObject* other = PyObjC_IdToPython(aNumber);
-        int r, ok;
+        PyObject* other = id_to_python(number);
+        int       r, ok;
 
         if (other == NULL) {
             PyObjC_GIL_FORWARD_EXC();
@@ -465,11 +476,11 @@
 }
 
 #define COMPARE_METHOD(NAME, OPERATOR)                                                   \
-    -(BOOL)NAME : (NSObject*)aNumber                                                     \
+    -(BOOL)NAME : (NSObject* _Nullable)number                                            \
     {                                                                                    \
         PyObjC_BEGIN_WITH_GIL                                                            \
-            PyObject* other = PyObjC_IdToPython(aNumber);                                \
-            int r;                                                                       \
+            PyObject* other = id_to_python(number);                                      \
+            int       r;                                                                 \
             if (other == NULL) {                                                         \
                 PyObjC_GIL_FORWARD_EXC();                                                \
             }                                                                            \
@@ -496,37 +507,11 @@ COMPARE_METHOD(isGreaterThanOrEqualTo, Py_GE)
 COMPARE_METHOD(isLessThan, Py_LT)
 COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
 
-- (BOOL)isEqualToNumber:(NSNumber*)aNumber
+- (BOOL)isEqualToNumber:(NSNumber*)number
 {
-    return [self isEqualTo:aNumber];
+    return [self isEqualTo:number];
 }
 
-#if 0
--(NSObject*)replacementObjectForArchiver:(NSArchiver*)archiver
-{
-    (void)archiver;
-    return (NSObject*)self;
-}
-
--(NSObject*)replacementObjectForKeyedArchiver:(NSKeyedArchiver*)archiver
-{
-    (void)archiver;
-    return (NSObject*)self;
-}
-
--(NSObject*)replacementObjectForCoder:(NSCoder*)archiver
-{
-    (void)archiver;
-    return (NSObject*)self;
-}
-
--(NSObject*)replacementObjectForPortCoder:(NSPortCoder*)archiver
-{
-    (void)archiver;
-    return (NSObject*)self;
-}
-#endif
-
 - (Class)classForArchiver
 {
     PyObjC_BEGIN_WITH_GIL
@@ -550,7 +535,7 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
     PyObjC_END_WITH_GIL
 }
 
-- (Class)classForKeyedArchiver
+- (Class _Nullable)classForKeyedArchiver
 {
     return [self classForArchiver];
 }
@@ -562,14 +547,18 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
 
 - (id)copy
 {
-    return [self copyWithZone:0];
+    return [self copyWithZone:NULL];
 }
 
-- (id)copyWithZone:(NSZone*)zone
+- (id)copyWithZone:(NSZone* _Nullable)zone __attribute__((__unused__))
 {
-    (void)zone;
+    /* XXX: This is ok if value is a python builtin
+     *      but not for arbitrary python objects.
+     */
     [self retain];
     return self;
 }
 
 @end
+
+NS_ASSUME_NONNULL_END
