@@ -1,17 +1,15 @@
 #include "pyobjc.h"
-#import "OC_PythonUnicode.h"
+
+NS_ASSUME_NONNULL_BEGIN
 
 @implementation OC_PythonUnicode
 
-+ (instancetype)unicodeWithPythonObject:(PyObject*)v
++ (instancetype _Nullable)unicodeWithPythonObject:(PyObject*)v
 {
-    OC_PythonUnicode* res;
-
-    res = [[self alloc] initWithPythonObject:v];
-    return [res autorelease];
+    return [[[self alloc] initWithPythonObject:v] autorelease];
 }
 
-- (id)initWithPythonObject:(PyObject*)v
+- (instancetype _Nullable)initWithPythonObject:(PyObject*)v
 {
     self = [super init];
     if (unlikely(self == nil))
@@ -23,6 +21,7 @@
 
 - (PyObject*)__pyobjc_PythonObject__
 {
+    /* XXX: Can value ever be NULL? */
     if (value == NULL) {
         Py_INCREF(Py_None);
         return Py_None;
@@ -88,8 +87,8 @@
         Py_CLEAR(value);
 
 #ifdef PyObjC_STR_CACHE_IMP
-        imp_length = 0xDEADBEEF;
-        imp_charAtIndex = 0xDEADBEEF;
+        imp_length        = 0xDEADBEEF;
+        imp_charAtIndex   = 0xDEADBEEF;
         imp_getCharacters = 0xDEADBEEF;
 #endif /* PyObjC_STR_CACHE_IMP */
 
@@ -98,7 +97,7 @@
     [super dealloc];
 }
 
-- (id)__realObject__
+- (id _Nullable)__realObject__
 {
 #ifdef Py_DEBUG
     if (!PyUnicode_IS_READY(value)) {
@@ -187,11 +186,13 @@
                   freeWhenDone:(BOOL)flag
 {
     int byteorder = 0;
+    /* XXX: Call super? */
     PyObjC_BEGIN_WITH_GIL
         /* Decode as a UTF-16 string in native byteorder */
         value =
             PyUnicode_DecodeUTF16((const char*)characters, length * 2, NULL, &byteorder);
         if (value == NULL) {
+            /* XXX: Maybe just return nil? */
             PyObjC_GIL_FORWARD_EXC();
         }
 
@@ -202,12 +203,12 @@
     return self;
 }
 
-- (id)initWithBytes:(const void*)bytes
-             length:(NSUInteger)length
-           encoding:(NSStringEncoding)encoding
+- (id _Nullable)initWithBytes:(const void*)bytes
+                       length:(NSUInteger)length
+                     encoding:(NSStringEncoding)encoding
 {
     char* py_encoding = NULL;
-    int byteorder = 0;
+    int   byteorder   = 0;
 
     /*
      * Call the super initializer first.
@@ -276,7 +277,7 @@
      * Python 3.3 or later anyway.
      */
 
-    NSString* tmpval = [[NSString alloc] initWithBytes:bytes
+    NSString*  tmpval    = [[NSString alloc] initWithBytes:bytes
                                                 length:length
                                               encoding:encoding];
     Py_ssize_t charcount = [tmpval length];
@@ -314,13 +315,13 @@
 - (void)pyobjcSetValue:(NSObject*)other
 {
     PyObjC_BEGIN_WITH_GIL
-        PyObject* v = PyObjC_IdToPython(other);
+        PyObject* v = id_to_python(other);
 
         SET_FIELD(value, v);
     PyObjC_END_WITH_GIL
 }
 
-- (id)initWithCoder:(NSCoder*)coder
+- (id _Nullable)initWithCoder:(NSCoder*)coder
 {
     int ver;
     if ([coder allowsKeyedCoding]) {
@@ -338,7 +339,7 @@
 
         if (PyObjC_Decoder != NULL) {
             PyObjC_BEGIN_WITH_GIL
-                PyObject* cdr = PyObjC_IdToPython(coder);
+                PyObject* cdr = id_to_python(coder);
                 PyObject* setValue;
                 PyObject* selfAsPython;
                 PyObject* v;
@@ -348,9 +349,12 @@
                 }
 
                 selfAsPython = PyObjCObject_New(self, 0, YES);
+                if (selfAsPython == NULL) {
+                    PyObjC_GIL_FORWARD_EXC();
+                }
                 setValue = PyObject_GetAttrString(selfAsPython, "pyobjcSetValue_");
 
-                v = PyObject_CallFunction(PyObjC_Decoder, "OO", cdr, setValue);
+                v = PyObjC_CallDecoder(cdr, setValue);
                 Py_DECREF(cdr);
                 Py_DECREF(setValue);
                 Py_DECREF(selfAsPython);
@@ -403,27 +407,27 @@
     }
 }
 
-- (NSObject*)replacementObjectForArchiver:(NSArchiver*)archiver
+- (NSObject* _Nullable)replacementObjectForArchiver:(NSArchiver*)archiver
+    __attribute__((__unused__))
 {
-    (void)(archiver);
     return self;
 }
 
-- (NSObject*)replacementObjectForKeyedArchiver:(NSKeyedArchiver*)archiver
+- (NSObject* _Nullable)replacementObjectForKeyedArchiver:(NSKeyedArchiver*)archiver
+    __attribute__((__unused__))
 {
-    (void)(archiver);
     return self;
 }
 
-- (NSObject*)replacementObjectForCoder:(NSCoder*)archiver
+- (NSObject* _Nullable)replacementObjectForCoder:(NSCoder*)archiver
+    __attribute__((__unused__))
 {
-    (void)(archiver);
     return self;
 }
 
-- (NSObject*)replacementObjectForPortCoder:(NSPortCoder*)archiver
+- (NSObject* _Nullable)replacementObjectForPortCoder:(NSPortCoder*)archiver
+    __attribute__((__unused__))
 {
-    (void)(archiver);
     return self;
 }
 
@@ -446,7 +450,7 @@
     return result;
 }
 
-- (Class)classForKeyedArchiver
+- (Class _Nullable)classForKeyedArchiver
 {
     return [OC_PythonUnicode class];
 }
@@ -460,3 +464,5 @@
 }
 
 @end /* implementation OC_PythonUnicode */
+
+NS_ASSUME_NONNULL_END
