@@ -12,8 +12,9 @@
 
 #include <dlfcn.h>
 
-static CFBundleRef
-CreateCFBundleFromNSBundle(NSBundle* bundle)
+NS_ASSUME_NONNULL_BEGIN
+
+static CFBundleRef _Nullable CreateCFBundleFromNSBundle(NSBundle* bundle)
 {
     CFURLRef bundleURL;
 
@@ -21,9 +22,9 @@ CreateCFBundleFromNSBundle(NSBundle* bundle)
     return CFBundleCreate(kCFAllocatorDefault, bundleURL);
 }
 
-PyObject*
-PyObjC_loadSpecialVar(PyObject* self __attribute__((__unused__)), PyObject* args,
-                      PyObject* kwds)
+PyObject* _Nullable PyObjC_loadSpecialVar(PyObject* self __attribute__((__unused__)),
+                                          PyObject* _Nullable args,
+                                          PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"bundle", "module_globals", "typeid",
                                "name",   "skip_undefined", NULL};
@@ -32,9 +33,9 @@ PyObjC_loadSpecialVar(PyObject* self __attribute__((__unused__)), PyObject* args
     NSString* name;
     PyObject* module_globals;
     Py_ssize_t typeid;
-    Py_ssize_t skip_undefined = 1;
+    Py_ssize_t  skip_undefined = 1;
     CFBundleRef cfBundle;
-    void* value;
+    void*       value;
 
     if (!PyArg_ParseTupleAndKeywords(
             args, kwds, "O&OiO&|i", keywords, PyObjCObject_Convert, &bundle,
@@ -73,7 +74,7 @@ PyObjC_loadSpecialVar(PyObject* self __attribute__((__unused__)), PyObject* args
         }
 
     } else {
-        PyObject* py_val = PyObjCCF_NewSpecial2(typeid, value);
+        PyObject* py_val = PyObjCCF_NewSpecialFromTypeID(typeid, value);
         if (py_val == NULL) {
             return NULL;
         }
@@ -89,19 +90,19 @@ PyObjC_loadSpecialVar(PyObject* self __attribute__((__unused__)), PyObject* args
     return Py_None;
 }
 
-PyObject*
-PyObjC_loadBundleVariables(PyObject* self __attribute__((__unused__)), PyObject* args,
-                           PyObject* kwds)
+PyObject* _Nullable PyObjC_loadBundleVariables(PyObject* self __attribute__((__unused__)),
+                                               PyObject* _Nullable args,
+                                               PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"bundle", "module_globals", "variableInfo",
                                "skip_undefined", NULL};
-    NSBundle* bundle;
-    PyObject* module_globals;
-    PyObject* variableInfo;
-    Py_ssize_t skip_undefined = 1;
-    CFBundleRef cfBundle;
-    PyObject* seq;
-    Py_ssize_t i, len;
+    NSBundle*    bundle;
+    PyObject*    module_globals;
+    PyObject*    variableInfo;
+    Py_ssize_t   skip_undefined = 1;
+    CFBundleRef  cfBundle;
+    PyObject*    seq;
+    Py_ssize_t   i, len;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&OO|i", keywords, PyObjCObject_Convert,
                                      &bundle, &module_globals, &variableInfo,
@@ -135,8 +136,8 @@ PyObjC_loadBundleVariables(PyObject* self __attribute__((__unused__)), PyObject*
     len = PySequence_Fast_GET_SIZE(seq);
     for (i = 0; i < len; i++) {
         PyObject* item = PySequence_Fast_GET_ITEM(seq, i);
-        void* value;
-        char* signature;
+        void*     value;
+        char*     signature;
         PyObject* py_name;
         NSString* name;
 
@@ -154,8 +155,7 @@ PyObjC_loadBundleVariables(PyObject* self __attribute__((__unused__)), PyObject*
             return NULL;
         }
 
-        name = PyObjC_PythonToId(py_name);
-        if (name == NULL && PyErr_Occurred()) {
+        if (depythonify_python_object(py_name, &name) == -1) {
             return NULL;
         }
 
@@ -196,19 +196,19 @@ PyObjC_loadBundleVariables(PyObject* self __attribute__((__unused__)), PyObject*
     return Py_None;
 }
 
-PyObject*
-PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)), PyObject* args,
-                           PyObject* kwds)
+PyObject* _Nullable PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)),
+                                               PyObject* _Nullable args,
+                                               PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"bundle", "module_globals", "functionInfo",
                                "skip_undefined", NULL};
-    NSBundle* bundle;
-    PyObject* module_globals;
-    PyObject* functionInfo;
-    int skip_undefined = 1;
-    CFBundleRef cfBundle;
-    PyObject* seq;
-    Py_ssize_t i, len;
+    NSBundle*    bundle;
+    PyObject*    module_globals;
+    PyObject*    functionInfo;
+    int          skip_undefined = 1;
+    CFBundleRef  cfBundle;
+    PyObject*    seq;
+    Py_ssize_t   i, len;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&OO|i", keywords, PyObjCObject_Convert,
                                      &bundle, &module_globals, &functionInfo,
@@ -247,10 +247,10 @@ PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)), PyObject*
     len = PySequence_Fast_GET_SIZE(seq);
     for (i = 0; i < len; i++) {
         PyObject* item = PySequence_Fast_GET_ITEM(seq, i);
-        void* value;
-        char* signature;
+        void*     value;
+        char*     signature;
         NSString* name;
-        char* c_name;
+        char*     c_name;
         PyObject* doc;
         PyObject* meta = NULL;
 
@@ -288,7 +288,7 @@ PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)), PyObject*
 
         if (value == NULL) {
             if (!skip_undefined) {
-                PyErr_SetString(PyObjCExc_Error, "cannot find a function");
+                PyErr_Format(PyObjCExc_Error, "cannot find a function: %R", item);
                 Py_DECREF(seq);
                 return NULL;
             }
@@ -300,7 +300,7 @@ PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)), PyObject*
             if (cfBundle == NULL) {
                 py_name = PyUnicode_FromString(c_name);
             } else {
-                py_name = PyObjC_IdToPython(name);
+                py_name = id_to_python(name);
             }
 
             py_val = PyObjCFunc_New(py_name, value, signature, doc, meta);
@@ -327,12 +327,11 @@ PyObjC_loadBundleFunctions(PyObject* self __attribute__((__unused__)), PyObject*
 
 typedef void (*function)(void);
 struct functionlist {
-    char* name;
+    char*    name;
     function func;
 };
 
-static function
-find_function(struct functionlist* functions, PyObject* name)
+static function _Nullable find_function(struct functionlist* functions, PyObject* name)
 {
     while (functions->name != NULL) {
         if (PyObjC_is_ascii_string(name, functions->name)) {
@@ -343,18 +342,18 @@ find_function(struct functionlist* functions, PyObject* name)
     return NULL;
 }
 
-PyObject*
-PyObjC_loadFunctionList(PyObject* self __attribute__((__unused__)), PyObject* args,
-                        PyObject* kwds)
+PyObject* _Nullable PyObjC_loadFunctionList(PyObject* self __attribute__((__unused__)),
+                                            PyObject* _Nullable args,
+                                            PyObject* _Nullable kwds)
 {
-    static char* keywords[] = {"function_list", "module_globals", "functionInfo",
+    static char*         keywords[] = {"function_list", "module_globals", "functionInfo",
                                "skip_undefined", NULL};
-    PyObject* pyFunctionsList;
-    PyObject* module_globals;
-    PyObject* functionInfo;
-    int skip_undefined = 1;
-    PyObject* seq;
-    Py_ssize_t i, len;
+    PyObject*            pyFunctionsList;
+    PyObject*            module_globals;
+    PyObject*            functionInfo;
+    int                  skip_undefined = 1;
+    PyObject*            seq;
+    Py_ssize_t           i, len;
     struct functionlist* function_list;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "OOO|i", keywords, &pyFunctionsList,
@@ -381,8 +380,8 @@ PyObjC_loadFunctionList(PyObject* self __attribute__((__unused__)), PyObject* ar
     len = PySequence_Fast_GET_SIZE(seq);
     for (i = 0; i < len; i++) {
         PyObject* item = PySequence_Fast_GET_ITEM(seq, i);
-        void* value;
-        char* signature;
+        void*     value;
+        char*     signature;
         PyObject* name;
         PyObject* doc;
         PyObject* meta = NULL;
@@ -429,3 +428,5 @@ PyObjC_loadFunctionList(PyObject* self __attribute__((__unused__)), PyObject* ar
     Py_INCREF(Py_None);
     return Py_None;
 }
+
+NS_ASSUME_NONNULL_END
