@@ -6,6 +6,12 @@
 
 #import <Foundation/NSInvocation.h>
 
+NS_ASSUME_NONNULL_BEGIN
+
+/* XXX: See comment at definition */
+static PyObject* _Nullable PyObjC_CallPython(id self, SEL selector, PyObject* arglist,
+                                             BOOL* isAlloc, BOOL* isCFAlloc);
+
 /* Special methods for Python subclasses of Objective-C objects */
 static void object_method_dealloc(ffi_cif* cif, void* retval, void** args, void* userarg);
 static void object_method_respondsToSelector(ffi_cif* cif, void* retval, void** args,
@@ -157,19 +163,16 @@ do_slots(PyObject* super_class, PyObject* clsdict)
         slot_value = PySequence_Fast_GET_ITEM(slots, i);
 
         if (PyUnicode_Check(slot_value)) {
-            PyObject* bytes = PyUnicode_AsEncodedString(slot_value, NULL, NULL);
-
-            if (bytes == NULL) {
+            const char* slot_name = PyUnicode_AsUTF8(slot_value);
+            if (slot_name == NULL) {
                 return -1;
             }
 
-            var = (PyObjCInstanceVariable*)PyObjCInstanceVariable_New(
-                PyBytes_AsString(bytes));
-            Py_DECREF(bytes);
+            var = (PyObjCInstanceVariable*)PyObjCInstanceVariable_New(slot_name);
 
         } else {
-            PyErr_Format(PyExc_TypeError,
-                         "__slots__ entry %" PY_FORMAT_SIZE_T "d is not a string", i);
+            PyErr_Format(PyExc_TypeError, "__slots__ entry %R is not a string",
+                         slot_value);
             Py_DECREF(slots);
             return -1;
         }
@@ -211,8 +214,7 @@ do_slots(PyObject* super_class, PyObject* clsdict)
  * methods.
  */
 
-static Class
-build_intermediate_class(Class base_class, char* name)
+static Class _Nullable build_intermediate_class(Class base_class, char* name)
 {
     Class intermediate_class = nil;
 
@@ -232,9 +234,8 @@ build_intermediate_class(Class base_class, char* name)
                 continue;
             }
         }
-
         PyObjCMethodSignature* methinfo =
-            PyObjCMethodSignature_FromSignature(cur->typestr, NO);
+            PyObjCMethodSignature_WithMetaData(cur->typestr, NULL, NO);
         if (methinfo == NULL)
             goto error_cleanup;
         IMP closure = PyObjCFFI_MakeClosure(methinfo, cur->func, base_class);
@@ -242,7 +243,7 @@ build_intermediate_class(Class base_class, char* name)
         if (closure == NULL)
             goto error_cleanup;
 
-        preclass_addMethod(intermediate_class, cur->selector, (IMP)closure, cur->typestr);
+        class_addMethod(intermediate_class, cur->selector, (IMP)closure, cur->typestr);
         closure = NULL;
     }
 
@@ -254,7 +255,7 @@ error_cleanup:
         objc_disposeClassPair(intermediate_class);
     }
 
-    return NULL;
+    return Nil;
 }
 
 /*
@@ -279,6 +280,7 @@ error_cleanup:
 
 /* PyObjC uses a number of typecode descriptors that aren't available in
  * the objc runtime. Remove these from the type string (inline).
+ *
  */
 static void
 tc2tc(char* buf)
@@ -302,6 +304,13 @@ tc2tc(char* buf)
 exit:
     switch (*buf) {
     case _C_NSBOOL:
+#ifdef __arm64__
+        *buf = _C_BOOL;
+#else
+        *buf = _C_CHR;
+#endif
+        break;
+
     case _C_CHAR_AS_INT:
     case _C_CHAR_AS_TEXT:
         *buf = _C_CHR;
@@ -324,7 +333,13 @@ exit:
                 buf++;
             }
             tc2tc(buf);
-            buf = (char*)PyObjCRT_SkipTypeSpec(buf);
+            char* new_buf = (char*)PyObjCRT_SkipTypeSpec(buf);
+            if (new_buf == NULL) {
+                /* Should not happen! */
+                PyObjCErr_InternalError();
+                return;
+            }
+            buf = new_buf;
         }
         break;
 
@@ -341,7 +356,13 @@ exit:
                 buf++;
             }
             tc2tc(buf);
-            buf = (char*)PyObjCRT_SkipTypeSpec(buf);
+            char* new_buf = (char*)PyObjCRT_SkipTypeSpec(buf);
+            if (new_buf == NULL) {
+                /* Should not happen! */
+                PyObjCErr_InternalError();
+                return;
+            }
+            buf = new_buf;
         }
         break;
 
@@ -353,14 +374,18 @@ exit:
     }
 }
 
+/* XXX: This function and tc2tc should be in objc_support.m  */
 int
 PyObjC_RemoveInternalTypeCodes(char* buf)
 {
-    while (buf && *buf) {
-        tc2tc(buf);
-        buf = (char*)PyObjCRT_SkipTypeSpec(buf);
-        if (buf == NULL)
+    char* _Nullable cur = buf;
+
+    while (*cur) {
+        tc2tc(cur);
+        cur = (char*)PyObjCRT_SkipTypeSpec(cur);
+        if (cur == NULL) {
             return -1;
+        }
     }
     return 0;
 }
@@ -384,10 +409,10 @@ need_intermediate(PyObject* class_dict)
     return NO;
 }
 
-Class
-PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
-                       PyObject* class_dict, PyObject* meta_dict,
-                       PyObject* hiddenSelectors, PyObject* hiddenClassSelectors)
+Class _Nullable PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
+                                       PyObject* class_dict, PyObject* meta_dict,
+                                       PyObject* hiddenSelectors,
+                                       PyObject* hiddenClassSelectors)
 {
     PyObject*  seq;
     PyObject*  key_list = NULL;
@@ -395,76 +420,64 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     PyObject*  value    = NULL;
     Py_ssize_t i;
     Py_ssize_t key_count;
-    Py_ssize_t protocol_count   = 0;
-    int        first_python_gen = 0;
-    Class      new_class        = NULL;
-    Class      new_meta_class   = NULL;
-    Class      cur_class;
+    Py_ssize_t protocol_count     = 0;
+    int        first_python_gen   = 0;
+    Class      new_class          = NULL;
+    Class      new_meta_class     = NULL;
     PyObject*  py_superclass      = NULL;
     int        have_intermediate  = 0;
     PyObject*  instance_variables = NULL;
     PyObject*  instance_methods   = NULL;
     PyObject*  class_methods      = NULL;
 
+    PyObjC_Assert(super_class != Nil, Nil);
+
     if (!PyList_Check(protocols)) {
         PyErr_Format(PyObjCExc_InternalError,
                      "protocol list not a python 'list' but '%s'",
                      Py_TYPE(protocols)->tp_name);
-        goto error_cleanup;
+        return Nil;
     }
 
     if (!PyDict_Check(class_dict)) {
         PyErr_Format(PyObjCExc_InternalError, "class dict not a python 'dict', but '%s'",
                      Py_TYPE(class_dict)->tp_name);
-        goto error_cleanup;
+        return Nil;
     }
 
-    if (super_class == NULL) {
-        PyErr_SetString(PyObjCExc_InternalError, "must have super_class");
-        goto error_cleanup;
-    }
-
-    if ((cur_class = objc_lookUpClass(name)) != NULL) {
-        /*
-         * NOTE: we used to allow redefinition of a class if the
-         * redefinition is in the same module. This code was removed
-         * because that functionality isn't possible with the ObjC 2.0
-         * runtime API.
-         */
-
+    if (objc_lookUpClass(name) != NULL) {
         PyErr_Format(PyObjCExc_Error, "%s is overriding existing Objective-C class",
                      name);
-        goto error_cleanup;
+        return Nil;
     }
 
     if (strspn(name, IDENT_CHARS) != strlen(name)) {
         PyErr_Format(PyObjCExc_Error, "'%s' not a valid name", name);
-        goto error_cleanup;
+        return Nil;
     }
 
-    PyDict_SetItemString(class_dict, "__objc_python_subclass__", Py_True);
+    if (PyDict_SetItemString(class_dict, "__objc_python_subclass__", Py_True) == -1) {
+        return Nil;
+    }
 
     py_superclass = PyObjCClass_New(super_class);
     if (py_superclass == NULL) {
-        return NULL;
+        return Nil;
     }
 
     instance_variables = PySet_New(NULL);
     if (instance_variables == NULL) {
-        return NULL;
+        goto error_cleanup;
     }
 
     instance_methods = PySet_New(NULL);
     if (instance_methods == NULL) {
-        Py_DECREF(instance_variables);
-        return NULL;
+        goto error_cleanup;
     }
 
     class_methods = PySet_New(NULL);
     if (class_methods == NULL) {
-        Py_DECREF(instance_variables);
-        Py_DECREF(instance_methods);
-        return NULL;
+        goto error_cleanup;
     }
 
     /* We must override copyWithZone: for python classes because the
@@ -483,22 +496,31 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
         && need_intermediate(class_dict)) {
         Class intermediate_class;
         char  buf[1024];
+        int   r;
 
         have_intermediate = 1;
 
-        snprintf(buf, 1024, "_PyObjCCopying_%s", class_getName(super_class));
+        r = snprintf(buf, 1024, "_PyObjCCopying_%s", class_getName(super_class));
+        if (r < 0 || r >= (int)sizeof(buf)) {
+            /* Formatting the name failed */
+            PyErr_SetString(PyObjCExc_InternalError,
+                            "Cannot calculate name of intermediate class");
+            goto error_cleanup;
+        }
+
         intermediate_class = objc_lookUpClass(buf);
         if (intermediate_class == NULL) {
             intermediate_class = build_intermediate_class(super_class, buf);
-            if (intermediate_class == NULL)
+            if (intermediate_class == Nil)
                 goto error_cleanup;
         }
         Py_DECREF(py_superclass);
 
         super_class   = intermediate_class;
         py_superclass = PyObjCClass_New(super_class);
-        if (py_superclass == NULL)
-            return NULL;
+        if (py_superclass == Nil) {
+            goto error_cleanup;
+        }
     }
 
     if (do_slots(py_superclass, class_dict) < 0) {
@@ -522,39 +544,40 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     }
 
     key_count = PyList_Size(key_list);
-    if (PyErr_Occurred()) {
-        Py_DECREF(key_list);
+    if (key_count == -1) {
         goto error_cleanup;
     }
 
     /* Allocate the class as soon as possible, for new selector objects */
     new_class = objc_allocateClassPair(super_class, name, 0);
-    if (new_class == 0) {
+    if (new_class == Nil) {
         PyErr_Format(PyObjCExc_Error, "Cannot allocateClassPair for %s", name);
         goto error_cleanup;
     }
 
-    new_meta_class = object_getClass(new_class);
+    /* Class is only Nil if new_class is nil */
+    new_meta_class = (Class _Nonnull)object_getClass(new_class);
 
     /* 0th round: protocols */
     protocol_count = PyList_Size(protocols);
-    if (protocol_count > 0) {
-        for (i = 0; i < protocol_count; i++) {
-            PyObject* wrapped_protocol;
-            wrapped_protocol = PyList_GET_ITEM(protocols, i);
-            if (!PyObjCFormalProtocol_Check(wrapped_protocol)) {
-                continue;
-            }
-
-            if (!preclass_addProtocol(
-                    new_class, PyObjCFormalProtocol_GetProtocol(wrapped_protocol))) {
-                goto error_cleanup;
-            }
-        }
-    }
-    if (PyErr_Occurred()) {
+    if (protocol_count == -1) {
         goto error_cleanup;
     }
+    for (i = 0; i < protocol_count; i++) {
+        PyObject* wrapped_protocol;
+        wrapped_protocol = PyList_GET_ITEM(protocols, i);
+        if (!PyObjCFormalProtocol_Check(wrapped_protocol)) {
+            continue;
+        }
+
+        /* PyObjCFormalProtocol_GetProtocol() is nonnull because we've already type
+         * checked */
+        if (!class_addProtocol(
+                new_class,
+                (Protocol* _Nonnull)PyObjCFormalProtocol_GetProtocol(wrapped_protocol))) {
+            goto error_cleanup;
+        }
+    }
 
     /* First step: call class setup hooks of entries in the class dict */
     for (i = 0; i < key_count; i++) {
@@ -572,14 +595,17 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
         /*
          * Check if the value has a class-setup hook, and if it does
          * call said hook.
+         * XXX: Create utility function for "call optional method"
+         *      or use VectorcallMethod and handle AttributeError.
          */
         PyObject* m = PyObject_GetAttrString(value, "__pyobjc_class_setup__");
         if (m == NULL) {
             PyErr_Clear();
 
         } else {
-            PyObject* rv = PyObject_CallFunction(m, "OOOO", key, class_dict,
-                                                 instance_methods, class_methods);
+            PyObject* args[5] = {NULL, key, class_dict, instance_methods, class_methods};
+            PyObject* rv      = PyObject_Vectorcall(m, args + 1,
+                                                    4 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
             Py_DECREF(m);
             if (rv == NULL) {
                 goto error_cleanup;
@@ -591,7 +617,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     /* The class_setup_hooks may have changed the class dictionary, hence
      * we need to recalculate the key list.
      */
-    Py_XDECREF(key_list);
+    Py_CLEAR(key_list);
 
     key_list = PyDict_Keys(class_dict);
     if (key_list == NULL) {
@@ -599,7 +625,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     }
 
     key_count = PyList_Size(key_list);
-    if (PyErr_Occurred()) {
+    if (key_count == -1) {
         Py_DECREF(key_list);
         goto error_cleanup;
     }
@@ -607,7 +633,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     /* Step 2b: Collect methods and instance variables in the class dict
      *          into the 3 sets.
      *
-     * FIXME: This work should be done by the class setup hook instead.
+     * XXX: This work should be done by the class setup hook instead.
      */
     for (i = 0; i < key_count; i++) {
         key = PyList_GET_ITEM(key_list, i);
@@ -639,9 +665,14 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
              * and instance method for the same selector in one
              * generation.
              */
-            char      buf[1024];
-            PyObject* pyname = PyUnicode_FromString(PyObjC_SELToPythonName(
-                PyObjCSelector_GetSelector(value), buf, sizeof(buf)));
+            char        buf[1024];
+            const char* py_meth_name = PyObjC_SELToPythonName(
+                PyObjCSelector_GetSelector(value), buf, sizeof(buf));
+            if (py_meth_name == NULL) {
+                goto error_cleanup;
+            }
+
+            PyObject* pyname = PyUnicode_FromString(py_meth_name);
 
             if (pyname == NULL) {
                 goto error_cleanup;
@@ -794,8 +825,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     }
 
     /* Keylist is not needed anymore */
-    Py_DECREF(key_list);
-    key_list = NULL;
+    Py_CLEAR(key_list);
 
     /* Step 3: Check instance variables */
 
@@ -811,11 +841,15 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
         value = PySequence_Fast_GET_ITEM(instance_variables, i);
 
         if (!PyObjCInstanceVariable_Check(value)) {
+            /* XXX: Why is this needed? */
             continue;
         }
 
         /* Our only check for now is that instance variable names must be unique */
         /* XXX: Is this really necessary? */
+        /* XXX: This doesn't check that the new class has unique instance variables!
+         *      Add test before fixing this
+         */
         if (class_getInstanceVariable(super_class, PyObjCInstanceVariable_GetName(value))
             != NULL) {
             PyErr_Format(PyObjCExc_Error,
@@ -943,6 +977,9 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
          */
 
         if (!have_intermediate) {
+            /* XXX: This loop is also in the function that builds the
+             *      intermediate class, refactor this.
+             */
             struct method_info* cur;
             for (cur = gMethods; cur->method_name != NULL; cur++) {
                 if (unlikely(cur->selector == NULL)) {
@@ -955,7 +992,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
                 }
 
                 PyObjCMethodSignature* methinfo =
-                    PyObjCMethodSignature_FromSignature(cur->typestr, NO);
+                    PyObjCMethodSignature_WithMetaData(cur->typestr, NULL, NO);
                 if (methinfo == NULL)
                     goto error_cleanup;
 
@@ -964,7 +1001,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
                 if (closure == NULL)
                     goto error_cleanup;
 
-                preclass_addMethod(new_class, cur->selector, closure, cur->typestr);
+                class_addMethod(new_class, cur->selector, closure, cur->typestr);
                 PyObject* sel =
                     PyObjCSelector_NewNative(new_class, cur->selector, cur->typestr, 0);
                 if (sel == NULL)
@@ -996,6 +1033,9 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
             size = sizeof(PyObject*);
         } else {
             type = PyObjCInstanceVariable_GetType(value);
+            if (type == NULL) {
+                goto error_cleanup;
+            }
             size = PyObjCRT_SizeOfType(type);
         }
         align = PyObjCRT_AlignOfType(type);
@@ -1005,8 +1045,8 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
             goto error_cleanup;
         }
 
-        if (!preclass_addIvar(new_class, PyObjCInstanceVariable_GetName(value), size,
-                              align, type)) {
+        if (!class_addIvar(new_class, PyObjCInstanceVariable_GetName(value), size, align,
+                           type)) {
 
             goto error_cleanup;
         }
@@ -1026,13 +1066,21 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
 
         meth = class_getInstanceMethod(super_class, PyObjCSelector_GetSelector(value));
         if (meth) {
-            is_override = 1;
-            if (!PyObjCRT_SignaturesEqual(method_getTypeEncoding(meth),
+            is_override               = 1;
+            const char* meth_encoding = method_getTypeEncoding(meth);
+            if (meth_encoding == NULL) {
+                PyErr_Format(PyObjCExc_BadPrototypeError,
+                             "%R cannot determine super_class type encoding", value);
+                goto error_cleanup;
+            }
+
+            if (!PyObjCRT_SignaturesEqual(meth_encoding,
                                           PyObjCSelector_GetNativeSignature(value))) {
 
-                PyErr_Format(PyObjCExc_BadPrototypeError,
-                             "%R has signature that is not compatible with super-class",
-                             value);
+                PyErr_Format(
+                    PyObjCExc_BadPrototypeError,
+                    "%R has signature that is not compatible with super-class: %s != %s",
+                    value, meth_encoding, PyObjCSelector_GetNativeSignature(value));
                 goto error_cleanup;
             }
         }
@@ -1048,8 +1096,8 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
             goto error_cleanup;
         }
 
-        if (!preclass_addMethod(new_class, PyObjCSelector_GetSelector(value), imp,
-                                PyObjCSelector_GetNativeSignature(value))) {
+        if (!class_addMethod(new_class, PyObjCSelector_GetSelector(value), imp,
+                             PyObjCSelector_GetNativeSignature(value))) {
             goto error_cleanup;
         }
     }
@@ -1068,14 +1116,21 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
 
         meth = class_getClassMethod(super_class, PyObjCSelector_GetSelector(value));
         if (meth) {
-            is_override = 1;
+            is_override               = 1;
+            const char* meth_encoding = method_getTypeEncoding(meth);
+            if (meth_encoding == NULL) {
+                PyErr_Format(PyObjCExc_BadPrototypeError,
+                             "%R: Cannot determine superclass type encoding", value);
+                goto error_cleanup;
+            }
 
-            if (!PyObjCRT_SignaturesEqual(method_getTypeEncoding(meth),
+            if (!PyObjCRT_SignaturesEqual(meth_encoding,
                                           PyObjCSelector_GetNativeSignature(value))) {
 
-                PyErr_Format(PyObjCExc_BadPrototypeError,
-                             "%R has signature that is not compatible with super-class",
-                             value);
+                PyErr_Format(
+                    PyObjCExc_BadPrototypeError,
+                    "%R has signature that is not compatible with super-class: %s != %s",
+                    value, meth_encoding, PyObjCSelector_GetNativeSignature(value));
                 goto error_cleanup;
             }
         }
@@ -1090,8 +1145,8 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
             goto error_cleanup;
         }
 
-        if (!preclass_addMethod(new_meta_class, PyObjCSelector_GetSelector(value), imp,
-                                PyObjCSelector_GetNativeSignature(value))) {
+        if (!class_addMethod(new_meta_class, PyObjCSelector_GetSelector(value), imp,
+                             PyObjCSelector_GetNativeSignature(value))) {
             goto error_cleanup;
         }
     }
@@ -1108,7 +1163,7 @@ PyObjCClass_BuildClass(Class super_class, PyObject* protocols, char* name,
     /*
      * NOTE: Class is not registered yet, we do that as lately as possible
      * because it is impossible to remove the registration from the
-     * objective-C runtime (at least on MacOS X).
+     * objective-C runtime.
      */
     return new_class;
 
@@ -1166,6 +1221,7 @@ free_ivars(id self, PyObject* cls)
             break;
         }
 
+        /* XXX: Why does this not access the dict slot directly? */
         clsDict = PyObject_GetAttrString(cls, "__dict__");
         if (clsDict == NULL) {
             PyErr_Clear();
@@ -1177,7 +1233,9 @@ free_ivars(id self, PyObject* cls)
          *
          * XXX: PyMapping_Values?
          */
-        clsValues = PyObject_CallMethod(clsDict, "values", NULL);
+        PyObject* args[2] = {NULL, clsDict};
+        clsValues         = PyObject_VectorcallMethod(PyObjCNM_values, args + 1,
+                                                      1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         Py_DECREF(clsDict);
         if (clsValues == NULL) {
             PyErr_Clear();
@@ -1231,12 +1289,15 @@ free_ivars(id self, PyObject* cls)
                         NSLog(@"ignoring exception %@ in destructor", localException);
                     }
                 Py_END_ALLOW_THREADS
-                *(id*)(((char*)self) + ivar_getOffset(var)) = NULL;
+                *(id*)(((char*)self) + ivar_getOffset(var)) = nil;
             }
             Py_DECREF(o);
         }
         Py_DECREF(iter);
 
+        /* XXX: Why does this use cls.__bases__()[0] instead of
+         *      the type slot with the primary superclass?
+         */
         o = PyObject_GetAttrString(cls, "__bases__");
         if (o == NULL) {
             PyErr_Clear();
@@ -1276,25 +1337,38 @@ object_method_dealloc(ffi_cif* cif __attribute__((__unused__)),
 
         PyErr_Fetch(&ptype, &pvalue, &ptraceback);
 
-        cls = PyObjCClass_New(object_getClass(self));
-
-        delmethod = PyObjCClass_GetDelMethod(cls);
-        if (delmethod != NULL) {
-            PyObject* s = _PyObjCObject_NewDeallocHelper(self);
-            obj         = PyObject_CallFunction(delmethod, "O", s);
-            _PyObjCObject_FreeDeallocHelper(s);
-            if (obj == NULL) {
-                PyErr_WriteUnraisable(delmethod);
-            } else {
-                Py_DECREF(obj);
+        /* object_getClass will only return Nil if its argument is nil */
+        cls = PyObjCClass_New((Class _Nonnull)object_getClass(self));
+        /* Note: In practice 'cls' will never be null */
+        if (cls != NULL) {
+            delmethod = PyObjCClass_GetDelMethod(cls);
+            if (delmethod != NULL) {
+                PyObject* s = _PyObjCObject_NewDeallocHelper(self);
+                if (s != NULL) {
+                    /* XXX: Why not use Vectorcall uncondationally (through compat layer
+                     * on py3.8 and earlier)? */
+#if PY_VERSION_HEX >= 0x03090000
+                    PyObject* args[2] = {NULL, s};
+                    obj               = PyObject_Vectorcall(delmethod, args + 1,
+                                                            1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+#else
+                    obj = PyObject_CallFunctionObjArgs(delmethod, s, NULL);
+#endif
+                    _PyObjCObject_FreeDeallocHelper(s);
+                    if (obj == NULL) {
+                        PyErr_WriteUnraisable(delmethod);
+                    } else {
+                        Py_DECREF(obj);
+                    }
+                    Py_DECREF(delmethod);
+                }
             }
-            Py_DECREF(delmethod);
+
+            free_ivars(self, cls);
+
+            PyErr_Restore(ptype, pvalue, ptraceback);
         }
 
-        free_ivars(self, cls);
-
-        PyErr_Restore(ptype, pvalue, ptraceback);
-
     PyObjC_END_WITH_GIL
 
     objc_superSetClass(spr, (Class)userdata);
@@ -1334,8 +1408,16 @@ object_method_copyWithZone_(ffi_cif* cif __attribute__((__unused__)), void* resp
     cls = object_getClass(self);
     while (cls != (Class)userdata) {
         unsigned ivarCount, i;
-        Ivar*    ivarList = class_copyIvarList(cls, &ivarCount);
+        /* Returns NULL only when setting ivarCount to 0 */
+        Ivar* ivarList = (Ivar* _Nonnull)class_copyIvarList(cls, &ivarCount);
 
+        /* XXX: This code is not 100% consistent with the code freeing slots,
+         *      in particular for ObjC variables, and the 'isSlot' attribute.
+         *      Document why this is (both here, and if necessary in user docs).
+         *      It might be possible to make the two consistent, but that could
+         *      break backward compatibility.
+         *      This code also needs tests!
+         */
         for (i = 0; i < ivarCount; i++) {
             Ivar        v = ivarList[i];
             const char* typestr;
@@ -1381,28 +1463,28 @@ object_method_respondsToSelector(ffi_cif* cif __attribute__((__unused__)), void*
     id   self      = *(id*)args[0];
     SEL  _meth     = *(SEL*)args[1];
     SEL  aSelector = *(SEL*)args[2];
-    int* pres      = (int*)retval; /* Actually BOOL. */
+    int* p_result  = (int*)retval; /* Actually BOOL. */
 
     struct objc_super spr;
     PyObject*         pyself;
     PyObject*         pymeth;
 
     PyObjC_BEGIN_WITH_GIL
-        /* First check if we respond */
+        /* First check if this class respond */
         pyself = PyObjCObject_New(self, PyObjCObject_kDEFAULT, YES);
         if (pyself == NULL) {
-            *pres = NO;
+            *p_result = NO;
             PyObjC_GIL_RETURNVOID;
         }
         pymeth = PyObjCObject_FindSelector(pyself, aSelector);
         Py_DECREF(pyself);
         if (pymeth) {
-            *pres = YES;
+            *p_result = YES;
 
             if (PyObjCSelector_Check(pymeth)
                 && (((PyObjCSelector*)pymeth)->sel_flags
                     & PyObjCSelector_kCLASS_METHOD)) {
-                *pres = NO;
+                *p_result = NO;
             }
 
             Py_DECREF(pymeth);
@@ -1416,8 +1498,8 @@ object_method_respondsToSelector(ffi_cif* cif __attribute__((__unused__)), void*
     objc_superSetClass(spr, (Class)userdata);
     objc_superSetReceiver(spr, self);
 
-    *pres = ((int (*)(struct objc_super*, SEL, SEL))objc_msgSendSuper)(&spr, _meth,
-                                                                       aSelector);
+    *p_result = ((int (*)(struct objc_super*, SEL, SEL))objc_msgSendSuper)(&spr, _meth,
+                                                                           aSelector);
     return;
 }
 
@@ -1433,22 +1515,28 @@ object_method_methodSignatureForSelector(ffi_cif* cif __attribute__((__unused__)
     struct objc_super   spr;
     PyObject*           pyself;
     PyObject*           pymeth;
-    NSMethodSignature** presult = (NSMethodSignature**)retval;
+    NSMethodSignature** p_result = (NSMethodSignature**)retval;
 
-    *presult = nil;
+    *p_result = nil;
 
     objc_superSetClass(spr, (Class)userdata);
     objc_superSetReceiver(spr, self);
 
+    /*
+     * XXX: This checks self and super in a different order than the
+     *      previous function, without a clear reason. Note that either
+     *      order should work because we check that Python methods overriding
+     *      an existing method have a compatible signature.
+     */
     @try {
-        *presult = ((NSMethodSignature * (*)(struct objc_super*, SEL, SEL))
-                        objc_msgSendSuper)(&spr, _meth, aSelector);
+        *p_result = ((NSMethodSignature * (*)(struct objc_super*, SEL, SEL))
+                         objc_msgSendSuper)(&spr, _meth, aSelector);
 
     } @catch (NSObject* localException) {
-        *presult = nil;
+        *p_result = nil;
     }
 
-    if (*presult != nil) {
+    if (*p_result != nil) {
         return;
     }
 
@@ -1469,7 +1557,8 @@ object_method_methodSignatureForSelector(ffi_cif* cif __attribute__((__unused__)
     PyObjC_END_WITH_GIL
 
     @try {
-        *presult = [NSMethodSignature
+        /* XXX: Shouldn't this be sel_native_signature? */
+        *p_result = [NSMethodSignature
             signatureWithObjCTypes:((PyObjCSelector*)pymeth)->sel_python_signature];
     } @catch (NSObject* localException) {
         PyObjC_BEGIN_WITH_GIL
@@ -1529,7 +1618,8 @@ object_method_forwardInvocation(ffi_cif* cif __attribute__((__unused__)),
             @throw;
 
             /* Avoid compiler warnings */
-            theSelector = @selector(init);
+            /* XXX: Is this really necessary with current compilers? */
+            __builtin_unreachable();
         }
     Py_END_ALLOW_THREADS
 
@@ -1553,8 +1643,9 @@ object_method_forwardInvocation(ffi_cif* cif __attribute__((__unused__)),
         return;
     }
 
-    signature = PyObjCMethodSignature_FromSignature(PyObjCSelector_Signature(pymeth), NO);
-    len       = Py_SIZE(signature);
+    signature =
+        PyObjCMethodSignature_WithMetaData(PyObjCSelector_Signature(pymeth), NULL, NO);
+    len = Py_SIZE(signature);
 
     Py_XDECREF(pymeth);
     pymeth = NULL;
@@ -1614,7 +1705,9 @@ object_method_forwardInvocation(ffi_cif* cif __attribute__((__unused__)),
             }
             PyMem_Free(argbuf);
             argbuf = NULL;
-            continue;
+            v      = Py_None;
+            Py_INCREF(Py_None);
+            break;
         default:
             v = pythonify_c_value(type, argbuf);
         }
@@ -1835,16 +1928,18 @@ object_method_forwardInvocation(ffi_cif* cif __attribute__((__unused__)),
 
 /*
  * XXX: Function PyObjC_CallPython should be moved
+ *
+ * XXX: Move API to vectorcall convention, and possibly inline in
+ * its sole caller.
  */
-PyObject*
-PyObjC_CallPython(id self, SEL selector, PyObject* arglist, BOOL* isAlloc,
-                  BOOL* isCFAlloc)
+static PyObject* _Nullable PyObjC_CallPython(id self, SEL selector, PyObject* arglist,
+                                             BOOL* isAlloc, BOOL* isCFAlloc)
 {
     PyObject* pyself = NULL;
     PyObject* pymeth = NULL;
     PyObject* result;
 
-    pyself = pythonify_c_value(@encode(id), &self);
+    pyself = id_to_python(self);
     if (pyself == NULL) {
         return NULL;
     }
@@ -1884,11 +1979,28 @@ PyObjC_CallPython(id self, SEL selector, PyObject* arglist, BOOL* isAlloc,
     }
 
     if (isAlloc != NULL) {
-        *isAlloc = PyObjCSelector_GetMetadata(pymeth)->rettype->alreadyRetained;
+        PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(pymeth);
+        if (methinfo == NULL) {
+            Py_DECREF(arglist);
+            Py_DECREF(pymeth);
+            Py_DECREF(pyself);
+            return NULL;
+
+        } else {
+            *isAlloc = methinfo->rettype->alreadyRetained;
+        }
     }
 
     if (isCFAlloc != NULL) {
-        *isCFAlloc = PyObjCSelector_GetMetadata(pymeth)->rettype->alreadyCFRetained;
+        PyObjCMethodSignature* methinfo = PyObjCSelector_GetMetadata(pymeth);
+        if (methinfo == NULL) {
+            Py_DECREF(arglist);
+            Py_DECREF(pymeth);
+            Py_DECREF(pyself);
+            return NULL;
+        } else {
+            *isCFAlloc = methinfo->rettype->alreadyCFRetained;
+        }
     }
 
     result = PyObject_Call(pymeth, arglist, NULL);
@@ -1958,7 +2070,7 @@ object_method_valueForKey_(ffi_cif* cif __attribute__((__unused__)), void* retva
                     }
                 }
 
-                /* Check that we don't accidently return
+                /* Check that we don't accidentally return
                  * an accessor method.
                  */
                 if (PyObjCSelector_Check(res)
@@ -2017,7 +2129,7 @@ object_method_setValue_forKey_(ffi_cif* cif __attribute__((__unused__)),
             [[self class] accessInstanceVariablesDirectly]) {
 
             PyGILState_STATE state = PyGILState_Ensure();
-            PyObject*        val   = pythonify_c_value(@encode(id), &value);
+            PyObject*        val   = id_to_python(value);
             if (val == NULL) {
                 PyErr_Clear();
                 PyGILState_Release(state);
@@ -2054,3 +2166,5 @@ object_method_setValue_forKey_(ffi_cif* cif __attribute__((__unused__)),
         }
     }
 }
+
+NS_ASSUME_NONNULL_END
