@@ -1,6 +1,8 @@
 #ifndef PyObjC_CLASS_LIST_H
 #define PyObjC_CLASS_LIST_H
 
+NS_ASSUME_NONNULL_BEGIN
+
 /*!
  * PYOBJC_EXPECTED_CLASS_COUNT: Hint about the number of classes to expect
  *
@@ -8,17 +10,6 @@
  */
 #define PYOBJC_EXPECTED_CLASS_COUNT 10000
 
-/*!
- * @header class-list.h
- * @abstract Get the list of classes in the Objective-C runtime
- * @discussion
- *    This module defines a function that returns the list of classes
- *    in the Objetive-C runtime (converted to Python)
- */
-
-Py_ssize_t PyObjC_ClassCount(void);
-
-
 /*!
  * @function PyObjC_GetClassList
  * @result Returns a list of Objective-C classes
@@ -26,6 +17,8 @@ Py_ssize_t PyObjC_ClassCount(void);
  *     This function returns a list containing the wrappers for all classes
  *     in the Objective-C runtime.
  */
-PyObject* PyObjC_GetClassList(void);
+PyObject* _Nullable PyObjC_GetClassList(void);
+
+NS_ASSUME_NONNULL_END
 
 #endif /* PyObjC_CLASS_LIST_H */
