@@ -13,6 +13,8 @@
  *    defining subclasses of objective-C classes.
  */
 
+NS_ASSUME_NONNULL_BEGIN
+
 static void
 ivar_dealloc(PyObject* _ivar)
 {
@@ -24,8 +26,7 @@ ivar_dealloc(PyObject* _ivar)
     Py_TYPE(ivar)->tp_free((PyObject*)ivar);
 }
 
-static PyObject*
-ivar_repr(PyObject* _self)
+static PyObject* _Nullable ivar_repr(PyObject* _self)
 {
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
     if (self->isOutlet) {
@@ -44,22 +45,31 @@ ivar_repr(PyObject* _self)
     }
 }
 
-static PyObject*
-ivar_descr_get(PyObject* _self, PyObject* obj, PyObject* type __attribute__((__unused__)))
+static PyObject* _Nullable ivar_descr_get(PyObject* _self, PyObject* _Nullable obj,
+                                          PyObject* type __attribute__((__unused__)))
 {
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    Ivar var;
-    id objc;
-    PyObject* res;
+    Ivar                    var;
+    id                      objc;
+    PyObject*               res;
+
+    if (!obj) {
+        /* Getting the instance variable from a class, return the descriptor itself
+         * to make it easier to introspect.
+         */
+        Py_INCREF(_self);
+        return _self;
+    }
 
     if (!obj || PyObjCClass_Check(obj)) {
+        /* XXX: Not sure if this is ever true... */
         PyErr_SetString(PyExc_TypeError, "Cannot access Objective-C instance-variables "
                                          "through class");
         return NULL;
     }
 
     if (!PyObjCObject_Check(obj)) {
-        PyErr_SetString(PyExc_TypeError, "objc_ivar descriptor on a non-objc object");
+        PyErr_SetString(PyExc_TypeError, "objc.ivar descriptor on a plain Python object");
         return NULL;
     }
 
@@ -79,7 +89,7 @@ ivar_descr_get(PyObject* _self, PyObject* obj, PyObject* type __attribute__((__u
     if (var == NULL) {
         PyErr_Format(
             PyExc_RuntimeError,
-            "objc_ivar descriptor for non-existing instance variable '%s' in class '%s'",
+            "objc.ivar descriptor for non-existing instance variable '%s' in class '%s'",
             self->name, class_getName(object_getClass(objc)));
         return NULL;
     }
@@ -96,10 +106,15 @@ ivar_descr_get(PyObject* _self, PyObject* obj, PyObject* type __attribute__((__u
     } else {
         const char* encoding = ivar_getTypeEncoding(var);
 
+        if (encoding == NULL) {
+            PyErr_SetString(PyObjCExc_Error, "Cannot extract type encoding from ivar");
+            return NULL;
+        }
+
         if (encoding[0] == _C_ID) {
             /* An object */
             id value = object_getIvar(objc, var);
-            res = pythonify_c_value(encoding, &value);
+            res      = pythonify_c_value(encoding, &value);
         } else {
             res = pythonify_c_value(encoding, ((char*)objc) + ivar_getOffset(var));
         }
@@ -108,13 +123,13 @@ ivar_descr_get(PyObject* _self, PyObject* obj, PyObject* type __attribute__((__u
 }
 
 static int
-ivar_descr_set(PyObject* _self, PyObject* obj, PyObject* value)
+ivar_descr_set(PyObject* _self, PyObject* _Nullable obj, PyObject* _Nullable value)
 {
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    Ivar var;
-    id objc;
-    Py_ssize_t size;
-    int res;
+    Ivar                    var;
+    id                      objc;
+    Py_ssize_t              size;
+    int                     res;
 
     if (value == NULL && !self->isSlot) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete Objective-C instance variables");
@@ -122,13 +137,13 @@ ivar_descr_set(PyObject* _self, PyObject* obj, PyObject* value)
     }
 
     if (!obj || PyObjCClass_Check(obj)) {
-        PyErr_SetString(PyExc_TypeError, "Cannot access Objective-C instance-variables "
+        PyErr_SetString(PyExc_TypeError, "Cannot set Objective-C instance-variables "
                                          "through class");
         return -1;
     }
 
     if (!PyObjCObject_Check(obj)) {
-        PyErr_SetString(PyExc_TypeError, "objc_ivar descriptor on a non-objc object");
+        PyErr_SetString(PyExc_TypeError, "objc.ivar descriptor on a plain Python object");
         return -1;
     }
 
@@ -216,13 +231,13 @@ ivar_descr_set(PyObject* _self, PyObject* obj, PyObject* value)
 }
 
 static int
-ivar_init(PyObject* _self, PyObject* args, PyObject* kwds)
+ivar_init(PyObject* _self, PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
-    static char* keywords[] = {"name", "type", "isOutlet", NULL};
-    PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    char* name = NULL;
-    char* type = @encode(id);
-    PyObject* isOutletObj = NULL;
+    static char*            keywords[]  = {"name", "type", "isOutlet", NULL};
+    PyObjCInstanceVariable* self        = (PyObjCInstanceVariable*)_self;
+    char*                   name        = NULL;
+    char*                   type        = @encode(id);
+    PyObject*               isOutletObj = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|syO:objc_ivar", keywords, &name, &type,
                                      &isOutletObj)) {
@@ -254,23 +269,23 @@ ivar_init(PyObject* _self, PyObject* args, PyObject* kwds)
         self->isOutlet = 0;
     }
 
-    self->ivar = NULL;
+    self->ivar   = NULL;
     self->isSlot = 0;
 
     return 0;
 }
 
-static PyObject*
-ivar_class_setup(PyObject* _self, PyObject* args, PyObject* kwds)
+static PyObject* _Nullable ivar_class_setup(PyObject* _self, PyObject* _Nullable args,
+                                            PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"name", "class_dict", "instance_method_list",
                                "class_method_list", NULL};
 
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    char* name;
-    PyObject* class_dict;
-    PyObject* instance_method_list;
-    PyObject* class_method_list;
+    char*                   name;
+    PyObject*               class_dict;
+    PyObject*               instance_method_list;
+    PyObject*               class_method_list;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "sO!O!O!", keywords, &name, &PyDict_Type,
                                      &class_dict, &PySet_Type, &instance_method_list,
@@ -288,8 +303,8 @@ ivar_class_setup(PyObject* _self, PyObject* args, PyObject* kwds)
 }
 
 static PyMethodDef ivar_methods[] = {{
-                                         .ml_name = "__pyobjc_class_setup__",
-                                         .ml_meth = (PyCFunction)ivar_class_setup,
+                                         .ml_name  = "__pyobjc_class_setup__",
+                                         .ml_meth  = (PyCFunction)ivar_class_setup,
                                          .ml_flags = METH_VARARGS | METH_KEYWORDS,
                                      },
 
@@ -302,7 +317,7 @@ PyDoc_STRVAR(
     "ivar(name, type='@', isOutlet=False) -> instance-variable\n" CLINIC_SEP "\n"
     "Creates a descriptor for accessing an Objective-C instance variable.\n\n"
     "This should only be used in the definition of Objective-C subclasses, and\n"
-    "will then automaticly define the instance variable in the objective-C side.\n"
+    "will then automatically define the instance variable in the objective-C side.\n"
     "\n"
     "'type' is optional and should be a signature string.\n"
     "\n"
@@ -310,8 +325,8 @@ PyDoc_STRVAR(
     "value is assigned to");
 
 PyDoc_STRVAR(ivar_typestr_doc, "The Objective-C type encoding");
-static PyObject*
-ivar_get_typestr(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable ivar_get_typestr(PyObject* _self, void* _Nullable closure
+                                            __attribute__((__unused__)))
 {
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
 
@@ -319,8 +334,8 @@ ivar_get_typestr(PyObject* _self, void* closure __attribute__((__unused__)))
 }
 
 PyDoc_STRVAR(ivar_name_doc, "The Objective-C name");
-static PyObject*
-ivar_get_name(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable ivar_get_name(PyObject* _self, void* _Nullable closure
+                                         __attribute__((__unused__)))
 {
     PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
 
@@ -333,44 +348,44 @@ ivar_get_name(PyObject* _self, void* closure __attribute__((__unused__)))
 }
 
 PyDoc_STRVAR(ivar_isOutlet_doc, "True if the instance variable is an IBOutlet");
-static PyObject*
-ivar_get_isOutlet(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable ivar_get_isOutlet(PyObject* _self, void* _Nullable closure
+                                             __attribute__((__unused__)))
 {
-    PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    PyObject* result = self->isOutlet ? Py_True : Py_False;
+    PyObjCInstanceVariable* self   = (PyObjCInstanceVariable*)_self;
+    PyObject*               result = self->isOutlet ? Py_True : Py_False;
     Py_INCREF(result);
     return result;
 }
 
 PyDoc_STRVAR(ivar_isSlot_doc, "True if the instance variable is a Python slot");
-static PyObject*
-ivar_get_isSlot(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable ivar_get_isSlot(PyObject* _self, void* _Nullable closure
+                                           __attribute__((__unused__)))
 {
-    PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)_self;
-    PyObject* result = self->isSlot ? Py_True : Py_False;
+    PyObjCInstanceVariable* self   = (PyObjCInstanceVariable*)_self;
+    PyObject*               result = self->isSlot ? Py_True : Py_False;
     Py_INCREF(result);
     return result;
 }
 
 static PyGetSetDef ivar_getset[] = {{
                                         .name = "__typestr__",
-                                        .get = ivar_get_typestr,
-                                        .doc = ivar_typestr_doc,
+                                        .get  = ivar_get_typestr,
+                                        .doc  = ivar_typestr_doc,
                                     },
                                     {
                                         .name = "__name__",
-                                        .get = ivar_get_name,
-                                        .doc = ivar_name_doc,
+                                        .get  = ivar_get_name,
+                                        .doc  = ivar_name_doc,
                                     },
                                     {
                                         .name = "__isOutlet__",
-                                        .get = ivar_get_isOutlet,
-                                        .doc = ivar_isOutlet_doc,
+                                        .get  = ivar_get_isOutlet,
+                                        .doc  = ivar_isOutlet_doc,
                                     },
                                     {
                                         .name = "__isSlot__",
-                                        .get = ivar_get_isSlot,
-                                        .doc = ivar_isSlot_doc,
+                                        .get  = ivar_get_isSlot,
+                                        .doc  = ivar_isSlot_doc,
                                     },
                                     {
                                         .name = NULL /* SENTINEL */
@@ -378,68 +393,23 @@ static PyGetSetDef ivar_getset[] = {{
 
 PyTypeObject PyObjCInstanceVariable_Type = {
     PyVarObject_HEAD_INIT(&PyType_Type, 0).tp_name = "ivar",
-    .tp_basicsize = sizeof(PyObjCInstanceVariable),
-    .tp_itemsize = 0,
-    .tp_dealloc = ivar_dealloc,
-    .tp_repr = ivar_repr,
-    .tp_getattro = PyObject_GenericGetAttr,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_doc = ivar_doc,
-    .tp_methods = ivar_methods,
-    .tp_getset = ivar_getset,
-    .tp_descr_get = ivar_descr_get,
-    .tp_descr_set = ivar_descr_set,
-    .tp_init = ivar_init,
-    .tp_alloc = PyType_GenericAlloc,
-    .tp_new = PyType_GenericNew,
+    .tp_basicsize                                  = sizeof(PyObjCInstanceVariable),
+    .tp_itemsize                                   = 0,
+    .tp_dealloc                                    = ivar_dealloc,
+    .tp_repr                                       = ivar_repr,
+    .tp_getattro                                   = PyObject_GenericGetAttr,
+    .tp_flags                                      = Py_TPFLAGS_DEFAULT,
+    .tp_doc                                        = ivar_doc,
+    .tp_methods                                    = ivar_methods,
+    .tp_getset                                     = ivar_getset,
+    .tp_descr_get                                  = ivar_descr_get,
+    .tp_descr_set                                  = ivar_descr_set,
+    .tp_init                                       = ivar_init,
+    .tp_alloc                                      = PyType_GenericAlloc,
+    .tp_new                                        = PyType_GenericNew,
 };
 
-/* Set the name of an ivar if it doesn't already have one
- * This should only be used during class construction.
- */
-int
-PyObjCInstanceVariable_SetName(PyObject* value, PyObject* name)
-{
-    if (!PyObjCInstanceVariable_Check(value)) {
-        PyErr_SetString(PyExc_TypeError, "unexpected type for ivar.setname");
-        return -1;
-    }
-
-    PyObjCInstanceVariable* self = (PyObjCInstanceVariable*)value;
-    if (self->name) {
-        return 0;
-    }
-
-    if (PyUnicode_Check(name)) {
-        PyObject* bytes = PyUnicode_AsEncodedString(name, NULL, NULL);
-        if (bytes == NULL) {
-            return -1;
-        }
-
-        char* b = PyBytes_AsString(bytes);
-        if (b == NULL || *b == '\0') {
-            PyErr_SetString(PyExc_ValueError, "Empty name");
-            return -1;
-        }
-
-        self->name = PyObjCUtil_Strdup(b);
-        Py_DECREF(bytes);
-        if (self->name == NULL) {
-            PyErr_NoMemory();
-            return -1;
-        }
-
-    } else {
-        PyErr_SetString(PyExc_TypeError,
-                        "Implied instance variable name is not a string");
-        return -1;
-    }
-
-    return (self->name == NULL ? -1 : 0);
-}
-
-PyObject*
-PyObjCInstanceVariable_New(char* name)
+PyObject* _Nullable PyObjCInstanceVariable_New(const char* name)
 {
     PyObject* result;
 
@@ -455,9 +425,9 @@ PyObjCInstanceVariable_New(char* name)
     }
 
     ((PyObjCInstanceVariable*)result)->isOutlet = 0;
-    ((PyObjCInstanceVariable*)result)->isSlot = 0;
-    ((PyObjCInstanceVariable*)result)->ivar = 0;
-    ((PyObjCInstanceVariable*)result)->name = PyObjCUtil_Strdup(name);
+    ((PyObjCInstanceVariable*)result)->isSlot   = 0;
+    ((PyObjCInstanceVariable*)result)->ivar     = 0;
+    ((PyObjCInstanceVariable*)result)->name     = PyObjCUtil_Strdup(name);
 
     if (((PyObjCInstanceVariable*)result)->name == NULL) {
         PyMem_Free(((PyObjCInstanceVariable*)result)->type);
@@ -466,3 +436,5 @@ PyObjCInstanceVariable_New(char* name)
     }
     return result;
 }
+
+NS_ASSUME_NONNULL_END
