@@ -1,7 +1,8 @@
 #include "pyobjc.h"
 
-static Ivar
-find_ivar(NSObject* base, const char* name)
+NS_ASSUME_NONNULL_BEGIN
+
+static Ivar _Nullable find_ivar(NSObject* base, const char* name)
 {
     Class cur = object_getClass((id)base);
     Ivar  ivar;
@@ -16,8 +17,8 @@ find_ivar(NSObject* base, const char* name)
     return nil;
 }
 
-PyObject*
-PyObjCIvar_Info(PyObject* self __attribute__((__unused__)), PyObject* object)
+PyObject* _Nullable PyObjCIvar_Info(PyObject* self __attribute__((__unused__)),
+                                    PyObject* object)
 {
     Class     cur;
     PyObject* v;
@@ -102,8 +103,8 @@ PyObjCIvar_Info(PyObject* self __attribute__((__unused__)), PyObject* object)
     return result;
 }
 
-PyObject*
-PyObjCIvar_Get(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
+PyObject* _Nullable PyObjCIvar_Get(PyObject* self __attribute__((__unused__)),
+                                   PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"obj", "name", NULL};
 
@@ -127,6 +128,13 @@ PyObjCIvar_Get(PyObject* self __attribute__((__unused__)), PyObject* args, PyObj
     }
 
     objcValue = PyObjCObject_GetObject(anObject);
+    if (objcValue == NULL) {
+        if (!PyErr_Occurred()) {
+            PyErr_SetString(PyExc_ValueError,
+                            "Getting instance variable from nil object");
+        }
+        return NULL;
+    }
 
     /* Shortcut for isa, mostly due to Objective-C 2.0 weirdness */
     if (strcmp(name, "isa") == 0) {
@@ -154,8 +162,8 @@ PyObjCIvar_Get(PyObject* self __attribute__((__unused__)), PyObject* args, PyObj
     return result;
 }
 
-PyObject*
-PyObjCIvar_Set(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
+PyObject* _Nullable PyObjCIvar_Set(PyObject* self __attribute__((__unused__)),
+                                   PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"obj", "name", "value", "updateRefCounts", NULL};
 
@@ -182,6 +190,12 @@ PyObjCIvar_Set(PyObject* self __attribute__((__unused__)), PyObject* args, PyObj
     }
 
     objcValue = PyObjCObject_GetObject(anObject);
+    if (objcValue == NULL) {
+        if (!PyErr_Occurred()) {
+            PyErr_SetString(PyExc_ValueError, "Setting instance variable of nil object");
+        }
+        return NULL;
+    }
 
     if (strcmp(name, "isa") == 0) {
         /*
@@ -204,7 +218,10 @@ PyObjCIvar_Set(PyObject* self __attribute__((__unused__)), PyObject* args, PyObj
             return NULL;
         }
 
-        curType           = Py_TYPE(anObject);
+        /* XXX: See comment in objc-object.m, change python
+         *      type by calling setattr(anObejct, "__class__", pycls)
+         */
+        curType = Py_TYPE(anObject);
         Py_SET_TYPE(anObject, (PyTypeObject*)pycls);
         Py_DECREF((PyObject*)curType);
         Py_INCREF(Py_None);
@@ -273,3 +290,5 @@ PyObjCIvar_Set(PyObject* self __attribute__((__unused__)), PyObject* args, PyObj
     Py_INCREF(Py_None);
     return Py_None;
 }
+
+NS_ASSUME_NONNULL_END
