@@ -1,5 +1,8 @@
 #ifndef PyObjC_OBJC_CLASS_H
 #define PyObjC_OBJC_CLASS_H
+
+NS_ASSUME_NONNULL_BEGIN
+
 /*!
  * @header objc-class.m
  * @abstract Definition of the wrapper for Objective-C classes
@@ -25,9 +28,6 @@
 extern PyTypeObject PyObjCMetaClass_Type;
 extern PyTypeObject PyObjCClass_Type;
 
-/*extern int setup_class_meta(void);*/
-
-
 /*!
  * @function PyObjCClass_Check
  * @abstract Check if an object is an Objective-C class
@@ -52,7 +52,6 @@ extern PyTypeObject PyObjCClass_Type;
  */
 extern PyObject* PyObjC_ClassExtender;
 
-
 /*!
  * @struct PyObjCClassObject
  * @abstract The type struct for Objective-C classes (Python 2.3 and later)
@@ -81,45 +80,92 @@ extern PyObject* PyObjC_ClassExtender;
 typedef struct _PyObjCClassObject {
     PyHeapTypeObject base;
     Class class;
-    PyObject* sel_to_py;
-    PyObject* delmethod;
-    PyObject* hiddenSelectors;
-    PyObject* hiddenClassSelectors;
+    PyObject* _Nullable sel_to_py;
+    PyObject* _Nullable delmethod;
+    PyObject* _Nullable hiddenSelectors;
+    PyObject* _Nullable hiddenClassSelectors;
+    PyObject* _Nullable lookup_cache;
 
-    Py_ssize_t dictoffset;
-    Py_ssize_t generation;
-    unsigned int useKVO:1;
-    unsigned int hasPythonImpl:1;
-    unsigned int isCFWrapper:1;
+    Py_ssize_t   dictoffset;
+    Py_ssize_t   generation;
+    unsigned int useKVO : 1;
+    unsigned int hasPythonImpl : 1;
+    unsigned int isCFWrapper : 1;
+    unsigned int isFinal : 1;
 } PyObjCClassObject;
 
-
-extern PyObject* PyObjCClass_DefaultModule;
-extern PyObject* PyObjCClass_New(Class objc_class);
-extern Class PyObjCClass_GetClass(PyObject* object);
-extern PyObject* PyObjCClass_FindSelector(PyObject* cls, SEL selector, BOOL class_method);
-extern void PyObjCClass_MaybeRescan(PyObject* class);
-extern int ObjC_RegisterClassProxy(Class cls, PyObject* classProxy);
-extern int PyObjCClass_CheckMethodList(PyObject* cls, int recursive);
+extern PyObject* _Nullable PyObjCClass_DefaultModule;
+extern PyObject* _Nullable PyObjCClass_New(Class objc_class);
+extern Class _Nullable PyObjCClass_GetClass(PyObject* object);
+extern PyObject* _Nullable PyObjCClass_FindSelector(PyObject* cls, SEL selector,
+                                                    BOOL class_method);
+extern void       PyObjCClass_MaybeRescan(PyObject* class);
+extern int        ObjC_RegisterClassProxy(Class cls, PyObject* classProxy);
+extern int        PyObjCClass_CheckMethodList(PyObject* cls, int recursive);
 extern Py_ssize_t PyObjCClass_DictOffset(PyObject* cls);
-extern PyObject* PyObjCClass_GetDelMethod(PyObject* cls);
+extern PyObject* _Nullable PyObjCClass_GetDelMethod(PyObject* cls);
 extern void PyObjCClass_SetDelMethod(PyObject* cls, PyObject* newval);
-extern int PyObjCClass_HasPythonImplementation(PyObject* cls);
-extern PyObject* PyObjCClass_ClassForMetaClass(PyObject* meta);
-extern PyObject* PyObjCClass_HiddenSelector(PyObject* tp, SEL sel, BOOL classMethod); /* returns borrowed */
-extern int PyObjCClass_SetHidden(PyObject* tp, SEL sel, BOOL classMethod, PyObject* metadata);
-extern int PyObjCClass_AddMethods(PyObject* cls, PyObject** methods, Py_ssize_t count);
-extern PyObject* PyObjCClass_ListProperties(PyObject* cls);
+extern int  PyObjCClass_HasPythonImplementation(PyObject* cls);
+extern PyObject* _Nullable PyObjCClass_ClassForMetaClass(PyObject* meta);
+extern PyObject* _Nullable PyObjCClass_HiddenSelector(
+    PyObject* tp, SEL sel, BOOL classMethod); /* returns borrowed */
+extern int PyObjCClass_SetHidden(PyObject* tp, SEL sel, BOOL classMethod,
+                                 PyObject* metadata);
+extern int PyObjCClass_AddMethods(PyObject*  cls, PyObject* _Nonnull* _Nonnull methods,
+                                  Py_ssize_t count);
+extern PyObject* _Nullable PyObjCClass_ListProperties(PyObject* cls);
 
 /* Returns a borrowed reference or NULL (without necessarily raising an exception) */
-extern PyObject* PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel);
-extern PyObject* PyObjCMetaClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel);
+extern PyObject* _Nullable PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name,
+                                                          SEL sel);
+extern PyObject* _Nullable PyObjCMetaClass_TryResolveSelector(PyObject* base,
+                                                              PyObject* name, SEL sel);
 
-static inline int PyObjCClass_IsCFWrapper(PyTypeObject* tp)
+static inline PyObject* _Nullable PyObjCClass_GetLookupCache(PyTypeObject* tp)
+{
+    return ((PyObjCClassObject*)tp)->lookup_cache;
+}
+
+static inline int
+PyObjCClass_AddToLookupCache(PyTypeObject* _tp, PyObject* name, PyObject* value)
+{
+#ifdef PyObjC_ENABLE_LOOKUP_CACHE
+    int                r;
+    PyObjCClassObject* tp = (PyObjCClassObject*)_tp;
+    if (tp->lookup_cache == NULL) {
+        tp->lookup_cache = PyDict_New();
+        if (tp->lookup_cache == NULL) {
+            return -1;
+        }
+    }
+    r = PyDict_SetItem(tp->lookup_cache, name, value);
+    return r;
+#else
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-value"
+
+    &_tp;
+    &name;
+    &value;
+#pragma clang diagnostic pop
+    return 0;
+#endif
+}
+
+static inline int
+PyObjCClass_IsCFWrapper(PyTypeObject* tp)
 {
     return ((PyObjCClassObject*)tp)->isCFWrapper;
 }
 
-PyObject* objc_class_locate(Class objc_class);
+static inline int
+PyObjCClass_IsFinal(PyTypeObject* tp)
+{
+    return ((PyObjCClassObject*)tp)->isFinal;
+}
+
+PyObject* _Nullable objc_class_locate(Class objc_class);
+
+NS_ASSUME_NONNULL_END
 
 #endif /* PyObjC_OBJC_CLASS_H */
