@@ -7,6 +7,8 @@
 
 #include <stddef.h>
 
+NS_ASSUME_NONNULL_BEGIN
+
 int
 PyObjCClass_SetHidden(PyObject* tp, SEL sel, BOOL classMethod, PyObject* metadata)
 {
@@ -41,13 +43,15 @@ PyObjCClass_SetHidden(PyObject* tp, SEL sel, BOOL classMethod, PyObject* metadat
     }
 
     v = PyBytes_InternFromString(sel_getName(sel));
+    if (v == NULL) {
+        return -1;
+    }
     r = PyDict_SetItem(hidden, v, metadata);
     Py_DECREF(v);
     return r;
 }
 
-PyObject*
-PyObjCClass_HiddenSelector(PyObject* tp, SEL sel, BOOL classMethod)
+PyObject* _Nullable PyObjCClass_HiddenSelector(PyObject* tp, SEL sel, BOOL classMethod)
 {
     PyObject*  mro;
     Py_ssize_t i, n;
@@ -163,8 +167,8 @@ static int update_convenience_methods(PyObject* cls);
  *
  *    The key is the Objective-C class, the value is its wrapper.
  */
-static NSMapTable* class_registry     = NULL;
-static NSMapTable* metaclass_to_class = NULL;
+static NSMapTable* _Nullable class_registry     = NULL;
+static NSMapTable* _Nullable metaclass_to_class = NULL;
 
 /*!
  * @function objc_class_register
@@ -188,7 +192,7 @@ objc_class_register(Class objc_class, PyObject* py_class)
 
     if (NSMapGet(class_registry, objc_class)) {
         PyErr_Format(PyObjCExc_InternalError, "Registering class '%.100s' more than once",
-		class_getName(objc_class));
+                     class_getName(objc_class));
         return -1;
     }
 
@@ -222,8 +226,7 @@ objc_metaclass_register(PyTypeObject* meta_class, Class class)
     return 0;
 }
 
-static Class
-objc_metaclass_locate(PyObject* meta_class)
+static Class _Nullable objc_metaclass_locate(PyObject* meta_class)
 {
     Class result;
 
@@ -244,9 +247,11 @@ objc_metaclass_locate(PyObject* meta_class)
  * @discussion
  *     This function does not raise an Python exception when the
  *     wrapper cannot be found.
+ *
+ * XXX: Is this function needed? Why not use the same registry
+ *      as used by objc-object?
  */
-PyObject*
-objc_class_locate(Class objc_class)
+PyObject* _Nullable objc_class_locate(Class objc_class)
 {
     PyObject* result;
 
@@ -264,8 +269,7 @@ objc_class_locate(Class objc_class)
  *
  * Returns a new reference.
  */
-static PyTypeObject*
-PyObjCClass_NewMetaClass(Class objc_class)
+static PyTypeObject* _Nullable PyObjCClass_NewMetaClass(Class objc_class)
 {
     PyTypeObject* result;
     Class         objc_meta_class = object_getClass(objc_class);
@@ -339,6 +343,7 @@ PyObjCClass_NewMetaClass(Class objc_class)
 
     if (objc_metaclass_register(result, objc_class) == -1) {
         /* Whoops, no such thing */
+        /* XXX */
         // objc_class_unregister(objc_meta_class);
         return NULL;
     }
@@ -347,7 +352,7 @@ PyObjCClass_NewMetaClass(Class objc_class)
 }
 
 /*
- * Create a new objective-C class, as a subclass of 'type'. This is
+ * class_new: Create a new objective-C class, as a subclass of 'type'. This is
  * PyObjCClass_Type.tp_new.
  *
  * Note: This function creates new _classes_
@@ -373,11 +378,11 @@ class_init(PyObject* cls, PyObject* args, PyObject* kwds)
     return PyType_Type.tp_init(cls, args, kwds);
 }
 
-static PyObject*
-class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObject* kwds)
+static PyObject* _Nullable class_new(PyTypeObject* type __attribute__((__unused__)),
+                                     PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
     static PyObject*   all_python_classes = NULL;
-    static char*       keywords[]         = {"name", "bases", "dict", "protocols", "final", NULL};
+    static char*       keywords[] = {"name", "bases", "dict", "protocols", "final", NULL};
     char*              name;
     PyObject*          bases;
     PyObject*          dict;
@@ -388,8 +393,8 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
     PyObject*          v;
     Py_ssize_t         i;
     Py_ssize_t         len;
-    Class              objc_class     = NULL;
-    Class              super_class    = NULL;
+    Class              objc_class = NULL;
+    Class              super_class; /* XXX    = NULL; */
     PyObject*          py_super_class = NULL;
     PyObjCClassObject* info;
     PyObject*          keys;
@@ -438,18 +443,27 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
                                          "be objective-C based");
         return NULL;
     }
+    if (py_super_class == (PyObject*)&PyObjCObject_Type) {
+        PyErr_SetString(PyExc_TypeError, "Cannot directly inherit from objc.objc_object");
+        return NULL;
+    }
 
     if (PyObjCClass_IsFinal((PyTypeObject*)py_super_class)) {
         PyErr_Format(PyExc_TypeError, "super class %s is final",
-                ((PyTypeObject*)py_super_class)->tp_name);
+                     ((PyTypeObject*)py_super_class)->tp_name);
         return NULL;
     }
 
     super_class = PyObjCClass_GetClass(py_super_class);
-    if (super_class) {
-        if (PyObjCClass_CheckMethodList(py_super_class, 1) < 0) {
-            return NULL;
-        }
+    if (super_class == Nil) {
+        /* This will always set an exception, only the root has
+         * a Nil class and that's excluded above.
+         */
+        PyObjC_Assert(PyErr_Occurred(), NULL);
+        return NULL;
+    }
+    if (PyObjCClass_CheckMethodList(py_super_class, 1) < 0) {
+        return NULL;
     }
 
     hiddenSelectors = PyDict_New();
@@ -681,7 +695,7 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
          */
         objc_class = PyObjCClass_BuildClass(super_class, protocols, name, dict, metadict,
                                             hiddenSelectors, hiddenClassSelectors);
-        if (objc_class == NULL) {
+        if (objc_class == Nil) {
             Py_DECREF(protocols);
             Py_DECREF(metadict);
             Py_DECREF(real_bases);
@@ -691,7 +705,8 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
         }
 
         /* PyObjCClass_BuildClass may have changed the super_class */
-        super_class    = class_getSuperclass(objc_class);
+        /* This is a subclass of an existing class, the superclass will never be Nil */
+        super_class    = (Class _Nonnull)class_getSuperclass(objc_class);
         py_super_class = PyObjCClass_New(super_class);
         if (py_super_class == NULL) {
             (void)PyObjCClass_UnbuildClass(objc_class);
@@ -843,7 +858,10 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
     }
 
     if (PyObjC_MakeBundleForClass != NULL && PyObjC_MakeBundleForClass != Py_None) {
-        PyObject* m = PyObject_CallObject(PyObjC_MakeBundleForClass, NULL);
+        PyObject* args[1] = {NULL};
+
+        PyObject* m = PyObject_Vectorcall(PyObjC_MakeBundleForClass, args + 1,
+                                          0 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         if (m == NULL) {
             (void)PyObjCClass_UnbuildClass(objc_class);
             Py_DECREF(old_dict);
@@ -904,6 +922,15 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
 
     /* call super-class implementation */
     args = Py_BuildValue("(sOO)", name, real_bases, dict);
+    if (args == NULL) {
+        Py_DECREF(metatype);
+        Py_DECREF(real_bases);
+        Py_DECREF(protocols);
+        Py_DECREF(old_dict);
+        Py_DECREF(hiddenSelectors);
+        Py_DECREF(hiddenClassSelectors);
+        return NULL;
+    }
 
     /* The actual superclass might be different due to introduction
      * of magic intermediate classes, therefore explicitly refer to the
@@ -1021,8 +1048,7 @@ class_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObje
     return res;
 }
 
-static PyObject*
-class_repr(PyObject* obj)
+static PyObject* _Nullable class_repr(PyObject* obj)
 {
     Class cls = PyObjCClass_GetClass(obj);
     if (cls) {
@@ -1053,9 +1079,10 @@ class_dealloc(PyObject* cls)
 }
 
 int
-PyObjCClass_CheckMethodList(PyObject* cls, int recursive)
+PyObjCClass_CheckMethodList(PyObject* start_cls, int recursive)
 {
     PyObjCClassObject* info;
+    PyObject* _Nullable cls = start_cls;
 
     info = (PyObjCClassObject*)cls;
 
@@ -1082,7 +1109,16 @@ PyObjCClass_CheckMethodList(PyObject* cls, int recursive)
             break;
         if (class_getSuperclass(info->class) == NULL)
             break;
-        cls = PyObjCClass_New(class_getSuperclass(info->class));
+        /* class_getSuperclass returns Nil only if its argument is Nil */
+        cls = PyObjCClass_New((Class _Nonnull)class_getSuperclass(info->class));
+        if (cls == NULL) {
+            /* Abort checking if the class object cannot be created. Should
+             * never happen in practice...
+             */
+            PyErr_Clear();
+            break;
+        }
+
         Py_DECREF(
             cls); /* We don't actually need the reference, convert to a borrowed one */
         info = (PyObjCClassObject*)cls;
@@ -1090,8 +1126,7 @@ PyObjCClass_CheckMethodList(PyObject* cls, int recursive)
     return 0;
 }
 
-static PyObject*
-metaclass_dir(PyObject* self)
+static PyObject* _Nullable metaclass_dir(PyObject* self)
 {
     PyObject*    result;
     Class        cls;
@@ -1106,34 +1141,59 @@ metaclass_dir(PyObject* self)
     }
 
     cls = objc_metaclass_locate(self);
+    if (cls == NULL) {
+        /* This happens for the root of the class tree */
+        return result;
+    }
+
+    PyObject* self_class = PyObjCClass_ClassForMetaClass(self);
+    if (self_class == NULL) {
+        Py_DECREF(result);
+        return NULL;
+    }
 
     while (cls != NULL) {
         /* Now add all method names */
-        methods = class_copyMethodList(object_getClass(cls), &method_count);
-        for (i = 0; i < method_count; i++) {
-            char*     name;
-            PyObject* item;
 
-            /* Check if the selector should be hidden */
-            if (PyObjCClass_HiddenSelector(PyObjCClass_ClassForMetaClass(self),
-                                           method_getName(methods[i]), YES)) {
+        /* class_copyMethodList only returns NULL when it sets method_count
+         * to 0
+         */
+        methods =
+            (Method* _Nonnull)class_copyMethodList(object_getClass(cls), &method_count);
+        for (i = 0; i < method_count; i++) {
+            const char* name;
+            PyObject*   item;
+            SEL         meth_name = method_getName(methods[i]);
+            if (meth_name == NULL) {
+                /* Ignore methods without a selector */
                 continue;
             }
 
-            name = (char*)PyObjC_SELToPythonName(method_getName(methods[i]), selbuf,
-                                                 sizeof(selbuf));
-            if (name == NULL)
+            /* Check if the selector should be hidden */
+            if (PyObjCClass_HiddenSelector(self_class, meth_name, YES)) {
                 continue;
+            }
+
+            name = PyObjC_SELToPythonName(meth_name, selbuf, sizeof(selbuf));
+            if (name == NULL) {
+                /* Ignore selectors that cannot be converted to a python name.
+                 * This should never happen with the size of selbuf.
+                 */
+                PyErr_Clear();
+                continue;
+            }
 
             item = PyUnicode_FromString(name);
             if (item == NULL) {
                 free(methods);
+                Py_DECREF(self_class);
                 Py_DECREF(result);
                 return NULL;
             }
 
             if (PyList_Append(result, item) == -1) {
                 free(methods);
+                Py_DECREF(self_class);
                 Py_DECREF(result);
                 Py_DECREF(item);
                 return NULL;
@@ -1144,19 +1204,23 @@ metaclass_dir(PyObject* self)
 
         cls = class_getSuperclass(cls);
     }
+    Py_DECREF(self_class);
     return result;
 }
 
 /* FIXME: This is a lightly modified version of _type_lookup in objc-object.m, need to
  * merge these */
-static inline PyObject*
-_type_lookup(PyTypeObject* tp, PyObject* name)
+static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name)
 {
-    Py_ssize_t i, n;
-    PyObject * mro, *base, *dict;
-    PyObject*  descr = NULL;
-    PyObject*  res;
-    SEL        sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
+    Py_ssize_t  i, n;
+    PyObject *  mro, *base, *dict;
+    PyObject*   descr = NULL;
+    PyObject*   res;
+    const char* sel_name = PyObjC_Unicode_Fast_Bytes(name);
+    if (sel_name == NULL) {
+        return NULL;
+    }
+    SEL sel = PyObjCSelector_DefaultSelector(sel_name);
 
     /* TODO: if sel.startswith('__') and sel.endswith('__'): look_in_runtime = False */
 
@@ -1202,14 +1266,16 @@ _type_lookup(PyTypeObject* tp, PyObject* name)
     return descr;
 }
 
-static inline PyObject*
-_type_lookup_harder(PyTypeObject* tp, PyObject* name)
+static inline PyObject* _Nullable _type_lookup_harder(PyTypeObject* tp, PyObject* name)
 /* See function of same name in objc-object.m for an explanation */
 {
-    Py_ssize_t i, n;
-    PyObject * mro, *base;
-    PyObject*  descr = NULL;
-    PyObject*  res;
+    Py_ssize_t  i, n;
+    PyObject *  mro, *base;
+    PyObject*   descr = NULL;
+    PyObject*   res;
+    const char* name_bytes = PyUnicode_AsUTF8(name);
+    if (name_bytes == NULL)
+        return NULL;
 
     /* Look in tp_dict of types in MRO */
     mro = tp->tp_mro;
@@ -1224,31 +1290,64 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name)
         Method*      methods;
         unsigned int method_count, j;
         char         selbuf[2048];
-        char*        sel_name;
+        const char*  sel_name;
 
         base = PyTuple_GET_ITEM(mro, i);
+        PyObjC_Assert(base != NULL, NULL);
 
         if (!PyObject_IsSubclass(base, (PyObject*)&PyObjCMetaClass_Type)) {
             continue;
         }
+        if (base == (PyObject*)&PyObjCClass_Type
+            || base == (PyObject*)&PyObjCMetaClass_Type) {
+            /* Root of the PyObjC class tree, these do not refer to an ObjC class */
+            continue;
+        }
 
-        cls     = objc_metaclass_locate(base);
-        methods = class_copyMethodList(object_getClass(cls), &method_count);
+        cls = objc_metaclass_locate(base);
+        PyObjC_Assert(cls != Nil, NULL);
+
+        PyObject* class_for_base = PyObjCClass_ClassForMetaClass(base);
+        if (class_for_base == NULL) {
+            return NULL;
+        }
+
+        /* class_copyMethodList only returns NULL when it sets method_count
+         * to 0
+         */
+        methods =
+            (Method* _Nonnull)class_copyMethodList(object_getClass(cls), &method_count);
         for (j = 0; j < method_count; j++) {
-            Method m = methods[j];
-
-            if (PyObjCClass_HiddenSelector(PyObjCClass_ClassForMetaClass(base),
-                                           method_getName(m), YES)) {
+            Method m         = methods[j];
+            SEL    meth_name = method_getName(m);
+            if (meth_name == NULL) {
+                /* Method without a selector, ignore these */
                 continue;
             }
 
-            sel_name =
-                (char*)PyObjC_SELToPythonName(method_getName(m), selbuf, sizeof(selbuf));
-            if (strcmp(sel_name, PyObjC_Unicode_Fast_Bytes(name)) == 0) {
+            if (PyObjCClass_HiddenSelector(class_for_base, meth_name, YES)) {
+                continue;
+            }
+
+            sel_name = PyObjC_SELToPythonName(method_getName(m), selbuf, sizeof(selbuf));
+            if (sel_name == NULL) {
+                /* Ignore selectors whose name cannot be converted to a python name */
+                PyErr_Clear();
+                continue;
+            }
+            if (strcmp(sel_name, name_bytes) == 0) {
                 /* Create (unbound) selector */
-                descr = PyObjCSelector_NewNative(cls, method_getName(m),
-                                                 method_getTypeEncoding(m), 1);
+                const char* encoding = method_getTypeEncoding(m);
+                if (encoding == NULL) {
+                    Py_DECREF(class_for_base);
+                    free(methods);
+                    PyErr_SetString(PyObjCExc_Error,
+                                    "Native selector with Nil type encoding");
+                    return NULL;
+                }
+                descr = PyObjCSelector_NewNative(cls, method_getName(m), encoding, 1);
                 free(methods);
+                Py_DECREF(class_for_base);
                 if (descr == NULL) {
                     return NULL;
                 }
@@ -1259,26 +1358,31 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name)
                     return NULL;
                 }
 
-                /* and return, as a borrowed reference */
+                /* and return, as a borrowed reference, safe because
+                 * it is in tp_dict.
+                 */
                 Py_DECREF(descr);
                 return descr;
             }
         }
+        Py_DECREF(class_for_base);
         free(methods);
     }
 
+    /* XXX: just checking... */
+    PyObjC_Assert(descr == NULL, NULL);
     return descr;
 }
 
-PyObject*
-PyObjCMetaClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel)
+PyObject* _Nullable PyObjCMetaClass_TryResolveSelector(PyObject* base, PyObject* name,
+                                                       SEL sel)
 {
     Class     cls;
     Method    m;
     PyObject* dict = ((PyTypeObject*)base)->tp_dict;
 
     Py_BEGIN_ALLOW_THREADS
-        @try {
+        @try { /* XXX: Can this raise?, and is it necessary to give up the GIL here? */
             cls = objc_metaclass_locate(base);
             m   = class_getClassMethod(cls, sel);
 
@@ -1333,8 +1437,8 @@ PyObjCMetaClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel)
 }
 
 /* FIXME: version of _type_lookup that only looks for instance methods */
-static inline PyObject*
-_type_lookup_instance(PyObject* class_dict, PyTypeObject* tp, PyObject* name)
+static inline PyObject* _Nullable _type_lookup_instance(PyObject*     class_dict,
+                                                        PyTypeObject* tp, PyObject* name)
 {
     Py_ssize_t i, n;
     PyObject * mro, *base, *dict;
@@ -1420,14 +1524,19 @@ _type_lookup_instance(PyObject* class_dict, PyTypeObject* tp, PyObject* name)
     return descr;
 }
 
-static inline PyObject*
-_type_lookup_instance_harder(PyObject* class_dict, PyTypeObject* tp, PyObject* name)
+static inline PyObject* _Nullable _type_lookup_instance_harder(PyObject*     class_dict,
+                                                               PyTypeObject* tp,
+                                                               PyObject*     name)
 {
-    Py_ssize_t i, n;
-    PyObject * mro, *base;
-    PyObject*  descr = NULL;
-    PyObject*  res;
-    SEL        sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
+    Py_ssize_t  i, n;
+    PyObject *  mro, *base;
+    PyObject*   descr = NULL;
+    PyObject*   res;
+    const char* name_bytes = PyObjC_Unicode_Fast_Bytes(name);
+    if (name_bytes == NULL) {
+        return NULL;
+    }
+    SEL sel = PyObjCSelector_DefaultSelector(name_bytes);
 
     /* Look in tp_dict of types in MRO */
     mro = tp->tp_mro;
@@ -1449,18 +1558,34 @@ _type_lookup_instance_harder(PyObject* class_dict, PyTypeObject* tp, PyObject* n
             continue;
         }
 
-        cls     = PyObjCClass_GetClass(base);
-        methods = class_copyMethodList(cls, &method_count);
+        cls = PyObjCClass_GetClass(base);
+
+        /* if the result of class_copyMethodList is NULL the method_count
+         * is set to 0. The code below will only use the result if method_count
+         * is greater than 0.
+         */
+        methods = (Method* _Nonnull)class_copyMethodList(cls, &method_count);
         for (j = 0; j < method_count; j++) {
             Method m = methods[j];
 
             sel_name =
                 (char*)PyObjC_SELToPythonName(method_getName(m), selbuf, sizeof(selbuf));
+            if (sel_name == NULL) {
+                /* Ignore methods whose selector cannot be converted */
+                PyErr_Clear();
+                continue;
+            }
 
             if (strcmp(sel_name, PyObjC_Unicode_Fast_Bytes(name)) == 0) {
                 /* Create (unbound) selector */
-                PyObject* result =
-                    PyObjCSelector_NewNative(cls, sel, method_getTypeEncoding(m), 0);
+                const char* encoding = method_getTypeEncoding(m);
+                if (encoding == NULL) {
+                    PyErr_SetString(PyObjCExc_Error,
+                                    "Native selector with Nil type encoding");
+                    free(methods);
+                    return NULL;
+                }
+                PyObject* result = PyObjCSelector_NewNative(cls, sel, encoding, 0);
                 free(methods);
                 if (result == NULL) {
                     return NULL;
@@ -1483,8 +1608,7 @@ _type_lookup_instance_harder(PyObject* class_dict, PyTypeObject* tp, PyObject* n
     return descr;
 }
 
-static PyObject*
-class_getattro(PyObject* self, PyObject* name)
+static PyObject* _Nullable class_getattro(PyObject* self, PyObject* name)
 {
     PyObject*    descr  = NULL;
     PyObject*    result = NULL;
@@ -1582,6 +1706,7 @@ class_getattro(PyObject* self, PyObject* name)
             f = Py_TYPE(descr)->tp_descr_get;
         }
         if (PyErr_Occurred()) {
+            /* XXX: can this happen without descr being NULL? */
             return NULL;
         }
     }
@@ -1599,13 +1724,20 @@ class_getattro(PyObject* self, PyObject* name)
 
     /* Try to find the method anyway */
     PyErr_Clear();
-    if (PyObjCClass_HiddenSelector(self, sel_getUid(PyObjC_Unicode_Fast_Bytes(name)),
-                                   YES)) {
+    const char* name_bytes = PyObjC_Unicode_Fast_Bytes(name);
+    if (name_bytes == NULL) {
+        return NULL;
+    }
+    if (PyObjCClass_HiddenSelector(self, sel_getUid(name_bytes), YES)) {
         PyErr_SetObject(PyExc_AttributeError, name);
         return NULL;
     }
 
-    result = PyObjCSelector_FindNative(self, PyObjC_Unicode_Fast_Bytes(name));
+    name_bytes = PyObjC_Unicode_Fast_Bytes(name);
+    if (name_bytes == NULL) {
+        return NULL;
+    }
+    result = PyObjCSelector_FindNative(self, name_bytes);
 
     if (result != NULL) {
         int res = PyDict_SetItem(((PyTypeObject*)self)->tp_dict, name, result);
@@ -1629,7 +1761,7 @@ done:
 }
 
 static int
-class_setattro(PyObject* self, PyObject* name, PyObject* value)
+class_setattro(PyObject* self, PyObject* name, PyObject* _Nullable value)
 {
     int res;
     if (value == NULL) {
@@ -1694,10 +1826,16 @@ class_setattro(PyObject* self, PyObject* name, PyObject* value)
         }
 
         if (curMethod) {
-            method_setImplementation(
-                curMethod, PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)newVal));
+            IMP newIMP = PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)newVal);
+            if (newIMP == NULL) {
+                Py_DECREF(newVal);
+                return -1;
+            }
+
+            method_setImplementation(curMethod, newIMP);
 
         } else {
+            /* XXX: Why use "strdup" here and not the pyobjc_util variant? */
             char* types = strdup(PyObjCSelector_Signature(newVal));
 
             if (types == NULL) {
@@ -1705,9 +1843,14 @@ class_setattro(PyObject* self, PyObject* name, PyObject* value)
                 return -1;
             }
 
-            b = class_addMethod(
-                curClass, PyObjCSelector_GetSelector(newVal),
-                PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)newVal), types);
+            IMP newIMP = PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)newVal);
+            if (newIMP == NULL) {
+                free(types);
+                Py_DECREF(newVal);
+                return -1;
+            }
+            b = class_addMethod(curClass, PyObjCSelector_GetSelector(newVal), newIMP,
+                                types);
 
             if (!b) {
                 free(types);
@@ -1741,8 +1884,7 @@ class_setattro(PyObject* self, PyObject* name, PyObject* value)
     return res;
 }
 
-static PyObject*
-class_richcompare(PyObject* self, PyObject* other, int op)
+static PyObject* _Nullable class_richcompare(PyObject* self, PyObject* other, int op)
 {
     Class     self_class;
     Class     other_class;
@@ -1845,8 +1987,8 @@ PyDoc_STRVAR(
     cls_get_classMethods_doc,
     "The attributes of this field are the class methods of this object. This can\n"
     "be used to force access to a class method.");
-static PyObject*
-cls_get_classMethods(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable cls_get_classMethods(PyObject* self, void* _Nullable closure
+                                                __attribute__((__unused__)))
 {
     return PyObjCMethodAccessor_New(self, 1);
 }
@@ -1855,14 +1997,14 @@ PyDoc_STRVAR(
     cls_get_instanceMethods_doc,
     "The attributes of this field are the instance methods of this object. This \n"
     "can be used to force access to an instance method.");
-static PyObject*
-cls_get_instanceMethods(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable cls_get_instanceMethods(PyObject* self, void* _Nullable closure
+                                                   __attribute__((__unused__)))
 {
     return PyObjCMethodAccessor_New(self, 0);
 }
 
-static PyObject*
-cls_get__name__(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable cls_get__name__(PyObject* self, void* _Nullable closure
+                                           __attribute__((__unused__)))
 {
     Class cls = PyObjCClass_GetClass(self);
     if (cls == NULL) {
@@ -1879,8 +2021,8 @@ cls_get__name__(PyObject* self, void* closure __attribute__((__unused__)))
 }
 
 PyDoc_STRVAR(cls_version_doc, "get/set the version of a class");
-static PyObject*
-cls_get_version(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable cls_get_version(PyObject* self, void* _Nullable closure
+                                           __attribute__((__unused__)))
 {
     Class cls = PyObjCClass_GetClass(self);
     if (cls == NULL) {
@@ -1892,8 +2034,8 @@ cls_get_version(PyObject* self, void* closure __attribute__((__unused__)))
 }
 
 static int
-cls_set_version(PyObject* self, PyObject* newVal,
-                void* closure __attribute__((__unused__)))
+cls_set_version(PyObject* self, PyObject* _Nullable newVal,
+                void* _Nullable closure __attribute__((__unused__)))
 {
     Class cls = PyObjCClass_GetClass(self);
     int   val;
@@ -1914,14 +2056,16 @@ cls_set_version(PyObject* self, PyObject* newVal,
 }
 
 static PyObject*
-cls_get_useKVO(PyObject* self, void* closure __attribute__((__unused__)))
+cls_get_useKVO(PyObject* self, void* _Nullable closure __attribute__((__unused__)))
 {
-    return PyBool_FromLong(((PyObjCClassObject*)self)->useKVO);
+    PyObject* result = ((PyObjCClassObject*)self)->useKVO ? Py_True : Py_False;
+    Py_INCREF(result);
+    return result;
 }
 
 static int
-cls_set_useKVO(PyObject* self, PyObject* newVal,
-               void* closure __attribute__((__unused__)))
+cls_set_useKVO(PyObject* self, PyObject* _Nullable newVal,
+               void* _Nullable closure __attribute__((__unused__)))
 {
     if (newVal == NULL) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete __useKVO__ attribute");
@@ -1932,15 +2076,17 @@ cls_set_useKVO(PyObject* self, PyObject* newVal,
     return 0;
 }
 
-static PyObject*
-cls_get_final(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable cls_get_final(PyObject* self, void* _Nullable closure
+                                         __attribute__((__unused__)))
 {
-    return PyBool_FromLong(((PyObjCClassObject*)self)->isFinal);
+    PyObject* result = ((PyObjCClassObject*)self)->isFinal ? Py_True : Py_False;
+    Py_INCREF(result);
+    return result;
 }
 
 static int
-cls_set_final(PyObject* self, PyObject* newVal,
-               void* closure __attribute__((__unused__)))
+cls_set_final(PyObject* self, PyObject* _Nullable newVal,
+              void* _Nullable closure __attribute__((__unused__)))
 {
     if (newVal == NULL) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete __objc_final__ attribute");
@@ -1991,8 +2137,7 @@ static PyGetSetDef class_getset[] = {
         .name = NULL /* SENTINEL */
     }};
 
-static PyObject*
-meth_dir(PyObject* self)
+static PyObject* _Nullable meth_dir(PyObject* self)
 {
     PyObject*    result;
     Class        cls;
@@ -2010,7 +2155,11 @@ meth_dir(PyObject* self)
 
     while (cls != NULL) {
         /* Now add all method names */
-        methods = class_copyMethodList(cls, &method_count);
+
+        /* class_copyMethodList only returns NULL when it sets method_count
+         * to 0
+         */
+        methods = (Method* _Nonnull)class_copyMethodList(cls, &method_count);
         for (i = 0; i < method_count; i++) {
             char*     name;
             PyObject* item;
@@ -2023,8 +2172,11 @@ meth_dir(PyObject* self)
 
             name = (char*)PyObjC_SELToPythonName(method_getName(methods[i]), selbuf,
                                                  sizeof(selbuf));
-            if (name == NULL)
+            if (name == NULL) {
+                /* Ignore methods whose SEL cannot be converted */
+                PyErr_Clear();
                 continue;
+            }
 
             item = PyUnicode_FromString(name);
             if (item == NULL) {
@@ -2052,6 +2204,12 @@ static PyMethodDef metaclass_methods[] = {{.ml_name  = "__dir__",
                                            .ml_meth  = (PyCFunction)metaclass_dir,
                                            .ml_flags = METH_NOARGS,
                                            .ml_doc   = "dir() hook, don't call directly"},
+#if PY_VERSION_HEX > 0x03090000
+                                          {.ml_name  = "__class_getitem__",
+                                           .ml_meth  = (PyCFunction)Py_GenericAlias,
+                                           .ml_flags = METH_O,
+                                           .ml_doc   = "See PEP 585"},
+#endif
                                           {
                                               .ml_name = NULL /* SENTINEL */
                                           }};
@@ -2060,13 +2218,14 @@ static PyMethodDef class_methods[] = {{.ml_name  = "__dir__",
                                        .ml_meth  = (PyCFunction)meth_dir,
                                        .ml_flags = METH_NOARGS,
                                        .ml_doc   = "dir() hook, don't call directly"},
+
                                       {
                                           .ml_name = NULL /* SENTINEL */
                                       }};
 
 /*
  * This is the class for type(NSObject), and is a subclass of type()
- * with an overridden tp_getattro that is used to dynamicly look up
+ * with an overridden tp_getattro that is used to dynamically look up
  * class methods.
  */
 PyTypeObject PyObjCMetaClass_Type = {
@@ -2109,8 +2268,7 @@ PyTypeObject PyObjCClass_Type = {
  *
  * Returns a new reference.
  */
-PyObject*
-PyObjCClass_New(Class objc_class)
+PyObject* _Nullable PyObjCClass_New(Class objc_class)
 {
     PyObject*          args;
     PyObject*          dict;
@@ -2156,7 +2314,14 @@ PyObjCClass_New(Class objc_class)
         PyTuple_SET_ITEM(bases, 0, (PyObject*)&PyObjCObject_Type);
         Py_INCREF(((PyObject*)&PyObjCObject_Type));
     } else {
-        PyTuple_SET_ITEM(bases, 0, PyObjCClass_New(class_getSuperclass(objc_class)));
+        PyObject* super_class =
+            PyObjCClass_New((Class _Nonnull)class_getSuperclass(objc_class));
+        if (super_class == NULL) {
+            Py_DECREF(hiddenSelectors);
+            Py_DECREF(bases);
+            return NULL;
+        }
+        PyTuple_SET_ITEM(bases, 0, super_class);
     }
     args      = PyTuple_New(3);
     className = class_getName(objc_class);
@@ -2232,8 +2397,7 @@ PyObjCClass_New(Class objc_class)
     return result;
 }
 
-PyObject*
-PyObjCClass_ListProperties(PyObject* aClass)
+PyObject* _Nullable PyObjCClass_ListProperties(PyObject* aClass)
 {
     Class     cls   = Nil;
     Protocol* proto = nil;
@@ -2294,6 +2458,9 @@ PyObjCClass_ListProperties(PyObject* aClass)
         const char* attr = property_getAttributes(props[i]);
         const char* e;
 
+        if (!attr)
+            continue;
+
         item = Py_BuildValue("{sssy}", "name", name, "raw_attr", attr);
         if (item == NULL) {
             goto error;
@@ -2458,14 +2625,20 @@ error:
     return NULL;
 }
 
-Class
-PyObjCClass_GetClass(PyObject* cls)
+Class _Nullable PyObjCClass_GetClass(PyObject* cls)
 {
     if (PyObjCClass_Check(cls)) {
+        /* XXX: This may return Nil without setting an error, check all callers */
         return ((PyObjCClassObject*)cls)->class;
 
     } else if (PyObjCMetaClass_Check(cls)) {
-        return objc_metaclass_locate(cls);
+        Class result = objc_metaclass_locate(cls);
+        if (result == Nil) {
+            PyErr_Format(PyObjCExc_InternalError, "Cannot find class for meta class %R",
+                         cls);
+            return Nil;
+        }
+        return result;
 
     } else {
         PyErr_Format(PyObjCExc_InternalError,
@@ -2475,8 +2648,8 @@ PyObjCClass_GetClass(PyObject* cls)
     }
 }
 
-PyObject*
-PyObjCClass_FindSelector(PyObject* cls, SEL selector, BOOL class_method)
+PyObject* _Nullable PyObjCClass_FindSelector(PyObject* cls, SEL selector,
+                                             BOOL class_method)
 {
     PyObjCClassObject* info;
     PyObject*          result;
@@ -2567,8 +2740,10 @@ PyObjCClass_FindSelector(PyObject* cls, SEL selector, BOOL class_method)
             char*     name;
             PyObject* py_name;
             name = PyObjC_SELToPythonName(selector, selbuf, sizeof(selbuf));
-            if (!name)
+            if (!name) {
+                PyErr_Clear();
                 continue;
+            }
 
             py_name = PyUnicode_FromString(name);
             if (!py_name) {
@@ -2609,8 +2784,7 @@ PyObjCClass_DictOffset(PyObject* cls)
     return ((PyObjCClassObject*)cls)->dictoffset;
 }
 
-PyObject*
-PyObjCClass_GetDelMethod(PyObject* cls)
+PyObject* _Nullable PyObjCClass_GetDelMethod(PyObject* cls)
 {
     PyObjCClassObject* info;
     info = (PyObjCClassObject*)cls;
@@ -2646,7 +2820,7 @@ PyObjCClass_HasPythonImplementation(PyObject* cls)
  *     registered) and then updates the class __dict__.
  *
  *     NOTE: We change the __dict__ using a setattro function because the type
- *     doesn't notice the existance of new special methods otherwise.
+ *     doesn't notice the existence of new special methods otherwise.
  *
  *     NOTE2: We use PyType_Type.tp_setattro instead of PyObject_SetAttr because
  *     the tp_setattro of Objective-C class wrappers does not allow some of
@@ -2656,7 +2830,6 @@ static int
 update_convenience_methods(PyObject* cls)
 {
     PyObject*  res;
-    PyObject*  args;
     PyObject*  dict;
     PyObject*  k;
     PyObject*  v;
@@ -2675,19 +2848,11 @@ update_convenience_methods(PyObject* cls)
         return -1;
     }
 
-    args = PyTuple_New(2);
-    if (args == NULL) {
-        Py_DECREF(dict);
-        return -1;
-    }
+    PyObject* args[3] = {NULL, cls, dict};
 
-    PyTuple_SET_ITEM(args, 0, cls);
-    PyTuple_SET_ITEM(args, 1, dict);
-    Py_INCREF(cls);
-
-    res = PyObject_Call(PyObjC_ClassExtender, args, NULL);
+    res = PyObject_Vectorcall(PyObjC_ClassExtender, args + 1,
+                              2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
     if (res == NULL) {
-        Py_DECREF(args);
         return -1;
     }
     Py_DECREF(res);
@@ -2716,18 +2881,20 @@ update_convenience_methods(PyObject* cls)
         }
     }
 
-    Py_DECREF(args);
-
+    Py_DECREF(dict);
     return 0;
 }
 
-PyObject*
-PyObjCClass_ClassForMetaClass(PyObject* meta)
+PyObject* _Nullable PyObjCClass_ClassForMetaClass(PyObject* meta)
 {
     if (meta == NULL)
         return NULL;
 
-    return PyObjCClass_New(objc_metaclass_locate(meta));
+    Class real_class = objc_metaclass_locate(meta);
+    if (real_class == Nil) {
+        return NULL;
+    }
+    return PyObjCClass_New(real_class);
 }
 
 int
@@ -2820,11 +2987,16 @@ PyObjCClass_AddMethods(PyObject* classObject, PyObject** methods, Py_ssize_t met
             goto cleanup_and_return_error;
         }
 
-        objcMethod->imp = PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)aMethod);
+        IMP imp = PyObjCFFI_MakeIMPForPyObjCSelector((PyObjCSelector*)aMethod);
+        if (imp == NULL) {
+            goto cleanup_and_return_error;
+        }
+        objcMethod->imp = imp;
 
         name = PyObject_GetAttrString(aMethod, "__name__");
 
         if (PyBytes_Check(name)) {
+            /* XXX: Why support bytes here? */
             PyObject* t =
                 PyUnicode_Decode(PyBytes_AsString(name), PyBytes_Size(name), NULL, NULL);
             if (t == NULL) {
@@ -2874,12 +3046,15 @@ PyObjCClass_AddMethods(PyObject* classObject, PyObject** methods, Py_ssize_t met
     }
 
     PyMem_Free(methodsToAdd);
+    methodsToAdd = NULL;
     if (curClassMethodIndex != 0) {
-        PyObjC_class_addMethodList(object_getClass(targetClass), classMethodsToAdd,
-                            (unsigned)curClassMethodIndex);
+        /* object_getClass will only return Nil if its argument is nil */
+        PyObjC_class_addMethodList((Class _Nonnull)object_getClass(targetClass),
+                                   classMethodsToAdd, (unsigned)curClassMethodIndex);
     }
 
     PyMem_Free(classMethodsToAdd);
+    classMethodsToAdd = NULL;
 
     r = PyDict_Merge(((PyTypeObject*)classObject)->tp_dict, extraDict, 1);
     if (r == -1)
@@ -2899,9 +3074,15 @@ PyObjCClass_AddMethods(PyObject* classObject, PyObject** methods, Py_ssize_t met
 cleanup_and_return_error:
     Py_XDECREF(metaDict);
     Py_XDECREF(extraDict);
-    if (methodsToAdd)
+    if (methodsToAdd) {
+        /* XXX: This should also clear entries */
         PyMem_Free(methodsToAdd);
-    if (classMethodsToAdd)
+    }
+    if (classMethodsToAdd) {
+        /* XXX: This should also clear entries */
         PyMem_Free(classMethodsToAdd);
+    }
     return -1;
 }
+
+NS_ASSUME_NONNULL_END
