@@ -9,8 +9,10 @@
 #include <objc/Object.h>
 #include <stddef.h>
 
-static PyObject*
-object_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObject* kwds)
+NS_ASSUME_NONNULL_BEGIN
+
+static PyObject* _Nullable object_new(PyTypeObject* type __attribute__((__unused__)),
+                                      PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
     static char* keywords[] = {"cobject", "c_void_p", NULL};
     PyObject*    cobject    = NULL;
@@ -31,7 +33,7 @@ object_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObj
             return NULL;
         }
 
-        return PyObjC_IdToPython(p);
+        return id_to_python(p);
 
     } else if (c_void_p != NULL) {
         NSObject* p;
@@ -61,7 +63,7 @@ object_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObj
         }
 
         Py_DECREF(attrval);
-        return PyObjC_IdToPython(p);
+        return id_to_python(p);
 
     } else {
         PyErr_SetString(PyExc_TypeError,
@@ -70,8 +72,7 @@ object_new(PyTypeObject* type __attribute__((__unused__)), PyObject* args, PyObj
     }
 }
 
-static PyObject*
-object_repr(PyObject* _self)
+static PyObject* _Nullable object_repr(PyObject* _self)
 {
     PyObjCObject* self = (PyObjCObject*)_self;
     PyObject*     res;
@@ -89,7 +90,9 @@ object_repr(PyObject* _self)
          * Don't call 'description' for uninitialized objects, that
          * is undefined behaviour and will crash the interpreter sometimes.
          */
-        res = PyObject_CallMethod((PyObject*)self, "description", NULL);
+        PyObject* args[2] = {NULL, (PyObject*)self};
+        res               = PyObject_VectorcallMethod(PyObjCNM_description, args + 1,
+                                                      1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         if (res == NULL) {
             PyErr_Clear();
         } else {
@@ -155,6 +158,7 @@ object_dealloc(PyObject* obj)
         } else {
             Py_BEGIN_ALLOW_THREADS
                 @try {
+                    /* XXX: Both branches are the same */
                     if (((PyObjCObject*)obj)->flags & PyObjCObject_kCFOBJECT) {
                         CFRelease(((PyObjCObject*)obj)->objc_object);
                     } else {
@@ -165,8 +169,8 @@ object_dealloc(PyObject* obj)
                     NSLog(@"PyObjC: Exception during dealloc of proxy: %@",
                           localException);
                 }
-            Py_END_ALLOW_THREADS((PyObjCObject*)obj)
-            ->objc_object = nil;
+            Py_END_ALLOW_THREADS;
+            ((PyObjCObject*)obj)->objc_object = nil;
         }
     }
 
@@ -206,7 +210,10 @@ object_verify_type(PyObject* obj)
          *
          * (The same can happen when other code perform ISA swiffling)
          */
-        PyTypeObject* tp = (PyTypeObject*)PyObjCClass_New(object_getClass(obj_inst));
+
+        /* object_getClass only returns Nil if its argument is nil */
+        PyTypeObject* tp =
+            (PyTypeObject*)PyObjCClass_New((Class _Nonnull)object_getClass(obj_inst));
 
         if (tp != Py_TYPE(obj)) {
             PyTypeObject* tmp;
@@ -217,8 +224,21 @@ object_verify_type(PyObject* obj)
                 return -1;
             }
 
-            tmp          = Py_TYPE(obj);
-            Py_TYPE(obj) = tp;
+            /* XXX: The correct way to do this is calling ``setattr(obj, "__class__",
+             * tp)`` Investigate the impact of that. Performance shouldn't be an issue
+             * here, a isa change should not happen often.
+             *
+             * XXX: the setattr call above works from Python, which is confusing
+             * behaviour (it changes the class of the proxy, not the ISA). Change
+             * object_setattr to either change the ObjC class as well, or to avoid
+             * changing the class. The latter is safer, as we cannot properly check if the
+             * old and new types are compatible.
+             *
+             * XXX: Probably need to call PyObject_Type.tp_setattro(...) to avoid hitting
+             * calling object_setattro in this file.
+             */
+            tmp = Py_TYPE(obj);
+            Py_SET_TYPE(obj, tp);
             Py_INCREF(tp);
             Py_DECREF(tmp);
 
@@ -244,10 +264,13 @@ object_verify_not_nil(PyObject* obj, PyObject* name)
     return 0;
 }
 
-PyObject*
-PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel)
+PyObject* _Nullable PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name,
+                                                   SEL sel)
 {
-    Class     cls  = PyObjCClass_GetClass(base);
+    Class cls = PyObjCClass_GetClass(base);
+    if (cls == NULL) {
+        return NULL;
+    }
     PyObject* dict = ((PyTypeObject*)base)->tp_dict;
     Method    m    = class_getInstanceMethod(cls, sel);
     if (m) {
@@ -265,8 +288,12 @@ PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel)
 #endif
 
         /* Create (unbound) selector */
-        PyObject* result =
-            PyObjCSelector_NewNative(cls, sel, method_getTypeEncoding(m), 0);
+        const char* encoding = method_getTypeEncoding(m);
+        if (encoding == NULL) {
+            PyErr_SetString(PyObjCExc_Error, "Native selector with Nil type encoding");
+            return NULL;
+        }
+        PyObject* result = PyObjCSelector_NewNative(cls, sel, encoding, 0);
         if (result == NULL) {
             return NULL;
         }
@@ -284,8 +311,7 @@ PyObjCClass_TryResolveSelector(PyObject* base, PyObject* name, SEL sel)
     return NULL;
 }
 
-static PyObject**
-_get_dictptr(PyObject* obj)
+static PyObject* _Nullable* _Nullable _get_dictptr(PyObject* obj)
 {
     Py_ssize_t dictoffset;
     id         obj_object;
@@ -297,14 +323,15 @@ _get_dictptr(PyObject* obj)
     return (PyObject**)(((char*)obj_object) + dictoffset);
 }
 
-static inline PyObject*
-_type_lookup(PyTypeObject* tp, PyObject* name)
+static inline PyObject* _Nullable _type_lookup(PyTypeObject* tp, PyObject* name,
+                                               const char* name_bytes)
 {
     Py_ssize_t i, n;
     PyObject * mro, *base, *dict;
-    PyObject*  descr = NULL;
+    PyObject*  first_class = NULL;
+    PyObject*  descr       = NULL;
     PyObject*  res;
-    SEL        sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
+    SEL        sel = PyObjCSelector_DefaultSelector(name_bytes);
 
     /* Look in tp_dict of types in MRO */
     mro = tp->tp_mro;
@@ -319,6 +346,20 @@ _type_lookup(PyTypeObject* tp, PyObject* name)
         base = PyTuple_GET_ITEM(mro, i);
 
         if (PyObjCClass_Check(base)) {
+            if (i == 0) {
+                first_class = base;
+            }
+            PyObject* cache = PyObjCClass_GetLookupCache((PyTypeObject*)base);
+            if (cache != NULL) {
+                descr = PyDict_GetItemWithError(cache, name);
+                if (descr == NULL && PyErr_Occurred()) {
+                    return NULL;
+                }
+                if (descr != NULL) {
+                    break;
+                }
+            }
+
             if (PyObjCClass_CheckMethodList(base, 0) < 0) {
                 return NULL;
             }
@@ -338,6 +379,12 @@ _type_lookup(PyTypeObject* tp, PyObject* name)
         if (descr == NULL && PyErr_Occurred()) {
             return NULL;
         } else if (descr != NULL) {
+            if (first_class != NULL) {
+                if (PyObjCClass_AddToLookupCache((PyTypeObject*)first_class, name, descr)
+                    == -1) {
+                    PyErr_Clear();
+                }
+            }
             break;
         }
 
@@ -361,8 +408,9 @@ _type_lookup(PyTypeObject* tp, PyObject* name)
     return descr;
 }
 
-static inline PyObject*
-_type_lookup_harder(PyTypeObject* tp, PyObject* name)
+/* XXX: Consider passing in name_bytes as well */
+static inline PyObject* _Nullable _type_lookup_harder(PyTypeObject* tp, PyObject* name,
+                                                      const char* name_bytes)
 {
     Py_ssize_t i, n;
     PyObject * mro, *base;
@@ -392,8 +440,14 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name)
         }
 
         cls = PyObjCClass_GetClass(base);
+        if (cls == NULL) {
+            return NULL;
+        }
 
-        methods = class_copyMethodList(cls, &method_count);
+        /* class_copyMethodList will only return NULL when it also
+         * sets method_count to 0.
+         */
+        methods = (Method* _Nonnull)class_copyMethodList(cls, &method_count);
         for (j = 0; j < method_count; j++) {
             Method m = methods[j];
             if (PyObjCClass_HiddenSelector(base, method_getName(m), NO)) {
@@ -402,14 +456,29 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name)
 
             sel_name =
                 (char*)PyObjC_SELToPythonName(method_getName(m), selbuf, sizeof(selbuf));
-            if (sel_name == NULL)
+            if (sel_name == NULL) {
+                PyErr_Clear();
                 continue;
+            }
 
-            if (strcmp(sel_name, PyObjC_Unicode_Fast_Bytes(name)) == 0) {
+            if (strcmp(sel_name, name_bytes) == 0) {
+                const char* encoding  = method_getTypeEncoding(m);
+                SEL         meth_name = method_getName(m);
+
+                if (encoding == NULL) {
+                    free(methods);
+                    PyErr_SetString(PyObjCExc_Error,
+                                    "Native selector with Nil type encoding");
+                    return NULL;
+                }
+                if (meth_name == NULL) {
+                    free(methods);
+                    PyErr_SetString(PyObjCExc_Error, "Native selector with Nil selector");
+                    return NULL;
+                }
 
                 /* Create (unbound) selector */
-                descr = PyObjCSelector_NewNative(cls, method_getName(m),
-                                                 method_getTypeEncoding(m), 0);
+                descr = PyObjCSelector_NewNative(cls, meth_name, encoding, 0);
                 free(methods);
                 if (descr == NULL) {
                     return NULL;
@@ -432,8 +501,7 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name)
     return descr;
 }
 
-static PyObject*
-object_getattro(PyObject* obj, PyObject* name)
+static PyObject* _Nullable object_getattro(PyObject* obj, PyObject* name)
 {
     PyTypeObject* tp    = NULL;
     PyObject*     descr = NULL;
@@ -442,26 +510,16 @@ object_getattro(PyObject* obj, PyObject* name)
     PyObject**    dictptr;
     const char*   namestr;
 
-    if (name == NULL) {
-        PyErr_SetString(PyExc_TypeError, "<nil> name");
-        return NULL;
-    }
+    PyObjC_Assert(name != NULL, NULL);
 
-    if (PyUnicode_Check(name)) {
-        if (PyObjC_Unicode_Fast_Bytes(name) == NULL)
-            return NULL;
-
-    } else {
+    if (!PyUnicode_Check(name)) {
         PyErr_Format(PyExc_TypeError, "attribute name must be string, got %s",
                      Py_TYPE(name)->tp_name);
         return NULL;
     }
 
-    namestr = PyObjC_Unicode_Fast_Bytes(name);
+    namestr = PyUnicode_AsUTF8(name);
     if (namestr == NULL) {
-        if (!PyErr_Occurred()) {
-            PyErr_SetString(PyExc_ValueError, "Empty name");
-        }
         return NULL;
     }
 
@@ -481,7 +539,7 @@ object_getattro(PyObject* obj, PyObject* name)
 
     /* replace _PyType_Lookup */
     if (descr == NULL) {
-        descr = _type_lookup(tp, name);
+        descr = _type_lookup(tp, name, namestr);
         if (descr == NULL && PyErr_Occurred()) {
             return NULL;
         }
@@ -500,7 +558,7 @@ object_getattro(PyObject* obj, PyObject* name)
         }
     }
 
-    if (strcmp(PyObjC_Unicode_Fast_Bytes(name), "__del__") == 0) {
+    if (PyObjC_is_ascii_string(name, "__del__")) {
 
         res = PyObjCClass_GetDelMethod((PyObject*)Py_TYPE(obj));
         goto done;
@@ -512,7 +570,7 @@ object_getattro(PyObject* obj, PyObject* name)
     if (dictptr != NULL) {
         PyObject* dict;
 
-        if (strcmp(PyObjC_Unicode_Fast_Bytes(name), "__dict__") == 0) {
+        if (PyObjC_is_ascii_string(name, "__dict__")) {
 
             res = *dictptr;
             if (res == NULL) {
@@ -549,7 +607,7 @@ object_getattro(PyObject* obj, PyObject* name)
          * for a method where the selector does not conform to the
          * naming convention that _type_lookup expects.
          */
-        descr = _type_lookup_harder(tp, name);
+        descr = _type_lookup_harder(tp, name, namestr);
 
         if (descr != NULL) {
             f = Py_TYPE(descr)->tp_descr_get;
@@ -597,7 +655,7 @@ done:
 }
 
 static int
-object_setattro(PyObject* obj, PyObject* name, PyObject* value)
+object_setattro(PyObject* obj, PyObject* name, PyObject* _Nullable value)
 {
     PyTypeObject* tp = Py_TYPE(obj);
     PyObject*     descr;
@@ -606,16 +664,17 @@ object_setattro(PyObject* obj, PyObject* name, PyObject* value)
     int           res;
     id            obj_inst;
     NSString*     obj_name;
+    const char*   namestr;
 
-    if (PyUnicode_Check(name)) {
-        if (PyObjC_Unicode_Fast_Bytes(name) == NULL)
-            return -1;
-
-    } else {
+    if (!PyUnicode_Check(name)) {
         PyErr_Format(PyExc_TypeError, "attribute name must be string, got %s",
                      Py_TYPE(name)->tp_name);
         return -1;
     }
+    namestr = PyObjC_Unicode_Fast_Bytes(name);
+    if (namestr == NULL) {
+        return -1;
+    }
 
     if (object_verify_not_nil(obj, name) == -1) {
         return -1;
@@ -641,7 +700,7 @@ object_setattro(PyObject* obj, PyObject* name, PyObject* value)
             }
         }
     }
-    descr = _type_lookup(tp, name);
+    descr = _type_lookup(tp, name, namestr);
     if (descr == NULL && PyErr_Occurred()) {
         return -1;
     }
@@ -716,18 +775,22 @@ done:
 
 PyDoc_STRVAR(objc_get_real_class_doc, "Return the current ISA of the object");
 
-static PyObject*
-objc_get_real_class(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable objc_get_real_class(PyObject* self,
+                                               void* closure __attribute__((__unused__)))
 {
     id        obj_object;
     PyObject* ret;
 
     obj_object = PyObjCObject_GetObject(self);
     PyObjC_Assert(obj_object != nil, NULL);
-    ret = PyObjCClass_New(object_getClass(obj_object));
+    /* object_getClass only return Nil if its argument is nil */
+    ret = PyObjCClass_New((Class _Nonnull)object_getClass(obj_object));
+    if (ret == NULL) {
+        return NULL;
+    }
     if (ret != (PyObject*)Py_TYPE(self)) {
         Py_DECREF(Py_TYPE(self));
-        Py_TYPE(self) = (PyTypeObject*)ret;
+        Py_SET_TYPE(self, (PyTypeObject*)ret);
         Py_INCREF(ret);
     }
     return ret;
@@ -738,21 +801,21 @@ PyDoc_STRVAR(
     "The attributes of this field are the instance methods of this object. This\n"
     "can be used to force access to an instance method.");
 
-static PyObject*
-obj_get_instanceMethods(PyObject* _self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable obj_get_instanceMethods(PyObject* _self, void* closure
+                                                   __attribute__((__unused__)))
 {
     PyObjCObject* self = (PyObjCObject*)_self;
     return PyObjCMethodAccessor_New((PyObject*)self, 0);
 }
 
-static PyObject*
-obj_get_flags(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable obj_get_flags(PyObject* self,
+                                         void*     closure __attribute__((__unused__)))
 {
-    return Py_BuildValue("I", PyObjCObject_GetFlags(self));
+    return PyLong_FromUnsignedLong(PyObjCObject_GetFlags(self));
 }
 
-static PyObject*
-obj_get_blocksignature(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable obj_get_blocksignature(PyObject* self, void* closure
+                                                  __attribute__((__unused__)))
 {
     if (PyObjCObject_IsBlock(self)) {
         PyObject* v = (PyObject*)PyObjCObject_GetBlock(self);
@@ -762,7 +825,7 @@ obj_get_blocksignature(PyObject* self, void* closure __attribute__((__unused__))
         } else {
             const char* typestr = PyObjCBlock_GetSignature(PyObjCObject_GetObject(self));
             if (typestr != NULL) {
-                v = (PyObject*)PyObjCMethodSignature_FromSignature(typestr, YES);
+                v = (PyObject*)PyObjCMethodSignature_WithMetaData(typestr, NULL, YES);
                 if (v == NULL) {
                     return NULL;
                 }
@@ -778,8 +841,8 @@ obj_get_blocksignature(PyObject* self, void* closure __attribute__((__unused__))
 }
 
 static int
-obj_set_blocksignature(PyObject* self, PyObject* newVal,
-                       void* closure __attribute__((__unused__)))
+obj_set_blocksignature(PyObject* self, PyObject* _Nullable newVal,
+                       void*     closure __attribute__((__unused__)))
 {
     if (newVal == NULL) {
         PyErr_SetString(PyExc_TypeError, "Cannot delete '__block_signature__'");
@@ -837,15 +900,13 @@ static PyGetSetDef obj_getset[] = {{
  * classes that doesn't work for us (it will write incomplete values to the
  * pickle). This method forces a failure during pickling.
  */
-static PyObject*
-meth_reduce(PyObject* self __attribute__((__unused__)))
+static PyObject* _Nullable meth_reduce(PyObject* self __attribute__((__unused__)))
 {
     PyErr_SetString(PyExc_TypeError, "Cannot pickle Objective-C objects");
     return NULL;
 }
 
-static PyObject*
-meth_sizeof(PyObject* self __attribute__((__unused__)))
+static PyObject* _Nullable meth_sizeof(PyObject* self __attribute__((__unused__)))
 {
     /* Basis __sizeof__ implementation for Cocoa objects.
      * This doesn't return the correct size for collections (such as NSArray),
@@ -864,15 +925,20 @@ meth_sizeof(PyObject* self __attribute__((__unused__)))
 static PyObject*
 meth_is_magic(PyObject* self)
 {
+    int is_magic;
+
     if (PyObjCObject_GetObject(self) == nil) {
-        return PyBool_FromLong(0);
+        is_magic = 0;
     } else {
-        return PyBool_FromLong(PyObjCObject_IsMagic(self));
+        is_magic = PyObjCObject_IsMagic(self);
     }
+
+    PyObject* result = is_magic ? Py_True : Py_False;
+    Py_INCREF(result);
+    return result;
 }
 
-static PyObject*
-as_cobject(PyObject* self)
+static PyObject* _Nullable as_cobject(PyObject* self)
 {
     if (PyObjCObject_GetObject(self) == nil) {
         Py_INCREF(Py_None);
@@ -882,14 +948,14 @@ as_cobject(PyObject* self)
     return PyCapsule_New(PyObjCObject_GetObject(self), "objc.__object__", NULL);
 }
 
-PyObject*
-PyObjC_get_c_void_p(void)
+/* XXX: Move this function to objc_util.m */
+PyObject* _Nullable PyObjC_get_c_void_p(void)
 {
     static PyObject* c_void_p = NULL;
     if (c_void_p == NULL) {
         PyObject* mod_ctypes = PyImport_ImportModule("ctypes");
         if (mod_ctypes == NULL) {
-            /* ctypes is nota available */
+            /* ctypes is not available */
             return NULL;
         }
 
@@ -903,29 +969,15 @@ PyObjC_get_c_void_p(void)
     return c_void_p;
 }
 
-static PyObject*
-as_ctypes_voidp(PyObject* self)
+static PyObject* _Nullable as_ctypes_voidp(PyObject* self)
 {
-    PyObject* c_void_p;
-
-    if (PyObjCObject_GetObject(self) == nil) {
-        Py_INCREF(Py_None);
-        return Py_None;
-    }
-
-    c_void_p = PyObjC_get_c_void_p();
-    if (c_void_p == NULL) {
-        return NULL;
-    }
-
-    return PyObject_CallFunction(c_void_p, "k", (long)PyObjCObject_GetObject(self));
+    return PyObjC_MakeCVoidP(PyObjCObject_GetObject(self));
 }
 
 /*
  * Implementation of __dir__, which is the hook used by dir() on python 2.6 or later.
  */
-static PyObject*
-meth_dir(PyObject* self)
+static PyObject* _Nullable meth_dir(PyObject* self)
 {
     PyObject*    result;
     Class        cls;
@@ -947,21 +999,31 @@ meth_dir(PyObject* self)
     cls = object_getClass(PyObjCObject_GetObject(self));
     while (cls != NULL) {
         /* Now add all instance method names */
-        methods = class_copyMethodList(cls, &method_count);
+
+        /* class_copyMethodList will only return NULL when it sets
+         * method_count to 0
+         */
+        methods = (Method* _Nonnull)class_copyMethodList(cls, &method_count);
         for (i = 0; i < method_count; i++) {
             char*     name;
             PyObject* item;
+            SEL       sel;
 
             /* Check if the selector should be hidden */
-            if (PyObjCClass_HiddenSelector((PyObject*)Py_TYPE(self),
-                                           method_getName(methods[i]), NO)) {
+            sel = method_getName(methods[i]);
+            if (sel == NULL)
+                continue;
+
+            if (PyObjCClass_HiddenSelector((PyObject*)Py_TYPE(self), sel, NO)) {
                 continue;
             }
 
             name = (char*)PyObjC_SELToPythonName(method_getName(methods[i]), selbuf,
                                                  sizeof(selbuf));
-            if (name == NULL)
+            if (name == NULL) {
+                PyErr_Clear();
                 continue;
+            }
 
             item = PyUnicode_FromString(name);
             if (item == NULL) {
@@ -1041,14 +1103,14 @@ PyObjCClassObject PyObjCObject_Type = {
  *  Allocate a proxy object for use during the call of __del__,
  *  this isn't a full-featured proxy object.
  */
-PyObject*
-_PyObjCObject_NewDeallocHelper(id objc_object)
+PyObject* _Nullable _PyObjCObject_NewDeallocHelper(id objc_object)
 {
     PyObject*     res;
     PyTypeObject* cls_type;
 
     PyObjC_Assert(objc_object != nil, NULL);
-    cls_type = (PyTypeObject*)PyObjCClass_New(object_getClass(objc_object));
+    cls_type =
+        (PyTypeObject*)PyObjCClass_New((Class _Nonnull)object_getClass(objc_object));
     if (cls_type == NULL) {
         return NULL;
     }
@@ -1086,18 +1148,20 @@ _PyObjCObject_FreeDeallocHelper(PyObject* obj)
         PyErr_Warn(PyObjCExc_ObjCRevivalWarning, buf);
 
         id objc_object = PyObjCObject_GetObject(obj);
+        if (objc_object != nil) {
 
-        if (((PyObjCObject*)obj)->flags & PyObjCObject_kSHOULD_NOT_RELEASE) {
-            /* pass */
+            if (((PyObjCObject*)obj)->flags & PyObjCObject_kSHOULD_NOT_RELEASE) {
+                /* pass */
 
-        } else if (((PyObjCObject*)obj)->flags & PyObjCObject_kUNINITIALIZED) {
-            /* pass */
+            } else if (((PyObjCObject*)obj)->flags & PyObjCObject_kUNINITIALIZED) {
+                /* pass */
 
-        } else {
-            CFRelease(objc_object);
+            } else {
+                CFRelease(objc_object);
+            }
+
+            PyObjC_UnregisterPythonProxy(objc_object, obj);
         }
-
-        PyObjC_UnregisterPythonProxy(objc_object, obj);
         ((PyObjCObject*)obj)->objc_object = nil;
 
         Py_DECREF(obj);
@@ -1107,10 +1171,12 @@ _PyObjCObject_FreeDeallocHelper(PyObject* obj)
     Py_DECREF(obj);
 }
 
-PyObject*
-PyObjCObject_New(id objc_object, int flags, int retain)
+PyObject* _Nullable PyObjCObject_New(id objc_object, int flags, int retain)
 {
-    Class         cls = object_getClass(objc_object);
+    PyObjC_Assert(objc_object != nil, NULL);
+
+    /* object_getClass only returns Nil if its argument is nil */
+    Class         cls = (Class _Nonnull)object_getClass(objc_object);
     PyTypeObject* cls_type;
     PyObject*     res;
 
@@ -1118,10 +1184,9 @@ PyObjCObject_New(id objc_object, int flags, int retain)
     if (res)
         return res;
 
-    PyObjC_Assert(objc_object != nil, NULL);
-
     cls_type = (PyTypeObject*)PyObjCClass_New(cls);
     if (cls_type == NULL) {
+        PyErr_SetString(PyObjCExc_Error, "Found method without selector in runtime");
         return NULL;
     }
 
@@ -1168,8 +1233,7 @@ PyObjCObject_New(id objc_object, int flags, int retain)
     return res;
 }
 
-PyObject*
-PyObjCObject_FindSelector(PyObject* object, SEL selector)
+PyObject* _Nullable PyObjCObject_FindSelector(PyObject* object, SEL selector)
 {
     PyObject* meth;
 
@@ -1183,7 +1247,7 @@ PyObjCObject_FindSelector(PyObject* object, SEL selector)
     }
 }
 
-id(PyObjCObject_GetObject)(PyObject* object)
+id _Nullable(PyObjCObject_GetObject)(PyObject* object)
 {
     if (!PyObjCObject_Check(object)) {
         PyErr_Format(PyExc_TypeError, "'objc.objc_object' expected, got '%s'",
@@ -1204,14 +1268,12 @@ PyObjCObject_ClearObject(PyObject* object)
     ((PyObjCObject*)object)->objc_object = nil;
 }
 
-PyObject*
-PyObjCObject_GetAttr(PyObject* obj, PyObject* name)
+PyObject* _Nullable PyObjCObject_GetAttr(PyObject* obj, PyObject* name)
 {
     return object_getattro(obj, name);
 }
 
-PyObject*
-PyObjCObject_GetAttrString(PyObject* obj, char* name)
+PyObject* _Nullable PyObjCObject_GetAttrString(PyObject* obj, char* name)
 {
     PyObject* pyname = PyUnicode_FromString(name);
     if (pyname == NULL)
@@ -1221,3 +1283,5 @@ PyObjCObject_GetAttrString(PyObject* obj, char* name)
     Py_DECREF(pyname);
     return rv;
 }
+
+NS_ASSUME_NONNULL_END
