@@ -10,8 +10,9 @@
 
 #include "pyobjc.h"
 
-#ifndef Py_HAVE_LOCAL_LOOKUP
+NS_ASSUME_NONNULL_BEGIN
 
+#ifndef Py_HAVE_LOCAL_LOOKUP
 
 /*
  * NOTE: This is a minor tweak of Python 2.5's super_getattro and is a rather
@@ -22,73 +23,50 @@
 typedef struct {
     PyObject_HEAD
 
-    PyTypeObject *type;
-    PyObject *obj;
-    PyTypeObject *obj_type;
+    PyTypeObject* type;
+    PyObject*     obj;
+    PyTypeObject* obj_type;
 } superobject;
 
-static PyObject *
-super_getattro(PyObject *self, PyObject *name)
+static PyObject* _Nullable super_getattro(PyObject* self, PyObject* name)
 {
-    superobject *su = (superobject *)self;
-    int skip = su->obj_type == NULL;
-    SEL sel;
+    superobject* su   = (superobject*)self;
+    int          skip = su->obj_type == NULL;
+    SEL          sel;
 
     if (!skip) {
         /* We want __class__ to return the class of the super object
          * (i.e. super, or a subclass), not the class of su->obj.
          */
         if (PyUnicode_Check(name)) {
-            skip = (PyUnicode_GET_SIZE(name) && PyObjC_is_ascii_string(name, "__class__"));
+            skip = PyObjC_is_ascii_string(name, "__class__");
 
-#if PY_MAJOR_VERSION == 2
-        } else if (PyString_Check(name)) {
-            skip = (
-                PyString_GET_SIZE(name) == 9 &&
-                strcmp(PyString_AS_STRING(name), "__class__") == 0);
-#endif
         } else {
             skip = 0;
         }
-
     }
 
     if (PyUnicode_Check(name)) {
-#ifdef PyObjC_FAST_UNICODE_ASCII
         const char* b = PyObjC_Unicode_Fast_Bytes(name);
-        if (name == NULL) {
+        if (b == NULL) {
             return NULL;
         }
 
         sel = PyObjCSelector_DefaultSelector(b);
 
-#else /* !PyObjC_FAST_UNICODE_ASCII */
-
-        PyObject* bytes = PyUnicode_AsEncodedString(name, NULL, NULL);
-        if (bytes == NULL) {
-            return NULL;
-        }
-        sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(bytes));
-#endif /* !PyObjC_FAST_UNICODE_ASCII */
-
-#if PY_MAJOR_VERSION == 2
-    } else if (PyBytes_Check(name)) {
-        sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name));
-#endif
-
     } else if (!skip) {
         PyErr_SetString(PyExc_TypeError, "attribute name is not a string");
         return NULL;
     }
 
     if (!skip) {
-        PyObject *mro, *res, *tmp, *dict;
-        PyTypeObject *starttype;
-        descrgetfunc f;
-        Py_ssize_t i, n;
+        PyObject *    mro, *res, *tmp, *dict;
+        PyTypeObject* starttype;
+        descrgetfunc  f;
+        Py_ssize_t    i, n;
 
         starttype = su->obj_type;
-        mro = starttype->tp_mro;
+        mro       = starttype->tp_mro;
 
         if (mro == NULL) {
             n = 0;
@@ -99,7 +77,7 @@ super_getattro(PyObject *self, PyObject *name)
         }
 
         for (i = 0; i < n; i++) {
-            if ((PyObject *)(su->type) == PyTuple_GET_ITEM(mro, i))
+            if ((PyObject*)(su->type) == PyTuple_GET_ITEM(mro, i))
                 break;
         }
 
@@ -122,51 +100,39 @@ super_getattro(PyObject *self, PyObject *name)
                 dict = Py_TYPE(tmp)->tp_dict;
 
             } else if (PyType_Check(tmp)) {
-                dict = ((PyTypeObject *)tmp)->tp_dict;
-
-#if PY_MAJOR_VERSION == 2
-            } else if (PyClass_Check(tmp)) {
-                dict = ((PyClassObject *)tmp)->cl_dict;
-#endif
+                dict = ((PyTypeObject*)tmp)->tp_dict;
 
             } else {
                 continue;
             }
 
-#if PY_MAJOR_VERSION == 3
             res = PyDict_GetItemWithError(dict, name);
             if (res == NULL && PyErr_Occurred()) {
                 return NULL;
-            }
-#else
-            res = PyDict_GetItem(dict, name);
-#endif
-            if (res != NULL) {
+            } else if (res != NULL) {
                 Py_INCREF(res);
                 f = Py_TYPE(res)->tp_descr_get;
                 if (f != NULL) {
-                    tmp = f(res,
+                    tmp = f(
+                        res,
                         /* Only pass 'obj' param if
                            this is instance-mode super
                            (See SF ID #743627)
                         */
-                        (su->obj == (PyObject *)
-                                su->obj_type
-                            ? (PyObject *)NULL
-                            : su->obj),
-                        (PyObject *)starttype);
+                        (su->obj == (PyObject*)su->obj_type ? (PyObject*)NULL : su->obj),
+                        (PyObject*)starttype);
                     Py_DECREF(res);
                     res = tmp;
                 }
                 return res;
             }
 
-
             if (PyObjCClass_Check(tmp)) {
                 if (!PyObjCClass_Check(su->obj)) {
                     res = PyObjCClass_TryResolveSelector(tmp, name, sel);
                 } else {
-                    res = PyObjCMetaClass_TryResolveSelector((PyObject*)Py_TYPE(tmp), name, sel);
+                    res = PyObjCMetaClass_TryResolveSelector((PyObject*)Py_TYPE(tmp),
+                                                             name, sel);
                 }
 
                 if (res) {
@@ -174,15 +140,13 @@ super_getattro(PyObject *self, PyObject *name)
                     f = Py_TYPE(res)->tp_descr_get;
                     if (f != NULL) {
                         tmp = f(res,
-                            /* Only pass 'obj' param if
-                               this is instance-mode super
-                               (See SF ID #743627)
-                            */
-                            (su->obj == (PyObject *)
-                                    su->obj_type
-                                ? (PyObject *)NULL
-                                : su->obj),
-                            (PyObject *)starttype);
+                                /* Only pass 'obj' param if
+                                   this is instance-mode super
+                                   (See SF ID #743627)
+                                */
+                                (su->obj == (PyObject*)su->obj_type ? (PyObject*)NULL
+                                                                    : su->obj),
+                                (PyObject*)starttype);
                         Py_DECREF(res);
                         res = tmp;
                     }
@@ -199,17 +163,17 @@ super_getattro(PyObject *self, PyObject *name)
 }
 
 PyTypeObject PyObjCSuper_Type = {
-    PyVarObject_HEAD_INIT(&PyType_Type, 0)
-    .tp_name        = "objc.super",
-    .tp_basicsize   = sizeof(superobject),
-    .tp_itemsize    = 0,
-    .tp_getattro    = super_getattro,
-    .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
-    .tp_alloc       = PyType_GenericAlloc,
-    .tp_new         = PyType_GenericNew,
-    .tp_free        = PyObject_GC_Del,
-    .tp_base        = &PySuper_Type,
+    PyVarObject_HEAD_INIT(&PyType_Type, 0).tp_name = "objc.super",
+    .tp_basicsize                                  = sizeof(superobject),
+    .tp_itemsize                                   = 0,
+    .tp_getattro                                   = super_getattro,
+    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
+    .tp_alloc = PyType_GenericAlloc,
+    .tp_new   = PyType_GenericNew,
+    .tp_free  = PyObject_GC_Del,
+    .tp_base  = &PySuper_Type,
 };
 
-
 #endif /* !Py_HAVE_LOCAL_LOOKUP */
+
+NS_ASSUME_NONNULL_END
