@@ -4,18 +4,32 @@
 
 #include "pyobjc.h"
 
-#import <Foundation/Foundation.h>
+NS_ASSUME_NONNULL_BEGIN
 
-PyObject* PyObjCExc_Error = NULL;
-PyObject* PyObjCExc_NoSuchClassError = NULL;
-PyObject* PyObjCExc_InternalError = NULL;
-PyObject* PyObjCExc_UnInitDeallocWarning = NULL;
-PyObject* PyObjCExc_ObjCRevivalWarning = NULL;
-PyObject* PyObjCExc_LockError = NULL;
-PyObject* PyObjCExc_BadPrototypeError = NULL;
-PyObject* PyObjCExc_UnknownPointerError = NULL;
-PyObject* PyObjCExc_DeprecationWarning = NULL;
-PyObject* PyObjCExc_ObjCPointerWarning = NULL;
+PyObject* PyObjCExc_Error;
+PyObject* PyObjCExc_NoSuchClassError;
+PyObject* PyObjCExc_InternalError;
+PyObject* PyObjCExc_UnInitDeallocWarning;
+PyObject* PyObjCExc_ObjCRevivalWarning;
+PyObject* PyObjCExc_LockError;
+PyObject* PyObjCExc_BadPrototypeError;
+PyObject* PyObjCExc_UnknownPointerError;
+PyObject* PyObjCExc_DeprecationWarning;
+PyObject* PyObjCExc_ObjCPointerWarning;
+
+PyObject* PyObjCNM_insert;
+PyObject* PyObjCNM_append;
+PyObject* PyObjCNM_strftime;
+PyObject* PyObjCNM_keys;
+PyObject* PyObjCNM_clear;
+PyObject* PyObjCNM_discard;
+PyObject* PyObjCNM_add;
+PyObject* PyObjCNM_values;
+PyObject* PyObjCNM_description;
+PyObject* PyObjCNM___get__;
+PyObject* PyObjCNM_date_format_string;
+PyObject* PyObjCNM_objc_memview_object;
+PyObject* PyObjCNM_objc_NULL;
 
 int
 PyObjCUtil_Init(PyObject* module)
@@ -42,13 +56,38 @@ PyObjCUtil_Init(PyObject* module)
     NEW_EXC(PyObjCExc_ObjCPointerWarning, "ObjCPointerWarning", PyExc_Warning);
 
 #undef NEW_EXC
+
+#define NEW_STR(identifier, strvalue)                                                    \
+    identifier = PyUnicode_InternFromString(strvalue);                                   \
+    if (identifier == NULL) {                                                            \
+        return -1;                                                                       \
+    }
+
+    NEW_STR(PyObjCNM_insert, "insert");
+    NEW_STR(PyObjCNM_append, "append");
+    NEW_STR(PyObjCNM_strftime, "strftime");
+    NEW_STR(PyObjCNM_keys, "keys");
+    NEW_STR(PyObjCNM_clear, "clear");
+    NEW_STR(PyObjCNM_discard, "discard");
+    NEW_STR(PyObjCNM_add, "add");
+    NEW_STR(PyObjCNM_values, "values");
+    NEW_STR(PyObjCNM_description, "description");
+    NEW_STR(PyObjCNM___get__, "__get__");
+    NEW_STR(PyObjCNM_date_format_string, "%Y-%m-%d %H:%M:%S %z");
+    NEW_STR(PyObjCNM_objc_memview_object, "objc.memview object");
+    NEW_STR(PyObjCNM_objc_NULL, "objc.NULL");
+
+#undef NEW_STR
+
     return 0;
 }
 
 static PyObject*
-ObjCErr_PyExcForName(const char* value)
+ObjCErr_PyExcForName(const char* _Nullable value)
 {
-    if (strcmp(value, "NSRangeException") == 0) {
+    if (value == NULL) {
+        return PyObjCExc_Error;
+    } else if (strcmp(value, "NSRangeException") == 0) {
         return PyExc_IndexError;
 
     } else if (strcmp(value, "NSInvalidArgumentException") == 0) {
@@ -68,16 +107,16 @@ void
 PyObjCErr_FromObjC(NSObject* localException)
 {
     NSDictionary* userInfo;
-    PyObject* dict;
-    PyObject* exception;
-    PyObject* v;
-    PyObject* buf;
-    PyObject* exc_type;
-    PyObject* exc_value;
-    PyObject* exc_traceback;
-    PyObject* c_localException_name;
-    PyObject* c_localException_reason;
-    NSObject* t;
+    PyObject*     dict;
+    PyObject*     exception;
+    PyObject*     v;
+    PyObject*     buf;
+    PyObject*     exc_type;
+    PyObject*     exc_value;
+    PyObject*     exc_traceback;
+    PyObject*     c_localException_name;
+    PyObject*     c_localException_reason;
+    NSObject*     t;
 
     PyObjC_BEGIN_WITH_GIL
         if (![localException isKindOfClass:[NSException class]]) {
@@ -90,7 +129,7 @@ PyObjCErr_FromObjC(NSObject* localException)
                 PyErr_NormalizeException(&exc_type, &exc_value, &exc_traceback);
             }
 
-            PyObject* exc = PyObjC_IdToPython(localException);
+            PyObject* exc = id_to_python(localException);
             if (exc == NULL) {
                 PyErr_Clear();
             } else {
@@ -127,13 +166,13 @@ PyObjCErr_FromObjC(NSObject* localException)
                 }
             }
 
-            t = [(NSException*)localException name];
+            t                     = [(NSException*)localException name];
             c_localException_name = pythonify_c_value(@encode(NSObject*), &t);
             if (c_localException_name == NULL) {
                 PyObjC_GIL_RETURNVOID;
             }
 
-            t = [(NSException*)localException reason];
+            t                       = [(NSException*)localException reason];
             c_localException_reason = pythonify_c_value(@encode(NSObject*), &t);
             if (c_localException_reason == NULL) {
                 Py_DECREF(c_localException_name);
@@ -185,6 +224,7 @@ PyObjCErr_FromObjC(NSObject* localException)
     PyObjC_END_WITH_GIL
 }
 
+/* XXX: To be removed */
 void
 PyObjCErr_ToObjC(void)
 {
@@ -194,13 +234,13 @@ PyObjCErr_ToObjC(void)
 NSException*
 PyObjCErr_AsExc(void)
 {
-    PyObject* exc_type;
-    PyObject* exc_value;
-    PyObject* exc_traceback;
-    PyObject* args;
-    PyObject* repr;
-    PyObject* typerepr;
-    NSException* val;
+    PyObject*            exc_type;
+    PyObject*            exc_value;
+    PyObject*            exc_traceback;
+    PyObject*            args;
+    PyObject*            repr;
+    PyObject*            typerepr;
+    NSException*         val;
     NSMutableDictionary* userInfo;
 
     PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
@@ -235,7 +275,7 @@ PyObjCErr_AsExc(void)
          */
         PyObject* v;
         NSString* reason = NULL;
-        NSString* name = NULL;
+        NSString* name   = NULL;
 
         /* NOTE: Don't use *WithError here because this function ignores errors */
         v = PyDict_GetItemString(args, "reason");
@@ -271,7 +311,7 @@ PyObjCErr_AsExc(void)
         }
     }
 
-    repr = PyObject_Str(exc_value);
+    repr     = PyObject_Str(exc_value);
     typerepr = PyObject_Str(exc_type);
     userInfo = [NSMutableDictionary dictionaryWithCapacity:3];
     [userInfo setObject:[[[OC_PythonObject alloc] initWithPyObject:exc_type] autorelease]
@@ -289,13 +329,25 @@ PyObjCErr_AsExc(void)
                      forKey:@"__pyobjc_exc_traceback__"];
     }
 
+    NSObject* oc_typerepr = nil;
+    NSObject* oc_repr     = nil;
+
+    if (typerepr) {
+        if (depythonify_python_object(typerepr, &oc_typerepr) == -1) {
+            /* Ignore errors in conversion */
+            PyErr_Clear();
+        }
+    }
+    if (repr) {
+        if (depythonify_python_object(repr, &oc_repr) == -1) {
+            /* Ignore errors in conversion */
+            PyErr_Clear();
+        }
+    }
+
     val = [NSException
         exceptionWithName:@"OC_PythonException"
-                   reason:[NSString
-                              stringWithFormat:@"%@: %@",
-                                               typerepr ? PyObjC_PythonToId(typerepr)
-                                                        : NULL,
-                                               repr ? PyObjC_PythonToId(repr) : NULL]
+                   reason:[NSString stringWithFormat:@"%@: %@", oc_typerepr, oc_repr]
                  userInfo:userInfo];
 
     Py_XDECREF(typerepr);
@@ -315,7 +367,7 @@ PyObjCErr_AsExc(void)
 }
 
 void
-PyObjCErr_ToObjCWithGILState(PyGILState_STATE* state)
+PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nullable state)
 {
     NSException* exc = PyObjCErr_AsExc();
     assert(exc != nil);
@@ -331,13 +383,12 @@ PyObjCErr_ToObjCWithGILState(PyGILState_STATE* state)
     abort();
 }
 
-char*
-PyObjCUtil_Strdup(const char* value)
+char* _Nullable PyObjCUtil_Strdup(const char* value)
 {
     Py_ssize_t len;
-    char* result;
+    char*      result;
 
-    len = strlen(value);
+    len    = strlen(value);
     result = PyMem_Malloc(len + 1);
     if (result == NULL)
         return NULL;
@@ -382,15 +433,14 @@ NSMapTableValueCallBacks PyObjCUtil_ObjCValueCallBacks = {
 #define SHOULD_IGNORE 2
 
 void
-PyObjC_FreeCArray(int code, void* array)
+PyObjC_FreeCArray(int code, Py_buffer* view)
 {
     if (code == SHOULD_FREE) {
-        PyMem_Free(array);
+        PyBuffer_Release(view);
     }
 }
 
-static inline PyTypeObject*
-fetch_array_type(void)
+static inline PyTypeObject* _Nullable fetch_array_type(void)
 {
     static PyTypeObject* array_type = NULL;
 
@@ -410,7 +460,7 @@ array_typestr(PyObject* array)
 {
     PyObject* typecode;
     PyObject* bytes;
-    char res;
+    char      res;
 
     typecode = PyObject_GetAttrString(array, "typecode");
     if (typecode == NULL) {
@@ -475,16 +525,6 @@ array_typestr(PyObject* array)
     return res;
 }
 
-static int
-buffer_get(BOOL writable, PyObject* obj, void** bufptr, Py_ssize_t* sizeptr)
-{
-    if (writable) {
-        return PyObject_AsWriteBuffer(obj, bufptr, sizeptr);
-    } else {
-        return PyObject_AsReadBuffer(obj, (const void**)bufptr, sizeptr);
-    }
-}
-
 static char struct_elem_code(const char* typestr);
 
 static char
@@ -503,7 +543,7 @@ array_elem_code(const char* typestr)
         return '\0';
     }
 
-    while (typestr && *typestr != _C_ARY_E) {
+    while (*typestr != _C_ARY_E) {
         switch (*typestr) {
         case _C_ARY_B:
             tmp = array_elem_code(typestr);
@@ -540,17 +580,24 @@ array_elem_code(const char* typestr)
             res = *typestr;
         }
 
-        typestr = PyObjCRT_SkipTypeSpec(typestr);
+        const char* next = PyObjCRT_SkipTypeSpec(typestr);
+        if (next == NULL) {
+            return '\0';
+        }
+        typestr = next;
     }
     return res;
 }
 
 static char
-struct_elem_code(const char* typestr)
+struct_elem_code(const char* start_typestr)
 {
     char res = '\0';
     char tmp;
 
+    PyObjC_Assert(start_typestr != NULL, '\0');
+
+    const char* _Nullable typestr = start_typestr;
     if (*typestr++ != _C_STRUCT_B) {
         return '\0';
     }
@@ -681,24 +728,25 @@ code_compatible(char array_code, char type_code)
  *
  * The pythonList should either be a python sequence with appropriate entries,
  * an array.array whose element-types match the element-types of the
- * 'elementType' or an appropriatly typed and shaped numeric array.
+ * 'elementType' or an appropriately typed and shaped numeric array.
  */
 int
 PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
-                      PyObject* pythonList, void** array, Py_ssize_t* size,
-                      PyObject** bufobj)
+                      PyObject* pythonList, void* _Nullable* _Nonnull array,
+                      Py_ssize_t* _Nullable size, PyObject* _Nullable* _Nonnull bufobj,
+                      Py_buffer* view)
 {
     Py_ssize_t eltsize = PyObjCRT_SizeOfType(elementType);
     Py_ssize_t i;
-    int r;
+    int        r;
 
     if (eltsize == -1) {
         return -1;
     }
 
-    if ((eltsize == 1 || eltsize == 0) &&
-        !(*elementType == _C_NSBOOL || *elementType == _C_BOOL ||
-          *elementType == _C_CHAR_AS_INT)) {
+    if ((eltsize == 1 || eltsize == 0)
+        && !(*elementType == _C_NSBOOL || *elementType == _C_BOOL
+             || *elementType == _C_CHAR_AS_INT)) {
         /* A simple byte-array */
 
         /* NOTE: PyUnicode is explicitly excluded because it
@@ -707,8 +755,6 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
          * you want.
          */
 
-        char* buf;
-        Py_ssize_t bufsize;
         int have_buffer;
 
         if (PyUnicode_Check(pythonList)) {
@@ -717,86 +763,95 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             return -1;
         }
 
-        have_buffer = buffer_get(writable, pythonList, (void**)&buf, &bufsize);
+        have_buffer = PyObject_GetBuffer(pythonList, view,
+                                         writable ? PyBUF_CONTIG : PyBUF_CONTIG_RO);
         if (have_buffer == -1) {
             if (writable) {
                 /* Ensure that the expected semantics still work
                  * when the passed in buffer is read-only
                  */
-                if (buffer_get(NO, pythonList, (void**)&buf, &bufsize) == -1) {
+                PyObject* byte_array;
+
+                if (PyObject_GetBuffer(pythonList, view, PyBUF_CONTIG_RO) == -1) {
                     return -1;
                 }
 
                 if (size == NULL || *size == -1) {
-                    *array = PyMem_Malloc(bufsize);
-                    if (*array == NULL) {
-                        return -1;
-                    }
-                    memcpy(*array, buf, bufsize);
-
+                    byte_array = PyByteArray_FromStringAndSize(view->buf, view->len);
                 } else {
-                    if ((exactSize && *size != bufsize) ||
-                        (!exactSize && *size > bufsize)) {
+                    if ((exactSize && *size != view->len)
+                        || (!exactSize && *size > view->len)) {
                         PyErr_Format(PyExc_ValueError,
                                      "Requesting buffer of %" PY_FORMAT_SIZE_T
                                      "d, have buffer "
                                      "of %" PY_FORMAT_SIZE_T "d",
-                                     *size, bufsize);
+                                     *size, view->len);
                         return -1;
                     }
 
-                    *array = PyMem_Malloc(*size);
-                    if (*array == NULL) {
-                        return -1;
-                    }
-
-                    memcpy(*array, buf, *size);
+                    byte_array = PyByteArray_FromStringAndSize(view->buf, *size);
                 }
+
+                PyBuffer_Release(view);
+
+                if (byte_array == NULL) {
+                    return -1;
+                }
+
+                if (PyObject_GetBuffer(byte_array, view, PyBUF_CONTIG) == -1) {
+                    Py_DECREF(byte_array);
+                    return -1;
+                }
+
+                Py_DECREF(byte_array); /* Reference is kept by the view */
+                *array = view->buf;
                 return SHOULD_FREE;
             }
         }
 
         if (have_buffer != -1) {
             if (size == NULL) {
-                *array = buf;
+                *array  = view->buf;
                 *bufobj = pythonList;
                 Py_INCREF(pythonList);
 
             } else if (*size == -1) {
-                *array = buf;
-                *size = bufsize;
+                *array  = view->buf;
+                *size   = view->len;
                 *bufobj = pythonList;
                 Py_INCREF(pythonList);
 
             } else {
-                if ((exactSize && *size != bufsize) || (!exactSize && *size > bufsize)) {
+                if ((exactSize && *size != view->len)
+                    || (!exactSize && *size > view->len)) {
                     PyErr_Format(PyExc_ValueError,
                                  "Requesting buffer of %" PY_FORMAT_SIZE_T
                                  "d, have buffer "
                                  "of %" PY_FORMAT_SIZE_T "d",
-                                 *size, bufsize);
+                                 *size, view->len);
                     return -1;
                 }
 
-                *array = buf;
+                *array  = view->buf;
                 *bufobj = pythonList;
                 Py_INCREF(pythonList);
             }
-            return SHOULD_IGNORE;
+            return SHOULD_FREE;
         }
 
         PyErr_Clear();
     }
 
     if (*elementType == _C_UNICHAR && PyUnicode_Check(pythonList)) {
+        PyObject* bytes_array;
 
-        *bufobj = _PyUnicode_EncodeUTF16(pythonList, NULL, -1);
+        bytes_array = _PyUnicode_EncodeUTF16(pythonList, NULL, -1);
 
-        if (*bufobj == NULL) {
+        if (bytes_array == NULL) {
             return -1;
         }
 
-        Py_ssize_t bufsize = PyBytes_Size(*bufobj) / 2;
+        Py_ssize_t bufsize = PyBytes_Size(bytes_array) / 2;
 
         if (*size == -1) {
             *size = bufsize;
@@ -813,17 +868,26 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             return -1;
         }
 
-        /* XXX: Update API protocol to make the extra copy not necessary
-         * Cannot use the code at the end because 'buffer' is assumed to be
-         * the value to return to the python caller.
-         */
-        *array = PyMem_Malloc(PyBytes_Size(*bufobj));
-        memcpy(*array, PyBytes_AsString(*bufobj), PyBytes_Size(*bufobj));
-        Py_DECREF(*bufobj);
-        *bufobj = NULL;
-        return SHOULD_FREE;
+        if (writable) {
+            PyObject* tmp = PyByteArray_FromObject(bytes_array);
+            Py_DECREF(bytes_array);
+            if (tmp == NULL) {
+                return -1;
+            }
 
-        /* *array = PyBytes_AsString(*bufobj); return SHOULD_IGNORE*/
+            bytes_array = tmp;
+        }
+
+        if (PyObject_GetBuffer(bytes_array, view,
+                               writable ? PyBUF_CONTIG : PyBUF_CONTIG_RO)
+            == -1) {
+            Py_DECREF(bytes_array);
+            return -1;
+        }
+
+        *array = view->buf;
+        Py_DECREF(bytes_array); /* Kept alive by view */
+        return SHOULD_FREE;
     }
 
     /* A more complex array */
@@ -853,8 +917,6 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
          * simple type of the same type as the array, or a struct/array
          * containing only elements of the type of the array.
          */
-        char* buf;
-        Py_ssize_t bufsize;
         char code = array_typestr(pythonList);
         if (code_compatible(code, *elementType)) {
             /* Simple array, ok */
@@ -892,7 +954,9 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             return -1;
         }
 
-        if (buffer_get(writable, pythonList, (void**)&buf, &bufsize) == -1) {
+        if (PyObject_GetBuffer(pythonList, view,
+                               writable ? PyBUF_CONTIG : PyBUF_CONTIG_RO)
+            == -1) {
             return -1;
         }
 
@@ -901,21 +965,21 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             return -1;
         }
 
-        if ((bufsize % eltsize) != 0) {
+        if ((view->len % eltsize) != 0) {
             PyErr_SetString(PyExc_ValueError, "Badly shaped array.array");
             return -1;
         }
 
-        *array = buf;
+        *array = view->buf;
 
         if (size == NULL) {
             /* pass */
 
         } else if (*size == -1) {
-            *size = bufsize / eltsize;
+            *size = view->len / eltsize;
 
         } else {
-            bufsize /= eltsize;
+            Py_ssize_t bufsize = view->len / eltsize;
 
             if ((exactSize && *size != bufsize) || (!exactSize && *size > bufsize)) {
                 PyErr_Format(PyExc_ValueError,
@@ -924,16 +988,16 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
                              *size, bufsize);
                 return -1;
             }
-            *array = buf;
+            *array = view->buf;
         }
         *bufobj = pythonList;
         Py_INCREF(pythonList);
-        return SHOULD_IGNORE;
+        return SHOULD_FREE;
 
     } else {
         Py_ssize_t seqlen;
         Py_ssize_t pycount;
-        PyObject* seq;
+        PyObject*  seq;
 
         if (*elementType == _C_NSBOOL) {
             if (PyBytes_Check(pythonList)) {
@@ -971,13 +1035,26 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             return -1;
         }
 
-        *array = PyMem_Malloc(eltsize * pycount);
-        if (*array == NULL) {
+        PyObject* bytes_array = PyByteArray_FromStringAndSize(NULL, 0);
+        if (bytes_array == NULL) {
             Py_DECREF(seq);
-            PyErr_NoMemory();
             return -1;
         }
 
+        if (PyByteArray_Resize(bytes_array, eltsize * pycount) == -1) {
+            Py_DECREF(bytes_array);
+            Py_DECREF(seq);
+            return -1;
+        }
+
+        if (PyObject_GetBuffer(bytes_array, view, PyBUF_CONTIG) == -1) {
+            Py_DECREF(bytes_array);
+            Py_DECREF(seq);
+            return -1;
+        }
+        *array = view->buf;
+        Py_DECREF(bytes_array); /* kept alive by view */
+
         if (size) {
             *size = pycount;
         }
@@ -990,7 +1067,7 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
             r = depythonify_c_value(elementType, item, ((char*)*array) + (i * eltsize));
             if (r == -1) {
                 Py_DECREF(seq);
-                PyMem_Free(*array);
+                PyBuffer_Release(view);
                 *array = NULL;
                 return -1;
             }
@@ -999,10 +1076,10 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
     }
 }
 
-PyObject*
-PyObjC_CArrayToPython(const char* elementType, void* array, Py_ssize_t size)
+PyObject* _Nullable PyObjC_CArrayToPython(const char* elementType, void* array,
+                                          Py_ssize_t size)
 {
-    PyObject* result;
+    PyObject*  result;
     Py_ssize_t i;
     Py_ssize_t eltsize;
 
@@ -1016,8 +1093,8 @@ PyObjC_CArrayToPython(const char* elementType, void* array, Py_ssize_t size)
             return PyBytes_FromStringAndSize(array, size);
         }
 
-        if (*elementType != _C_NSBOOL && *elementType != _C_BOOL &&
-            *elementType != _C_CHAR_AS_INT) {
+        if (*elementType != _C_NSBOOL && *elementType != _C_BOOL
+            && *elementType != _C_CHAR_AS_INT) {
             /* Special case for buffer-like objects */
             return PyBytes_FromStringAndSize(array, size);
         }
@@ -1025,7 +1102,7 @@ PyObjC_CArrayToPython(const char* elementType, void* array, Py_ssize_t size)
 
     if (*elementType == _C_UNICHAR) {
         int byteorder = 0;
-        result = PyUnicode_DecodeUTF16(array, size * 2, NULL, &byteorder);
+        result        = PyUnicode_DecodeUTF16(array, size * 2, NULL, &byteorder);
         return result;
     }
 
@@ -1056,7 +1133,7 @@ PyObjC_IsPythonKeyword(const char* word)
      * are actually used in Cocoa.
      */
     static const char* keywords[] = {"class", "raise", "from", NULL};
-    const char** cur;
+    const char**       cur;
 
     for (cur = keywords; *cur != NULL; cur++) {
         if (strcmp(word, *cur) == 0) {
@@ -1066,6 +1143,9 @@ PyObjC_IsPythonKeyword(const char* word)
     return 0;
 }
 
+/*
+ * XXX: This function may or may not raise an exception on error.
+ */
 int
 PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
 {
@@ -1073,11 +1153,14 @@ PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
     const char* end;
     const char* next;
 
-    cur = signature;
+    cur  = signature;
     *buf = '\0';
 
     while (*cur != '\0') {
         next = end = PyObjCRT_SkipTypeSpec(cur);
+        if (end == NULL) {
+            return -1;
+        }
         end -= 1;
         while (end != cur && isdigit(*end)) {
             end--;
@@ -1092,16 +1175,16 @@ PyObjCRT_SimplifySignature(const char* signature, char* buf, size_t buflen)
         buflen -= (end - cur);
         buf += (end - cur);
         *buf = '\0';
-        cur = next;
+        cur  = next;
     }
     return 0;
 }
 
-PyObject*
-PyObjC_CArrayToPython2(const char* elementType, void* array, Py_ssize_t size,
-                       bool alreadyRetained, bool alreadyCFRetained)
+PyObject* _Nullable PyObjC_CArrayToPython2(const char* elementType, void* array,
+                                           Py_ssize_t size, bool alreadyRetained,
+                                           bool alreadyCFRetained)
 {
-    PyObject* result;
+    PyObject*  result;
     Py_ssize_t i;
     Py_ssize_t eltsize;
 
@@ -1118,8 +1201,8 @@ PyObjC_CArrayToPython2(const char* elementType, void* array, Py_ssize_t size,
         if (*elementType == _C_CHAR_AS_TEXT) {
             return PyBytes_FromStringAndSize(array, size);
         }
-        if (*elementType != _C_NSBOOL && *elementType != _C_BOOL &&
-            *elementType != _C_CHAR_AS_INT) {
+        if (*elementType != _C_NSBOOL && *elementType != _C_BOOL
+            && *elementType != _C_CHAR_AS_INT) {
             /* Special case for buffer-like objects */
             return PyBytes_FromStringAndSize(array, size);
         }
@@ -1127,7 +1210,7 @@ PyObjC_CArrayToPython2(const char* elementType, void* array, Py_ssize_t size,
 
     if (*elementType == _C_UNICHAR) {
         int byteorder = 0;
-        result = PyUnicode_DecodeUTF16(array, size * 2, NULL, &byteorder);
+        result        = PyUnicode_DecodeUTF16(array, size * 2, NULL, &byteorder);
         return result;
     }
 
@@ -1211,23 +1294,22 @@ PyObjC_is_ascii_prefix(PyObject* unicode_string, const char* ascii_string, size_
     return strncmp((const char*)(PyUnicode_DATA(unicode_string)), ascii_string, n) == 0;
 }
 
-PyObject*
-PyObjC_ImportName(const char* name)
+PyObject* _Nullable PyObjC_ImportName(const char* name)
 {
     PyObject* py_name;
     PyObject* mod;
-    char* c = strrchr(name, '.');
+    char*     c = strrchr(name, '.');
 
     if (c == NULL) {
         /* Toplevel module */
         py_name = PyUnicode_FromString(name);
-        mod = PyImport_Import(py_name);
+        mod     = PyImport_Import(py_name);
         Py_DECREF(py_name);
         return mod;
 
     } else {
         py_name = PyUnicode_FromStringAndSize(name, c - name);
-        mod = PyImport_Import(py_name);
+        mod     = PyImport_Import(py_name);
         Py_DECREF(py_name);
         if (mod == NULL) {
             return NULL;
@@ -1239,12 +1321,16 @@ PyObjC_ImportName(const char* name)
     }
 }
 
-PyObject*
-PyObjC_AdjustSelf(PyObject* object)
+PyObject* _Nullable PyObjC_AdjustSelf(PyObject* object)
 {
-    if (PyType_Check(object) &&
-        PyType_IsSubtype((PyTypeObject*)object, &PyObjCClass_Type)) {
+    if (PyType_Check(object)
+        && PyType_IsSubtype((PyTypeObject*)object, &PyObjCClass_Type)) {
         PyObject* temp = PyObjCClass_ClassForMetaClass(object);
+        if (temp == NULL) {
+            Py_DECREF(object);
+            PyErr_Format(PyObjCExc_Error, "Cannot find class for metaclass %R", object);
+            return NULL;
+        }
         Py_INCREF(temp);
         Py_DECREF(object);
         return temp;
@@ -1257,7 +1343,7 @@ PyObjCRT_SignaturesEqual(const char* sig1, const char* sig2)
 {
     char buf1[1024];
     char buf2[1024];
-    int r;
+    int  r;
 
     /* Return 0 if the two signatures are not equal */
     if (strcmp(sig1, sig2) == 0)
@@ -1278,11 +1364,10 @@ PyObjCRT_SignaturesEqual(const char* sig1, const char* sig2)
     return strcmp(buf1, buf2) == 0;
 }
 
-PyObject*
-PyObjC_FindSELInDict(PyObject* clsdict, SEL selector)
+PyObject* _Nullable PyObjC_FindSELInDict(PyObject* clsdict, SEL selector)
 {
-    PyObject* values;
-    PyObject* seq;
+    PyObject*  values;
+    PyObject*  seq;
     Py_ssize_t i, len;
 
     values = PyDict_Values(clsdict);
@@ -1315,19 +1400,22 @@ PyObjC_FindSELInDict(PyObject* clsdict, SEL selector)
     return NULL;
 }
 
-char*
-PyObjC_SELToPythonName(SEL sel, char* buf, size_t buflen)
+char* _Nullable PyObjC_SELToPythonName(SEL sel, char* buf, size_t buflen)
 {
+    /* XXX: strXcpy instead */
     size_t res = snprintf(buf, buflen, "%s", sel_getName(sel));
-    char* cur;
+    char*  cur;
 
     if (res != strlen(sel_getName(sel))) {
+        PyErr_SetString(PyExc_RuntimeError, "selector too long to calculate python name");
         return NULL;
     }
 
     if (PyObjC_IsPythonKeyword(buf)) {
         res = snprintf(buf, buflen, "%s__", sel_getName(sel));
         if (res != 2 + strlen(sel_getName(sel))) {
+            PyErr_SetString(PyExc_RuntimeError,
+                            "selector too long to calculate python name");
             return NULL;
         }
         return buf;
@@ -1336,13 +1424,12 @@ PyObjC_SELToPythonName(SEL sel, char* buf, size_t buflen)
     cur = strchr(buf, ':');
     while (cur) {
         *cur = '_';
-        cur = strchr(cur, ':');
+        cur  = strchr(cur, ':');
     }
     return buf;
 }
 
-PyObject*
-PyObjCDict_GetItemStringWithError(PyObject* dict, const char* key)
+PyObject* _Nullable PyObjCDict_GetItemStringWithError(PyObject* dict, const char* key)
 {
     PyObject* result;
     PyObject* keystring = PyUnicode_FromString(key);
@@ -1355,3 +1442,79 @@ PyObjCDict_GetItemStringWithError(PyObject* dict, const char* key)
 
     return result;
 }
+
+int
+PyObjC_CheckArgCount(PyObject* callable, size_t min_args, size_t max_args, size_t nargsf)
+{
+    size_t nargs = PyVectorcall_NARGS(nargsf);
+    if (nargs < min_args || nargs > max_args) {
+        if (min_args == max_args) {
+            if (min_args == 0) {
+                PyErr_Format(PyExc_TypeError, "%R expected no arguments, got %zu",
+                             callable, nargs);
+            } else {
+                PyErr_Format(PyExc_TypeError, "%R expected %zu arguments, got %zu",
+                             callable, min_args, nargs);
+            }
+        } else {
+            PyErr_Format(PyExc_TypeError,
+                         "%R expected between %zu and %zu arguments, got %zu", callable,
+                         min_args, max_args, nargs);
+        }
+        return -1;
+    }
+    return 0;
+}
+
+int
+PyObjC_CheckNoKwnames(PyObject* callable, PyObject* _Nullable kwnames)
+{
+    if (kwnames == NULL)
+        return 0;
+    if (PyObject_Size(kwnames) == 0)
+        return 0;
+    if (PyErr_Occurred())
+        return -1;
+    PyErr_Format(PyExc_TypeError, "%R does not accept keyword arguments", callable);
+    return -1;
+}
+
+PyObject* _Nullable PyObjC_MakeCVoidP(void* _Nullable ptr)
+{
+    if (ptr == NULL) {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+    PyObject* c_void_p = PyObjC_get_c_void_p();
+    if (c_void_p == NULL) {
+        return NULL;
+    }
+
+    PyObject* pyptr = PyLong_FromVoidPtr(ptr);
+    if (pyptr == NULL) {
+        return NULL;
+    }
+    PyObject* args[2] = {NULL, pyptr};
+    PyObject* res =
+        PyObject_Vectorcall(c_void_p, args + 1, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+    Py_DECREF(pyptr);
+    return res;
+}
+
+PyObject* _Nullable PyObjC_CallCopyFunc(PyObject* arg)
+{
+    PyObject* args[2] = {NULL, arg};
+
+    return PyObject_Vectorcall(PyObjC_CopyFunc, args + 1,
+                               1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+}
+
+PyObject* _Nullable PyObjC_CallDecoder(PyObject* cdr, PyObject* setValue)
+{
+    PyObject* args[3] = {NULL, cdr, setValue};
+
+    return PyObject_Vectorcall(PyObjC_Decoder, args + 1,
+                               2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
+}
+
+NS_ASSUME_NONNULL_END
