@@ -1,7 +1,9 @@
 #include "pyobjc.h"
 
-static NSMapTable* python_proxies = NULL;
-static NSMapTable* objc_proxies = NULL;
+NS_ASSUME_NONNULL_BEGIN
+
+static NSMapTable* _Nonnull python_proxies;
+static NSMapTable* _Nonnull objc_proxies;
 
 /*
  * Iff true the python->objc proxy registry uses a zero-ing weakref
@@ -13,20 +15,17 @@ int PyObjC_weakref_proxy_registry = 0;
 int
 PyObjC_InitProxyRegistry(void)
 {
-    python_proxies = NSCreateMapTable(
-            PyObjCUtil_PointerKeyCallBacks,
-            PyObjCUtil_PointerValueCallBacks,
-            0);
+    python_proxies = NSCreateMapTable(PyObjCUtil_PointerKeyCallBacks,
+                                      PyObjCUtil_PointerValueCallBacks, 0);
 
     if (python_proxies == NULL) {
-        PyErr_SetString(PyExc_RuntimeError, "Cannot create NSMapTable for python_proxies");
+        PyErr_SetString(PyExc_RuntimeError,
+                        "Cannot create NSMapTable for python_proxies");
         return -1;
     }
 
-    objc_proxies = NSCreateMapTable(
-            PyObjCUtil_PointerKeyCallBacks,
-            PyObjCUtil_PointerValueCallBacks,
-            0);
+    objc_proxies = NSCreateMapTable(PyObjCUtil_PointerKeyCallBacks,
+                                    PyObjCUtil_PointerValueCallBacks, 0);
     if (objc_proxies == NULL) {
         PyErr_SetString(PyExc_RuntimeError, "Cannot create NSMapTable for objc_proxies");
         return -1;
@@ -53,7 +52,8 @@ PyObjC_UnregisterPythonProxy(id original, PyObject* proxy)
 {
     PyObject* v;
 
-    if (original == nil) return;
+    if (original == nil)
+        return;
 
     v = NSMapGet(python_proxies, original);
     if (v == proxy) {
@@ -66,7 +66,8 @@ PyObjC_UnregisterObjCProxy(PyObject* original, id proxy)
 {
     id v;
 
-    if (original == NULL) return;
+    if (original == NULL)
+        return;
 
     v = NSMapGet(objc_proxies, original);
     if (v == proxy) {
@@ -74,8 +75,7 @@ PyObjC_UnregisterObjCProxy(PyObject* original, id proxy)
     }
 }
 
-PyObject*
-PyObjC_FindPythonProxy(id original)
+PyObject* _Nullable PyObjC_FindPythonProxy(id original)
 {
     PyObject* v;
 
@@ -90,8 +90,7 @@ PyObjC_FindPythonProxy(id original)
     return v;
 }
 
-id
-PyObjC_FindObjCProxy(PyObject* original)
+id _Nullable PyObjC_FindObjCProxy(PyObject* original)
 {
     if (original == Py_None) {
         return nil;
@@ -101,8 +100,7 @@ PyObjC_FindObjCProxy(PyObject* original)
     }
 }
 
-id
-PyObjC_FindOrRegisterObjCProxy(PyObject* value, id proxy)
+id _Nullable PyObjC_FindOrRegisterObjCProxy(PyObject* value, id proxy)
 {
     id result = PyObjC_FindObjCProxy(value);
     if (result == NULL) {
@@ -115,3 +113,5 @@ PyObjC_FindOrRegisterObjCProxy(PyObject* value, id proxy)
         return result;
     }
 }
+
+NS_ASSUME_NONNULL_END
