@@ -28,15 +28,6 @@ python_to_id(PyObject* object)
     return result;
 }
 
-static PyObject*
-id_to_python(id object)
-{
-    PyObject* result;
-
-    result = pythonify_c_value(@encode(id), &object);
-    return result;
-}
-
 static Class
 sel_get_class(PyObject* sel)
 {
@@ -70,7 +61,8 @@ static void
 fill_super_cls(struct objc_super* super, Class cls, Class self)
 {
     objc_superSetReceiver(*super, self);
-    objc_superSetClass(*super, object_getClass(cls));
+    /* object_getClass will only return Nil if the argument is nil */
+    objc_superSetClass(*super, (Class _Nonnull)object_getClass(cls));
 }
 
 static int
@@ -84,7 +76,8 @@ struct pyobjc_api objc_api = {
     .api_version = PYOBJC_API_VERSION,
     .struct_len  = sizeof(struct pyobjc_api),
     .register_method_mapping =
-        (RegisterMethodMappingFunctionType*)PyObjC_RegisterMethodMapping,
+        /* Cast is needed because of ffi_cif */
+    (RegisterMethodMappingFunctionType*)PyObjC_RegisterMethodMapping,
     .obj_get_object           = PyObjCObject_GetObject,
     .cls_get_class            = PyObjCClass_GetClass,
     .python_to_id             = python_to_id,
@@ -118,6 +111,8 @@ struct pyobjc_api objc_api = {
     .memview_check             = PyObjCMemView_Check,
     .memview_new               = PyObjCMemView_New,
     .memview_getbuffer         = PyObjCMemView_GetBuffer,
+    .checkargcount             = PyObjC_CheckArgCount,
+    .checknokwnames            = PyObjC_CheckNoKwnames,
 };
 
 int
