@@ -6,36 +6,37 @@
  */
 #include "pyobjc.h"
 
+NS_ASSUME_NONNULL_BEGIN
+
 BOOL PyObjC_UpdatingMetaData = NO;
 
-PyObject*
-PyObjC_NewRegistry(void)
-{
-    return PyDict_New();
-}
+PyObject* _Nullable PyObjC_NewRegistry(void) { return PyDict_New(); }
 
 int
 PyObjC_AddToRegistry(PyObject* registry, PyObject* class_name, PyObject* selector,
                      PyObject* value)
 {
-    int result;
+    int       result;
     PyObject* sublist;
-    PyObject* item = Py_BuildValue("(OO)", class_name, value);
-    if (item == NULL) {
+
+    if (!PyUnicode_Check(class_name) && !PyBytes_Check(class_name)) {
+        PyErr_SetString(PyExc_TypeError, "class_name should be str or bytes");
+        return -1;
+    }
+    if (!PyBytes_Check(selector)) {
+        PyErr_SetString(PyExc_TypeError, "selector should be bytes");
         return -1;
     }
 
     sublist = PyDict_GetItemWithError(registry, selector);
     if (sublist == NULL && PyErr_Occurred()) {
-        Py_DECREF(item);
         return -1;
     }
     if (sublist == NULL) {
         sublist = PyList_New(0);
-        result = PyDict_SetItem(registry, selector, sublist);
+        result  = PyDict_SetItem(registry, selector, sublist);
         Py_DECREF(sublist);
         if (result == -1) {
-            Py_DECREF(item);
             return -1;
         }
     }
@@ -43,20 +44,45 @@ PyObjC_AddToRegistry(PyObject* registry, PyObject* class_name, PyObject* selecto
     if (!PyObjC_UpdatingMetaData) {
         PyObjC_MappingCount += 1;
     }
+
+    /*
+     * Check if there is a registration for *class_name* in
+     * *sublist*, if so replace that registration.
+     */
+    for (Py_ssize_t i = 0; i < PyList_GET_SIZE(sublist); i++) {
+        PyObject* item = PyList_GET_ITEM(sublist, i);
+
+        PyObjC_Assert(PyTuple_CheckExact(item), -1);
+        PyObjC_Assert(PyTuple_GET_SIZE(item) == 2, -1);
+
+        int r = PyObject_RichCompareBool(PyTuple_GET_ITEM(item, 0), class_name, Py_EQ);
+        if (r == -1)
+            return -1;
+        if (r) {
+            Py_DECREF(PyTuple_GET_ITEM(item, 1));
+            PyTuple_SET_ITEM(item, 1, value);
+            Py_INCREF(value);
+            return 0;
+        }
+    }
+
+    PyObject* item = Py_BuildValue("(OO)", class_name, value);
+    if (item == NULL) {
+        return -1;
+    }
     result = PyList_Append(sublist, item);
     Py_DECREF(item);
     return result;
 }
 
-PyObject*
-PyObjC_FindInRegistry(PyObject* registry, Class cls, SEL selector)
+PyObject* _Nullable PyObjC_FindInRegistry(PyObject* registry, Class cls, SEL selector)
 {
     Py_ssize_t i;
     Py_ssize_t len;
-    PyObject* cur;
-    Class found_class = nil;
-    PyObject* found_value = NULL;
-    PyObject* sublist;
+    PyObject*  cur;
+    Class      found_class = nil;
+    PyObject*  found_value = NULL;
+    PyObject*  sublist;
 
     if (registry == NULL) {
         return NULL;
@@ -79,11 +105,7 @@ PyObjC_FindInRegistry(PyObject* registry, Class cls, SEL selector)
             continue;
         }
 
-        if (!PyTuple_CheckExact(cur)) {
-            PyErr_SetString(PyObjCExc_InternalError,
-                            "Exception registry element isn't a tuple");
-            return NULL;
-        }
+        PyObjC_Assert(PyTuple_CheckExact(cur), NULL);
 
         PyObject* nm = PyTuple_GET_ITEM(cur, 0);
         if (PyUnicode_Check(nm)) {
@@ -106,8 +128,8 @@ PyObjC_FindInRegistry(PyObject* registry, Class cls, SEL selector)
             continue;
         }
 
-        if (!class_isSubclassOf(cls, cur_class) &&
-            !class_isSubclassOf(cls, object_getClass(cur_class))) {
+        if (!class_isSubclassOf(cls, cur_class)
+            && !class_isSubclassOf(cls, object_getClass(cur_class))) {
             continue;
         }
 
@@ -126,12 +148,12 @@ PyObjC_FindInRegistry(PyObject* registry, Class cls, SEL selector)
     return found_value;
 }
 
-PyObject*
-PyObjC_CopyRegistry(PyObject* registry, PyObjC_ItemTransform value_transform)
+PyObject* _Nullable PyObjC_CopyRegistry(PyObject*            registry,
+                                        PyObjC_ItemTransform value_transform)
 {
-    PyObject* result = PyDict_New();
-    PyObject* key;
-    PyObject* sublist;
+    PyObject*  result = PyDict_New();
+    PyObject*  key;
+    PyObject*  sublist;
     Py_ssize_t pos = 0;
     if (result == NULL) {
         return NULL;
@@ -139,8 +161,16 @@ PyObjC_CopyRegistry(PyObject* registry, PyObjC_ItemTransform value_transform)
 
     while (PyDict_Next(registry, &pos, &key, &sublist)) {
         Py_ssize_t i, len;
-        PyObject* sl_new;
-        len = PyList_Size(sublist);
+        PyObject*  sl_new;
+
+#ifdef PyObjC_DEBUG
+        if (!PyList_CheckExact(sublist)) {
+            PyErr_SetString(PyObjCExc_InternalError, "sublist of registry is not a list");
+            goto error;
+        }
+#endif
+
+        len    = PyList_GET_SIZE(sublist);
         sl_new = PyList_New(len);
         if (sl_new == NULL)
             goto error;
@@ -154,7 +184,7 @@ PyObjC_CopyRegistry(PyObject* registry, PyObjC_ItemTransform value_transform)
             PyObject* item;
             PyObject* new_item;
 
-            item = PyList_GET_ITEM(sublist, i);
+            item     = PyList_GET_ITEM(sublist, i);
             new_item = Py_BuildValue("(ON)", PyTuple_GET_ITEM(item, 0),
                                      value_transform(PyTuple_GET_ITEM(item, 1)));
             if (new_item == NULL)
@@ -170,3 +200,5 @@ error:
     Py_DECREF(result);
     return NULL;
 }
+
+NS_ASSUME_NONNULL_END
