@@ -6,12 +6,16 @@
  *
  * - Call the method from python
  * - Call the python implementation of a method from Objective-C
+ *
+ * XXX: Add API to dump the registry for inspection.
  */
 #include "pyobjc.h"
 
-struct registry
-{
-    PyObjC_CallFunc call_to_objc;
+NS_ASSUME_NONNULL_BEGIN
+
+/* XXX: Consider using a minimal python type instead of a capsule */
+struct registry {
+    PyObjC_CallFunc       call_to_objc;
     PyObjCFFI_ClosureFunc call_to_python;
 };
 
@@ -29,46 +33,43 @@ init_registry(void)
 {
     if (signature_registry == NULL) {
         signature_registry = PyDict_New();
-        if (signature_registry == NULL) return -1;
+        if (signature_registry == NULL)
+            return -1;
     }
 
     if (special_registry == NULL) {
         special_registry = PyDict_New();
-        if (special_registry == NULL) return -1;
+        if (special_registry == NULL)
+            return -1;
     }
 
     return 0;
 }
 
-#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 7
-
-static void memblock_capsule_cleanup(void* ptr)
+static void
+memblock_capsule_cleanup(PyObject* ptr)
 {
-    PyMem_Free(ptr);
+    void* mem = PyCapsule_GetPointer(ptr, "objc.__memblock__");
+
+#ifdef PyObjC_DEBUG
+    if (mem == NULL)
+        PyObjCErr_InternalError();
+#endif
+
+    PyMem_Free(mem);
 }
 
-#else /* Python 2.7 and 3.x */
-
-static void memblock_capsule_cleanup(PyObject* ptr)
-{
-    PyMem_Free(PyCapsule_GetPointer(ptr, "objc.__memblock__"));
-}
-
-#endif /* Python 2.7 and 3.x */
-
-
 /*
  * Add a custom mapping for a method in a class
  */
 int
-PyObjC_RegisterMethodMapping(Class class, SEL sel,
-    PyObjC_CallFunc call_to_objc,
-    PyObjCFFI_ClosureFunc call_to_python)
+PyObjC_RegisterMethodMapping(_Nullable Class class, SEL sel, PyObjC_CallFunc call_to_objc,
+                             PyObjCFFI_ClosureFunc call_to_python)
 {
     struct registry* v;
-    PyObject* pyclass;
-    PyObject* entry;
-    PyObject* lst;
+    PyObject*        pyclass;
+    PyObject*        entry;
+    PyObject*        lst;
 
     if (signature_registry == NULL) {
         if (init_registry() < 0) {
@@ -78,7 +79,7 @@ PyObjC_RegisterMethodMapping(Class class, SEL sel,
 
     if (!call_to_python) {
         PyErr_SetString(PyObjCExc_Error,
-            "PyObjC_RegisterMethodMapping: all functions required");
+                        "PyObjC_RegisterMethodMapping: all functions required");
         return -1;
     }
 
@@ -101,30 +102,29 @@ PyObjC_RegisterMethodMapping(Class class, SEL sel,
         PyErr_NoMemory();
         return -1;
     }
-    v->call_to_objc = call_to_objc;
+    v->call_to_objc   = call_to_objc;
     v->call_to_python = call_to_python;
 
     entry = PyTuple_New(2);
-    if (entry == NULL) return -1;
+    if (entry == NULL) {
+        PyMem_Free(v);
+        return -1;
+    }
 
     PyTuple_SET_ITEM(entry, 0, pyclass);
-    PyTuple_SET_ITEM(entry, 1, PyCapsule_New(v, "objc.__memblock__", memblock_capsule_cleanup));
+    PyTuple_SET_ITEM(entry, 1,
+                     PyCapsule_New(v, "objc.__memblock__", memblock_capsule_cleanup));
 
     if (PyTuple_GET_ITEM(entry, 1) == NULL) {
         Py_DECREF(entry);
         return -1;
     }
 
-#if PY_MAJOR_VERSION == 3
     lst = PyDict_GetItemStringWithError(special_registry, sel_getName(sel));
     if (lst == NULL && PyErr_Occurred()) {
         Py_DECREF(entry);
         return -1;
-    }
-#else
-    lst = PyDict_GetItemString(special_registry, sel_getName(sel));
-#endif
-    if (lst == NULL) {
+    } else if (lst == NULL) {
         lst = PyList_New(0);
         if (PyDict_SetItemString(special_registry, sel_getName(sel), lst) == -1) {
             Py_DECREF(lst);
@@ -148,26 +148,21 @@ PyObjC_RegisterMethodMapping(Class class, SEL sel,
     return 0;
 }
 
-
-
-int PyObjC_RegisterSignatureMapping(
-    char* signature,
-    PyObjC_CallFunc call_to_objc,
-    PyObjCFFI_ClosureFunc call_to_python)
+int
+PyObjC_RegisterSignatureMapping(char* signature, PyObjC_CallFunc call_to_objc,
+                                PyObjCFFI_ClosureFunc call_to_python)
 {
     struct registry* v;
-    PyObject* entry;
-    char signature_buf[1024];
-    int r;
+    PyObject*        entry;
+    char             signature_buf[1024];
+    int              r;
 
     if (signature_registry == NULL) {
-        if (init_registry() < 0) return -1;
+        if (init_registry() < 0)
+            return -1;
     }
 
-    r = PyObjCRT_SimplifySignature(
-            signature,
-            signature_buf,
-            sizeof(signature_buf));
+    r = PyObjCRT_SimplifySignature(signature, signature_buf, sizeof(signature_buf));
     if (r == -1) {
         PyErr_SetString(PyObjCExc_Error, "cannot simplify signature");
         return -1;
@@ -175,7 +170,7 @@ int PyObjC_RegisterSignatureMapping(
 
     if (!call_to_objc || !call_to_python) {
         PyErr_SetString(PyObjCExc_Error,
-           "PyObjC_RegisterSignatureMapping: all functions required");
+                        "PyObjC_RegisterSignatureMapping: all functions required");
         return -1;
     }
 
@@ -184,7 +179,7 @@ int PyObjC_RegisterSignatureMapping(
         PyErr_NoMemory();
         return -1;
     }
-    v->call_to_objc = call_to_objc;
+    v->call_to_objc   = call_to_objc;
     v->call_to_python = call_to_python;
 
     entry = PyCapsule_New(v, "objc.__memblock__", memblock_capsule_cleanup);
@@ -198,7 +193,7 @@ int PyObjC_RegisterSignatureMapping(
         return -1;
     }
 
-    if (PyDict_SetItem(signature_registry, key, entry) < 0){
+    if (PyDict_SetItem(signature_registry, key, entry) < 0) {
         Py_DECREF(key);
         Py_DECREF(entry);
         return -1;
@@ -210,27 +205,27 @@ int PyObjC_RegisterSignatureMapping(
     return 0;
 }
 
-
-static struct registry*
-search_special(Class class, SEL sel)
+/*
+ * May of may not raise an exception when the return value is NULL
+ */
+static struct registry* _Nullable search_special(Class class, SEL sel)
 {
-    PyObject* result = NULL;
-    PyObject* special_class = NULL;
-    PyObject* search_class = NULL;
-    PyObject* lst;
+    PyObject*  result        = NULL;
+    PyObject*  special_class = NULL;
+    PyObject*  search_class  = NULL;
+    PyObject*  lst;
     Py_ssize_t i;
 
-    if (special_registry == NULL) goto error;
-    if (!class) goto error;
+    if (special_registry == NULL)
+        goto error;
+    if (!class)
+        goto error;
 
     search_class = PyObjCClass_New(class);
-    if (search_class == NULL) goto error;
+    if (search_class == NULL)
+        goto error;
 
-#if PY_MAJOR_VERSION == 3
     lst = PyDict_GetItemStringWithError(special_registry, sel_getName(sel));
-#else
-    lst = PyDict_GetItemString(special_registry, sel_getName(sel));
-#endif
     if (lst == NULL) {
         goto error;
     }
@@ -247,56 +242,53 @@ search_special(Class class, SEL sel)
      *   was added later).
      */
     for (i = 0; i < PyList_GET_SIZE(lst); i++) {
-        PyObject* entry = PyList_GET_ITEM(lst, i);
+        PyObject* entry   = PyList_GET_ITEM(lst, i);
         PyObject* pyclass = PyTuple_GET_ITEM(entry, 0);
 
-        if (pyclass == NULL) continue;
-        if (pyclass != Py_None && !PyType_IsSubtype((PyTypeObject*)search_class, (PyTypeObject*)pyclass)) {
+        if (pyclass == NULL)
+            continue;
+        if (pyclass != Py_None
+            && !PyType_IsSubtype((PyTypeObject*)search_class, (PyTypeObject*)pyclass)) {
             continue;
         }
 
         if (!special_class) {
             /* No match yet, use */
             special_class = pyclass;
-            result = PyTuple_GET_ITEM(entry, 1);
+            result        = PyTuple_GET_ITEM(entry, 1);
 
         } else if (pyclass == Py_None) {
             /* Already have a match, Py_None is less specific */
             continue;
 
-        } else if (PyType_IsSubtype(
-                    (PyTypeObject*)special_class,
-                    (PyTypeObject*)pyclass
-                )) {
+        } else if (PyType_IsSubtype((PyTypeObject*)special_class,
+                                    (PyTypeObject*)pyclass)) {
             /* special_type is a superclass of search_class,
              * but a subclass of the current match, hence it is
              * a more specific match or a simular match later in the
              * list.
              */
             special_class = pyclass;
-            result = PyTuple_GET_ITEM(entry, 1);
-            }
+            result        = PyTuple_GET_ITEM(entry, 1);
+        }
     }
     Py_XDECREF(search_class);
-    if (!result) goto error;
+    if (!result)
+        goto error;
 
     return PyCapsule_GetPointer(result, "objc.__memblock__");
 
 error:
-    if (!PyErr_Occurred()) {
-        PyErr_Format(PyObjCExc_Error,
-            "PyObjC: don't know how to call method '%s'", sel_getName(sel));
-    }
     return NULL;
 }
 
-
 PyObjC_CallFunc
 PyObjC_FindCallFunc(Class class, SEL sel)
 {
     struct registry* special;
 
-    if (special_registry == NULL) return PyObjCFFI_Caller;
+    if (special_registry == NULL)
+        return PyObjCFFI_Caller;
 
     special = search_special(class, sel);
     if (special) {
@@ -308,57 +300,44 @@ PyObjC_FindCallFunc(Class class, SEL sel)
     return PyObjCFFI_Caller;
 }
 
-static struct registry*
-find_signature(const char* signature)
+static struct registry* _Nullable find_signature(const char* signature)
 {
     PyObject* o;
-    struct registry* r;
-    char signature_buf[1024];
-    int res;
+    char      signature_buf[1024];
+    int       res;
 
-    res = PyObjCRT_SimplifySignature(
-            signature,
-            signature_buf,
-            sizeof(signature_buf));
+    res = PyObjCRT_SimplifySignature(signature, signature_buf, sizeof(signature_buf));
     if (res == -1) {
         PyErr_SetString(PyObjCExc_Error, "cannot simplify signature");
         return NULL;
     }
 
-    if (signature_registry == NULL) goto error;
+    if (signature_registry == NULL)
+        goto error;
 
     PyObject* key = PyBytes_FromString(signature_buf);
     if (key == NULL) {
         return NULL;
     }
-#if PY_MAJOR_VERSION == 3
     o = PyDict_GetItemWithError(signature_registry, key);
-#else
-    o = PyDict_GetItem(signature_registry, key);
-#endif
     Py_DECREF(key);
-    if (o == NULL) goto error;
+    if (o == NULL)
+        goto error;
 
-    r = PyCapsule_GetPointer(o, "objc.__memblock__");
-    return r;
+    return PyCapsule_GetPointer(o, "objc.__memblock__");
 
 error:
-    if (!PyErr_Occurred()) {
-        PyErr_Format(PyObjCExc_Error,
-            "PyObjC: don't know how to call a method with "
-            "signature '%s'", signature);
-    }
     return NULL;
 }
 
 extern IMP
-PyObjC_MakeIMP(Class class, Class super_class, PyObject* sel, PyObject* imp)
+PyObjC_MakeIMP(Class class, Class _Nullable super_class, PyObject* sel, PyObject* imp)
 {
-    struct registry* generic;
-    struct registry* special;
-    SEL aSelector = PyObjCSelector_GetSelector(sel);
-    PyObjCFFI_ClosureFunc func = NULL;
-    IMP retval;
+    struct registry*       generic;
+    struct registry*       special;
+    SEL                    aSelector = PyObjCSelector_GetSelector(sel);
+    PyObjCFFI_ClosureFunc  func      = NULL;
+    IMP                    retval;
     PyObjCMethodSignature* methinfo;
 
     if (super_class != nil) {
@@ -371,25 +350,31 @@ PyObjC_MakeIMP(Class class, Class super_class, PyObject* sel, PyObject* imp)
     }
 
     if (func == NULL) {
-        generic = find_signature(PyObjCSelector_Signature(sel));
+        const char* sel_signature = PyObjCSelector_Signature(sel);
+        if (sel_signature == NULL) {
+            return NULL;
+        }
+        generic = find_signature(sel_signature);
         if (generic != NULL) {
             func = generic->call_to_python;
         }
     }
 
     if (func == PyObjCUnsupportedMethod_IMP) {
-        PyErr_Format(PyExc_TypeError,
-            "Implementing %s in Python is not supported",
-            sel_getName(aSelector));
+        PyErr_Format(PyExc_TypeError, "Implementing %s in Python is not supported",
+                     sel_getName(aSelector));
         return NULL;
     }
 
     if (func != NULL) {
+        const char* sel_signature = PyObjCSelector_Signature(sel);
+        if (sel_signature == NULL) {
+            return NULL;
+        }
         methinfo = PyObjCMethodSignature_ForSelector(
-                class, (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kCLASS_METHOD) != 0,
-                PyObjCSelector_GetSelector(sel),
-                PyObjCSelector_Signature(sel),
-                PyObjCNativeSelector_Check(sel));
+            class, (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kCLASS_METHOD) != 0,
+            PyObjCSelector_GetSelector(sel), sel_signature,
+            PyObjCNativeSelector_Check(sel));
         if (methinfo == NULL) {
             return NULL;
         }
@@ -401,67 +386,48 @@ PyObjC_MakeIMP(Class class, Class super_class, PyObject* sel, PyObject* imp)
         return retval;
     } else {
         PyErr_Clear();
+
+        const char* sel_signature = PyObjCSelector_Signature(sel);
+        if (sel_signature == NULL) {
+            return NULL;
+        }
         methinfo = PyObjCMethodSignature_ForSelector(
-                class, (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kCLASS_METHOD) != 0,
-                PyObjCSelector_GetSelector(sel),
-                PyObjCSelector_Signature(sel),
-                PyObjCNativeSelector_Check(sel));
+            class, (PyObjCSelector_GetFlags(sel) & PyObjCSelector_kCLASS_METHOD) != 0,
+            PyObjCSelector_GetSelector(sel), sel_signature,
+            PyObjCNativeSelector_Check(sel));
         if (methinfo == NULL) {
             return NULL;
         }
-        retval = PyObjCFFI_MakeIMPForSignature(
-                methinfo, PyObjCSelector_GetSelector(sel), imp);
+        retval = blockimpForSignature(PyObjCSelector_GetSelector(sel), sel_signature, imp,
+                                      methinfo);
+        if (retval != NULL) {
+            return retval;
+        }
+        retval =
+            PyObjCFFI_MakeIMPForSignature(methinfo, PyObjCSelector_GetSelector(sel), imp);
         Py_DECREF(methinfo);
         return retval;
     }
 }
 
 void
-PyObjCUnsupportedMethod_IMP(
-    ffi_cif* cif __attribute__((__unused__)),
-    void* resp __attribute__((__unused__)),
-    void** args,
-    void* userdata __attribute__((__unused__)))
+PyObjCUnsupportedMethod_IMP(ffi_cif* cif __attribute__((__unused__)),
+                            void* resp __attribute__((__unused__)), void** args,
+                            void* userdata __attribute__((__unused__)))
 {
     [NSException raise:NSInvalidArgumentException
-        format:@"Implementing %s from Python is not supported for %@",
-            sel_getName(*(SEL*)args[1]),
-            *(id*)args[0]];
+                format:@"Implementing %s from Python is not supported for %@",
+                       sel_getName(*(SEL*)args[1]), *(id*)args[0]];
 }
 
 PyObject*
-PyObjCUnsupportedMethod_Caller(
-    PyObject* meth,
-    PyObject* self,
-    PyObject* args __attribute__((__unused__)))
+PyObjCUnsupportedMethod_Caller(PyObject* meth, PyObject* self,
+                               PyObject* const* args __attribute__((__unused__)),
+                               size_t           nargs __attribute__((__unused__)))
 {
-#if PY_MAJOR_VERSION == 2
-    PyObject* repr;
-
-    repr = PyObject_Repr(self);
-    if (repr == NULL || !PyString_Check(repr)) {
-        Py_XDECREF(repr);
-        PyErr_Format(PyExc_TypeError,
-            "Cannot call '%s' on instances of '%s' from Python",
-            sel_getName(PyObjCSelector_GetSelector(meth)),
-            Py_TYPE(self)->tp_name);
-        return NULL;
-    }
-
-    PyErr_Format(PyExc_TypeError,
-        "Cannot call '%s' on '%s' from Python",
-        sel_getName(PyObjCSelector_GetSelector(meth)),
-        PyString_AS_STRING(repr));
-    Py_DECREF(repr);
+    PyErr_Format(PyExc_TypeError, "Cannot call '%s' on '%R' from Python",
+                 sel_getName(PyObjCSelector_GetSelector(meth)), self);
     return NULL;
-
-#else
-    PyErr_Format(PyExc_TypeError,
-        "Cannot call '%s' on '%R' from Python",
-        sel_getName(PyObjCSelector_GetSelector(meth)),
-        self);
-    return NULL;
-
-#endif
-
 }
+
+NS_ASSUME_NONNULL_END
