@@ -10,11 +10,13 @@
 
 #import <Foundation/Foundation.h>
 
+NS_ASSUME_NONNULL_BEGIN
+
 static int*
 makeIntArrayOf5(void)
 {
     static int result[5];
-    int i;
+    int        i;
     for (i = 0; i < 5; i++) {
         result[i] = i * i;
     }
@@ -28,11 +30,10 @@ makeStringArray(void)
     return result;
 }
 
-static int*
-makeIntArrayOf_(int count)
+static int* _Nullable makeIntArrayOf_(int count)
 {
     static int* result = NULL;
-    int i;
+    int         i;
 
     if (result) {
         free(result);
@@ -47,29 +48,19 @@ makeIntArrayOf_(int count)
     return result;
 }
 
-static int*
-nullIntArrayOf5(void)
+static int* _Nullable nullIntArrayOf5(void) { return NULL; }
+
+static char** _Nullable nullStringArray(void) { return NULL; }
+
+static int* _Nullable nullIntArrayOf_(int count __attribute__((__unused__)))
 {
     return NULL;
 }
 
-static char**
-nullStringArray(void)
-{
-    return NULL;
-}
-
-static int*
-nullIntArrayOf_(int count __attribute__((__unused__)))
-{
-    return NULL;
-}
-
-static NSArray*
-makeIntArray_count_(int* data, unsigned count)
+static NSArray* _Nullable makeIntArray_count_(int* data, unsigned count)
 {
     NSMutableArray* array;
-    unsigned i;
+    unsigned        i;
 
     array = [NSMutableArray arrayWithCapacity:count];
 
@@ -79,8 +70,7 @@ makeIntArray_count_(int* data, unsigned count)
     return array;
 }
 
-static NSArray*
-nullIntArray_count_(int* data, unsigned count)
+static NSArray* _Nullable nullIntArray_count_(int* data, unsigned count)
 {
     if (data) {
         return makeIntArray_count_(data, count);
@@ -89,17 +79,15 @@ nullIntArray_count_(int* data, unsigned count)
     }
 }
 
-static NSArray*
-makeIntArray_countPtr_(int* data, unsigned* countPtr)
+static NSArray* _Nullable makeIntArray_countPtr_(int* data, unsigned* countPtr)
 {
     return makeIntArray_count_(data, *countPtr);
 }
 
-static NSArray*
-make4Tuple_(double* data)
+static NSArray* _Nullable make4Tuple_(double* data)
 {
     NSMutableArray* array;
-    unsigned i;
+    unsigned        i;
 
     array = [NSMutableArray array];
 
@@ -109,8 +97,7 @@ make4Tuple_(double* data)
     return array;
 }
 
-static NSArray*
-null4Tuple_(double* data)
+static NSArray* _Nullable null4Tuple_(double* data)
 {
     if (data) {
         return make4Tuple_(data);
@@ -119,22 +106,24 @@ null4Tuple_(double* data)
     }
 }
 
-static NSArray*
-makeStringArray_(char** data)
+static NSArray* _Nullable makeStringArray_(char** data)
 {
     NSMutableArray* array;
 
     array = [NSMutableArray array];
 
     while (*data != NULL) {
-        [array addObject:[NSString stringWithUTF8String:*data]];
+        NSObject* val = [NSString stringWithUTF8String:*data];
+        if (val == NULL)
+            continue;
+
+        [array addObject:val];
         data++;
     }
     return array;
 }
 
-static NSArray*
-nullStringArray_(char** data)
+static NSArray* _Nullable nullStringArray_(char** data)
 {
     if (data) {
         return makeStringArray_(data);
@@ -143,8 +132,7 @@ nullStringArray_(char** data)
     }
 }
 
-static NSArray*
-makeObjectArray_(id* data)
+static NSArray* _Nullable makeObjectArray_(id* data)
 {
     NSMutableArray* array;
 
@@ -216,10 +204,10 @@ static void
 reverseArray_count_(float* data, int count)
 {
     float t;
-    int i;
+    int   i;
     for (i = 0; i < count / 2; i++) {
-        t = data[i];
-        data[i] = data[count - 1 - i];
+        t                   = data[i];
+        data[i]             = data[count - 1 - i];
         data[count - 1 - i] = t;
     }
 }
@@ -236,7 +224,7 @@ nullreverseArray_count_(float* data, int count)
 static void
 reverseStrings_(char** data)
 {
-    int count, i;
+    int   count, i;
     char* t;
 
     for (count = 0; data[count] != NULL; count++) {
@@ -244,8 +232,8 @@ reverseStrings_(char** data)
     }
 
     for (i = 0; i < count / 2; i++) {
-        t = data[i];
-        data[i] = data[count - 1 - i];
+        t                   = data[i];
+        data[i]             = data[count - 1 - i];
         data[count - 1 - i] = t;
     }
 }
@@ -264,11 +252,11 @@ reverse4Tuple_(short* data)
 {
     short t;
 
-    t = data[0];
+    t       = data[0];
     data[0] = data[3];
     data[3] = t;
 
-    t = data[1];
+    t       = data[1];
     data[1] = data[2];
     data[2] = t;
 }
@@ -299,24 +287,33 @@ static void
 swapX_andY_(double* x, double* y)
 {
     double t = *x;
-    *x = *y;
-    *y = t;
+    *x       = *y;
+    *y       = t;
 }
 
-static NSArray*
-input_output_inputAndOutput_(int* x, int* y, int* z)
+static NSArray* _Nullable input_output_inputAndOutput_(int* x, int* y, int* z)
 {
-    char buf[64];
+    char            buf[64];
+    NSString*       value;
     NSMutableArray* result = [NSMutableArray array];
 
     snprintf(buf, sizeof(buf), "%p", x);
-    [result addObject:[NSString stringWithUTF8String:buf]];
+    value = [NSString stringWithUTF8String:buf];
+    if (!value)
+        return NULL;
+    [result addObject:value];
 
     snprintf(buf, sizeof(buf), "%p", y);
-    [result addObject:[NSString stringWithUTF8String:buf]];
+    value = [NSString stringWithUTF8String:buf];
+    if (!value)
+        return NULL;
+    [result addObject:value];
 
     snprintf(buf, sizeof(buf), "%p", z);
-    [result addObject:[NSString stringWithUTF8String:buf]];
+    value = [NSString stringWithUTF8String:buf];
+    if (!value)
+        return NULL;
+    [result addObject:value];
 
     if (y) {
         if (x) {
@@ -383,11 +380,11 @@ maybeReverseArray_(short* data)
     return 2;
 }
 
-static NSArray*
+static NSArray* __attribute__((__format__(__NSString__, 1, 2)))
 makeArrayWithFormat_(NSString* fmt, ...)
 {
     va_list ap;
-    char buffer[2048];
+    char    buffer[2048];
 
     va_start(ap, fmt);
 #pragma clang diagnostic push
@@ -400,26 +397,33 @@ makeArrayWithFormat_(NSString* fmt, ...)
     return [NSArray arrayWithObjects:fmt, [NSString stringWithUTF8String:buffer], NULL];
 }
 
-#pragma GCC diagnostic push
+#pragma GCC diagnostic   push
 #pragma clang diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#pragma GCC diagnostic   ignored "-Wformat-nonliteral"
 #pragma clang diagnostic ignored "-Wformat-nonliteral"
 
-static NSArray*
+static NSArray* _Nullable __attribute__((__format__(__printf__, 1, 2)))
 makeArrayWithCFormat_(char* fmt, ...)
 {
-    va_list ap;
-    char buffer[2048];
+    va_list   ap;
+    char      buffer[2048];
+    NSString* a1;
+    NSString* a2;
 
     va_start(ap, fmt);
     vsnprintf(buffer, sizeof(buffer), fmt, ap);
     va_end(ap);
 
-    return [NSArray arrayWithObjects:[NSString stringWithUTF8String:fmt],
-                                     [NSString stringWithUTF8String:buffer], NULL];
+    a1 = [NSString stringWithUTF8String:fmt];
+    if (!a1)
+        return NULL;
+    a2 = [NSString stringWithUTF8String:buffer];
+    if (!a2)
+        return NULL;
+    return [NSArray arrayWithObjects:a1, a2, NULL];
 }
 
-#pragma GCC diagnostic pop
+#pragma GCC diagnostic   pop
 #pragma clang diagnostic pop
 
 static int
@@ -438,7 +442,7 @@ maybeFillArray_(int* data)
 typedef void (*F)(void);
 static struct function {
     char* name;
-    F function;
+    F     function;
 } gFunctionMap[] = {{"makeIntArrayOf5", (F)makeIntArrayOf5},
                     {"makeStringArray", (F)makeStringArray},
                     {"makeIntArrayOf_", (F)makeIntArrayOf_},
@@ -491,7 +495,7 @@ static struct PyModuleDef mod_module = {PyModuleDef_HEAD_INIT,
                                         NULL,
                                         NULL};
 
-PyObject* PyInit_metadatafunction(void);
+PyObject* _Nullable PyInit_metadatafunction(void);
 
 PyObject* __attribute__((__visibility__("default"))) PyInit_metadatafunction(void)
 {
@@ -518,3 +522,5 @@ PyObject* __attribute__((__visibility__("default"))) PyInit_metadatafunction(voi
 
     return m;
 }
+
+NS_ASSUME_NONNULL_END
