@@ -1,11 +1,19 @@
 #include "pyobjc.h"
+NS_ASSUME_NONNULL_BEGIN
+
+/*
+ * XXX: Vectorcall variants for as_tuple and as_buffer
+ *
+ * Both should be fairly easy as there is only one argument,
+ * although supporting keyword arguments complicates things.
+ */
 
-#define VARLIST_TYPE(obj) (((char*)(obj)) + sizeof(PyObjC_VarList))
 typedef struct {
     PyObject_HEAD
 
     void*      array;
     Py_ssize_t itemsize;
+    char       typestr[];
 } PyObjC_VarList;
 
 PyDoc_STRVAR(object_as_tuple_doc,
@@ -13,8 +21,27 @@ PyDoc_STRVAR(object_as_tuple_doc,
              "Return a tuple containing the first ``count`` elements of "
              "this list");
 
-static PyObject*
-object_as_tuple(PyObject* _self, PyObject* args, PyObject* kwds)
+static int
+check_index(PyObjC_VarList* self, Py_ssize_t idx)
+{
+    if (idx < 0) {
+        /* Indeterminate size, cannot access from the 'end' */
+        PyErr_SetString(PyExc_ValueError,
+                        "objc.varlist does not support negative indexes");
+        return -1;
+    }
+
+    if (idx >= PY_SSIZE_T_MAX / self->itemsize) {
+        /* Integer overflow, index cannot be correct */
+        PyErr_Format(PyExc_IndexError, "Index '%" PY_FORMAT_SIZE_T "d' out of range",
+                     idx);
+        return -1;
+    }
+    return 0;
+}
+
+static PyObject* _Nullable object_as_tuple(PyObject* _self, PyObject* args,
+                                           PyObject* kwds)
 {
     static char* keywords[] = {"count", NULL};
 
@@ -27,14 +54,21 @@ object_as_tuple(PyObject* _self, PyObject* args, PyObject* kwds)
         return NULL;
     }
 
+    if (length >= PY_SSIZE_T_MAX / self->itemsize) {
+        /* Integer overflow, index cannot be correct */
+        PyErr_Format(PyExc_OverflowError, "Index '%" PY_FORMAT_SIZE_T "d' out of range",
+                     length);
+        return NULL;
+    }
+
     result = PyTuple_New(length);
     if (result == NULL) {
         return NULL;
     }
 
     for (i = 0; i < length; i++) {
-        PyObject* v = pythonify_c_value(VARLIST_TYPE(self), ((unsigned char*)self->array)
-                                                                + (i * self->itemsize));
+        PyObject* v = pythonify_c_value(self->typestr, ((unsigned char*)self->array)
+                                                           + (i * self->itemsize));
         if (v == NULL) {
             Py_DECREF(result);
             return NULL;
@@ -49,8 +83,8 @@ PyDoc_STRVAR(object_as_buffer_doc,
              "Return a memoryview referencing the memory for the first ``count`` "
              "elements of this list.");
 
-static PyObject*
-object_as_buffer(PyObject* _self, PyObject* args, PyObject* kwds)
+static PyObject* _Nullable object_as_buffer(PyObject* _self, PyObject* args,
+                                            PyObject* kwds)
 {
     static char* keywords[] = {"count", NULL};
 
@@ -65,7 +99,9 @@ object_as_buffer(PyObject* _self, PyObject* args, PyObject* kwds)
 
     if (length >= PY_SSIZE_T_MAX / self->itemsize) {
         /* Calculating the buffer size would overflow */
-        PyErr_SetString(PyExc_OverflowError, "Buffer too long");
+        PyErr_Format(PyExc_OverflowError, "Index '%" PY_FORMAT_SIZE_T "d' out of range",
+                     length);
+
         return NULL;
     }
 
@@ -78,40 +114,33 @@ object_as_buffer(PyObject* _self, PyObject* args, PyObject* kwds)
     return PyMemoryView_FromBuffer(&info);
 }
 
-static PyObject*
-object__getitem__(PyObject* _self, Py_ssize_t idx)
+static PyObject* _Nullable object__getitem__(PyObject* _self, Py_ssize_t idx)
 {
     PyObjC_VarList* self = (PyObjC_VarList*)_self;
 
-    if (idx < 0) {
-        /* Indeterminate size, cannot access from the 'end' */
-        PyErr_SetString(PyExc_ValueError, "Negative indexes are not supported");
+    if (check_index(self, idx) == -1) {
         return NULL;
     }
 
-    if (idx >= PY_SSIZE_T_MAX / self->itemsize) {
-        /* Interger overflow, index cannot be correct */
-        PyErr_Format(PyExc_IndexError, "Index '%" PY_FORMAT_SIZE_T "d' out of range",
-                     idx);
-        return NULL;
-    }
-
-    return pythonify_c_value(VARLIST_TYPE(self),
+    return pythonify_c_value(self->typestr,
                              ((unsigned char*)self->array) + (idx * self->itemsize));
 }
 
-static PyObject*
-object__getslice__(PyObject* _self, Py_ssize_t start, Py_ssize_t stop)
+static PyObject* _Nullable object__getslice__(PyObject* _self, Py_ssize_t start,
+                                              Py_ssize_t stop)
 {
     PyObjC_VarList* self = (PyObjC_VarList*)_self;
     PyObject*       result;
     Py_ssize_t      idx;
 
-    if (start < 0 || stop < 0) {
-        PyErr_SetString(PyExc_ValueError,
-                        "objc.varlist doesn't support slices with negative indexes");
+    if (check_index(self, start) == -1) {
         return NULL;
     }
+
+    if (check_index(self, stop) == -1) {
+        return NULL;
+    }
+
     if (stop < start) {
         stop = start;
     }
@@ -119,8 +148,8 @@ object__getslice__(PyObject* _self, Py_ssize_t start, Py_ssize_t stop)
     result = PyTuple_New(stop - start);
 
     for (idx = start; idx < stop; idx++) {
-        PyObject* v = pythonify_c_value(VARLIST_TYPE(self), ((unsigned char*)self->array)
-                                                                + (idx * self->itemsize));
+        PyObject* v = pythonify_c_value(self->typestr, ((unsigned char*)self->array)
+                                                           + (idx * self->itemsize));
         if (v == NULL) {
             Py_DECREF(result);
             return NULL;
@@ -138,11 +167,14 @@ object__setslice__(PyObject* _self, Py_ssize_t start, Py_ssize_t stop, PyObject*
     Py_ssize_t      idx;
     PyObject*       seq;
 
-    if (start < 0 || stop < 0) {
-        PyErr_SetString(PyExc_ValueError,
-                        "objc.varlist doesn't support slices with negative indexes");
+    if (check_index(self, start) == -1) {
         return -1;
     }
+
+    if (check_index(self, stop) == -1) {
+        return -1;
+    }
+
     if (stop < start) {
         stop = start;
     }
@@ -161,9 +193,8 @@ object__setslice__(PyObject* _self, Py_ssize_t start, Py_ssize_t stop, PyObject*
 
     for (idx = start; idx < stop; idx++) {
         PyObject* v = PySequence_Fast_GET_ITEM(seq, idx - start);
-        int       r =
-            depythonify_c_value(VARLIST_TYPE(self), v,
-                                ((unsigned char*)self->array) + (idx * self->itemsize));
+        int       r = depythonify_c_value(
+                  self->typestr, v, ((unsigned char*)self->array) + (idx * self->itemsize));
         if (r == -1) {
             Py_DECREF(seq);
             return -1;
@@ -174,24 +205,15 @@ object__setslice__(PyObject* _self, Py_ssize_t start, Py_ssize_t stop, PyObject*
 }
 
 static int
-object__setitem__(PyObject* _self, Py_ssize_t idx, PyObject* value)
+object__setitem__(PyObject* _self, Py_ssize_t idx, PyObject* _Nullable value)
 {
     PyObjC_VarList* self = (PyObjC_VarList*)_self;
 
-    if (idx < 0) {
-        /* Indeterminate size, cannot access from the 'end' */
-        PyErr_SetString(PyExc_ValueError, "Negative indexes are not supported");
+    if (check_index(self, idx) == -1) {
         return -1;
     }
 
-    if (idx >= PY_SSIZE_T_MAX / self->itemsize) {
-        /* Interger overflow, index cannot be correct */
-        PyErr_Format(PyExc_IndexError, "Index '%" PY_FORMAT_SIZE_T "d' out of range",
-                     idx);
-        return -1;
-    }
-
-    return depythonify_c_value(VARLIST_TYPE(self), value,
+    return depythonify_c_value(self->typestr, value,
                                ((unsigned char*)self->array) + (idx * self->itemsize));
 }
 
@@ -201,19 +223,22 @@ static PySequenceMethods object_tp_as_list = {
 };
 
 static Py_ssize_t
-sl_ind_get(PyObject* value)
+sl_ind_get(PyObject* value, int is_start)
 {
 
     if (value == Py_None) {
-        return -1;
+        /* Unspecified index, this is only supported for the start
+         * index (as we only support step size 1).
+         */
+        if (is_start) {
+            return 0;
+        } else {
+            PyErr_SetString(PyExc_ValueError, "Slice end must be specified");
+            return -1;
+        }
 
     } else if (PyIndex_Check(value)) {
-        Py_ssize_t result;
-        result = PyNumber_AsSsize_t(value, PyExc_IndexError);
-        if (result == -1 && PyErr_Occurred()) {
-            result = -1;
-        }
-        return result;
+        return PyNumber_AsSsize_t(value, PyExc_IndexError);
 
     } else {
         PyErr_Format(PyExc_ValueError, "Slice index of unsupported type '%.200s'",
@@ -222,8 +247,7 @@ sl_ind_get(PyObject* value)
     }
 }
 
-static PyObject*
-object_subscript(PyObject* self, PyObject* item)
+static PyObject* _Nullable object_subscript(PyObject* self, PyObject* item)
 {
     if (PyIndex_Check(item)) {
         Py_ssize_t i = PyNumber_AsSsize_t(item, PyExc_IndexError);
@@ -236,12 +260,12 @@ object_subscript(PyObject* self, PyObject* item)
         Py_ssize_t     start, stop, step;
         PySliceObject* sl = (PySliceObject*)item;
 
-        start = sl_ind_get(sl->start);
+        start = sl_ind_get(sl->start, 1);
         if (start == -1 && PyErr_Occurred()) {
             return NULL;
         }
 
-        stop = sl_ind_get(sl->stop);
+        stop = sl_ind_get(sl->stop, 0);
         if (stop == -1 && PyErr_Occurred()) {
             return NULL;
         }
@@ -249,13 +273,14 @@ object_subscript(PyObject* self, PyObject* item)
         if (sl->step == Py_None) {
             step = 1;
         } else {
-            step = sl_ind_get(sl->stop);
+            step = sl_ind_get(sl->step, 0);
             if (step == -1 && PyErr_Occurred()) {
                 return NULL;
             }
         }
 
         if (step != 1) {
+            /* XXX: Might be cool to support other steps... */
             PyErr_Format(PyExc_ValueError,
                          "objc.varlist doesn't support slice steps other than 1");
             return NULL;
@@ -271,7 +296,7 @@ object_subscript(PyObject* self, PyObject* item)
 }
 
 static int
-object_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
+object_ass_subscript(PyObject* self, PyObject* item, PyObject* _Nullable value)
 {
     if (PyIndex_Check(item)) {
         Py_ssize_t i = PyNumber_AsSsize_t(item, PyExc_IndexError);
@@ -284,12 +309,12 @@ object_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
         Py_ssize_t     start, stop, step;
         PySliceObject* sl = (PySliceObject*)item;
 
-        start = sl_ind_get(sl->start);
+        start = sl_ind_get(sl->start, 1);
         if (start == -1 && PyErr_Occurred()) {
             return -1;
         }
 
-        stop = sl_ind_get(sl->stop);
+        stop = sl_ind_get(sl->stop, 0);
         if (stop == -1 && PyErr_Occurred()) {
             return -1;
         }
@@ -297,13 +322,14 @@ object_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
         if (sl->step == Py_None) {
             step = 1;
         } else {
-            step = sl_ind_get(sl->stop);
+            step = sl_ind_get(sl->step, 0);
             if (step == -1 && PyErr_Occurred()) {
                 return -1;
             }
         }
 
         if (step != 1) {
+            /* XXX: Might be cool to support other steps... */
             PyErr_Format(PyExc_ValueError,
                          "objc.varlist doesn't support slice steps other than 1");
             return -1;
@@ -321,13 +347,11 @@ object_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
 static PyMappingMethods object_tp_as_mapping = {.mp_subscript     = object_subscript,
                                                 .mp_ass_subscript = object_ass_subscript};
 
-static PyObject*
-object_new(PyTypeObject* type __attribute__((__unused__)),
-           PyObject*     args __attribute__((__unused__)),
-           PyObject*     kwds __attribute__((__unused__)))
+static PyObject* _Nullable object_new(PyTypeObject* type __attribute__((__unused__)),
+                                      PyObject*     args __attribute__((__unused__)),
+                                      PyObject*     kwds __attribute__((__unused__)))
 {
-    PyErr_SetString(PyExc_TypeError,
-                    "Cannot create instances of 'objc.varlist' in Python");
+    PyErr_SetString(PyExc_TypeError, "Cannot create instances of 'objc.varlist'");
     return NULL;
 }
 
@@ -341,12 +365,13 @@ PyDoc_STRVAR(object_doc,
              "objc.varlist\n"
              "\n"
              "A list of an unspecified size. Use ``obj.as_tuple(count)`` to \n"
-             "convertto a python tuple, or ``obj[index]`` to fetch a single item");
+             "convert to a python tuple, or ``obj[index]`` to fetch a single item");
 
-static PyObject*
-object_typestr_get(PyObject* self, void* closure __attribute__((__unused__)))
+static PyObject* _Nullable object_typestr_get(PyObject* _self, void* _Nullable closure
+                                              __attribute__((__unused__)))
 {
-    return PyBytes_FromString(VARLIST_TYPE(self));
+    PyObjC_VarList* self = (PyObjC_VarList*)_self;
+    return PyBytes_FromString(self->typestr);
 }
 
 static PyGetSetDef object_getset[] = {
@@ -391,9 +416,11 @@ PyObjC_VarList_New(const char* tp, void* array)
 {
     PyObjC_VarList* result;
     const char*     end;
-    char*           tp_buf;
 
     end = PyObjCRT_SkipTypeSpec(tp);
+    if (end == NULL) {
+        return NULL;
+    }
     while (end > tp && isdigit(end[-1])) {
         end--;
     }
@@ -404,15 +431,21 @@ PyObjC_VarList_New(const char* tp, void* array)
         return NULL;
     }
     PyObject_Init((PyObject*)result, &PyObjC_VarList_Type);
-    tp_buf           = VARLIST_TYPE(result);
     result->array    = array;
     result->itemsize = PyObjCRT_AlignedSize(tp);
-    memcpy(tp_buf, tp, end - tp);
-    tp_buf[end - tp] = '\0';
+    if (unlikely(result->itemsize == -1)) {
+        Py_DECREF(result);
+        return NULL;
+    }
 
-    if (*tp_buf == _C_VOID) {
-        *tp_buf = _C_CHAR_AS_TEXT;
+    memcpy(result->typestr, tp, end - tp);
+    result->typestr[end - tp] = '\0';
+
+    if (result->typestr[0] == _C_VOID) {
+        result->typestr[0] = _C_CHAR_AS_TEXT;
     }
 
     return (PyObject*)result;
 }
+
+NS_ASSUME_NONNULL_END
