@@ -5,21 +5,14 @@
 # str, int, float) a pure ObjC program should be able to read
 # that archive as a datastructure with the corresponding Cocoa
 # classes.
-from PyObjCTools.TestSupport import *
-
-import objc
+import os
+import platform
 import subprocess
 import tempfile
-from distutils.sysconfig import get_config_var
-import os
-import sys
-import platform
+from plistlib import loads
 
-if sys.version_info[0] == 2:
-    from plistlib import readPlistFromString as loads, Data
-
-else:
-    from plistlib import loads, Data
+import objc
+from PyObjCTools.TestSupport import TestCase
 
 
 MYDIR = os.path.dirname(os.path.abspath(__file__))
@@ -31,6 +24,7 @@ NSKeyedArchiver = objc.lookUpClass("NSKeyedArchiver")
 NSUnarchiver = objc.lookUpClass("NSUnarchiver")
 NSKeyedUnarchiver = objc.lookUpClass("NSKeyedUnarchiver")
 NSSet = objc.lookUpClass("NSSet")
+NSString = objc.lookUpClass("NSString")
 
 
 class TestNSKeyedArchivingInterop(TestCase):
@@ -58,7 +52,7 @@ class TestNSKeyedArchivingInterop(TestCase):
             os.unlink(cls.progpath)
 
     def test_interop_string(self):
-        for testval in (b"hello world".decode("utf-8"), "goodbye moon"):
+        for testval in ("hello world", "goodbye moon"):
             v = NSArray.arrayWithObject_(testval)
             data = NSKeyedArchiver.archivedDataWithRootObject_(v)
 
@@ -86,7 +80,7 @@ class TestNSKeyedArchivingInterop(TestCase):
             self.assertEqual(converted, [testval])
 
     def test_interop_int(self):
-        for testval in (-42, 0, 42, -2 ** 62, 2 ** 62):
+        for testval in (-42, 0, 42, -(2 ** 62), 2 ** 62):
             v = NSArray.arrayWithObject_(testval)
             data = NSKeyedArchiver.archivedDataWithRootObject_(v)
 
@@ -113,12 +107,8 @@ class TestNSKeyedArchivingInterop(TestCase):
                 [self.progpath, "keyed", fp.name],
             )
 
-    @onlyPython3
     def test_interop_data(self):
         for testval in (b"hello world",):
-            if sys.version_info[0] == 2:
-                testval = buffer(testval)
-
             v = NSArray.arrayWithObject_(testval)
             data = NSKeyedArchiver.archivedDataWithRootObject_(v)
 
@@ -160,8 +150,7 @@ class TestNSKeyedArchivingInterop(TestCase):
             self.assertTrue(converted.startswith(b"{("))
             self.assertTrue(converted.endswith(b")}\n"))
             converted = b"{" + converted[2:-3] + b"}"
-            converted = eval(converted.decode("utf-8"), dict(a="a", b="b"))
-
+            converted = eval(converted.decode("utf-8"), {"a": "a", "b": "b"})
             self.assertEqual(converted, set(testval))
 
     def test_interop_dict(self):
@@ -204,7 +193,7 @@ class TestNSArchivingInterop(TestCase):
             os.unlink(cls.progpath)
 
     def test_interop_string(self):
-        for testval in (b"hello world".decode("utf-8"), "goodbye moon"):
+        for testval in ("hello world", "goodbye moon"):
             v = NSArray.arrayWithObject_(testval)
             data = NSArchiver.archivedDataWithRootObject_(v)
 
@@ -232,7 +221,7 @@ class TestNSArchivingInterop(TestCase):
             self.assertEqual(converted, [testval])
 
     def test_interop_int(self):
-        for testval in (-42, 0, 42, -2 ** 62, 2 ** 62):
+        for testval in (-42, 0, 42, -(2 ** 62), 2 ** 62):
             v = NSArray.arrayWithObject_(testval)
             data = NSArchiver.archivedDataWithRootObject_(v)
 
@@ -259,12 +248,8 @@ class TestNSArchivingInterop(TestCase):
                 [self.progpath, "plain", fp.name],
             )
 
-    @onlyPython3
     def test_interop_data(self):
         for testval in (b"hello world",):
-            if sys.version_info[0] == 2:
-                testval = buffer(testval)
-
             v = NSArray.arrayWithObject_(testval)
             data = NSArchiver.archivedDataWithRootObject_(v)
 
@@ -306,7 +291,7 @@ class TestNSArchivingInterop(TestCase):
             self.assertTrue(converted.startswith(b"{("))
             self.assertTrue(converted.endswith(b")}\n"))
             converted = b"{" + converted[2:-3] + b"}"
-            converted = eval(converted.decode("utf-8"), dict(a="a", b="b"))
+            converted = eval(converted.decode("utf-8"), {"a": "a", "b": "b"})
             self.assertEqual(converted, set(testval))
 
     def test_interop_dict(self):
@@ -328,7 +313,7 @@ class Class1:
     pass
 
 
-class Class2(object):
+class Class2:
     pass
 
 
@@ -392,7 +377,3 @@ class TestLoadingOlderVersions(TestCase):
 
         locals()["test_%s" % (fname.replace(".", "_").replace("-", "_"))] = test
         del test
-
-
-if __name__ == "__main__":
-    main()
