@@ -1,10 +1,8 @@
-from PyObjCTools.TestSupport import *
-
-import sys
 import inspect
 
-from objc import _callable_docstr as mod
 import objc
+from objc import _callable_docstr as mod
+from PyObjCTools.TestSupport import TestCase
 
 NSArray = objc.lookUpClass("NSArray")
 
@@ -70,7 +68,7 @@ class TestDescribeType(TestCase):
             "struct hello*",
         )
 
-        handle = objc.createOpaquePointerType("NamedPointer", b"^{NamedTestPointer1=}")
+        _ = objc.createOpaquePointerType("NamedPointer", b"^{NamedTestPointer1=}")
         self.assertEqual(mod.describe_type(b"^{NamedTestPointer1=}"), "NamedPointer")
 
     def test_unknown(self):
@@ -100,7 +98,11 @@ class TestDescribeType(TestCase):
         )
         self.assertEqual(
             mod.describe_type(
-                objc._C_STRUCT_B + b"name=" + objc._C_ID + objc._C_INT + objc._C_STRUCT_E
+                objc._C_STRUCT_B
+                + b"name="
+                + objc._C_ID
+                + objc._C_INT
+                + objc._C_STRUCT_E
             ),
             "struct name",
         )
@@ -116,7 +118,7 @@ class TestDescribeType(TestCase):
             "struct name",
         )
 
-        strType = objc.createStructType(
+        _ = objc.createStructType(
             "NamedTestStruct", b'{NamedTestStruct1="a"i"b"i}', None
         )
         self.assertEqual(mod.describe_type(b"{NamedTestStruct1=ii}"), "NamedTestStruct")
@@ -151,14 +153,17 @@ class TestDescribeType(TestCase):
 class TestDescribeCallable(TestCase):
     def setUp(self):
         dct = {}
-        func = objc.loadBundleFunctions(
+        objc.loadBundleFunctions(
             None,
             dct,
             [
                 ("NSTemporaryDirectory", objc._C_ID),
                 (
                     "NSSearchPathForDirectoriesInDomains",
-                    objc._C_ID + objc._C_NSUInteger + objc._C_NSUInteger + objc._C_NSBOOL,
+                    objc._C_ID
+                    + objc._C_NSUInteger
+                    + objc._C_NSUInteger
+                    + objc._C_NSBOOL,
                 ),
             ],
         )
@@ -193,14 +198,14 @@ class TestDescribeCallable(TestCase):
                 [
                     (
                         (NSArray.array.__name__, NSArray.array.__metadata__()),
-                        dict(ismethod=True),
+                        {"ismethod": True},
                     ),
                     (
                         (
                             self.NSTemporaryDirectory.__name__,
                             self.NSTemporaryDirectory.__metadata__(),
                         ),
-                        dict(ismethod=False),
+                        {"ismethod": False},
                     ),
                 ],
             )
@@ -219,7 +224,10 @@ class TestDescribeCallable(TestCase):
         self.assertEqual(
             mod.describe_callable_metadata(
                 "array",
-                {"arguments": [{"type": b"i"}, {"type": b"f"}], "retval": {"type": b"v"}},
+                {
+                    "arguments": [{"type": b"i"}, {"type": b"f"}],
+                    "retval": {"type": b"v"},
+                },
                 ismethod=False,
             ),
             "void array(int arg0, float arg1);",
@@ -237,7 +245,8 @@ class TestDescribeCallable(TestCase):
             "void array(int arg0, float arg1, ...);",
         )
 
-        # This is metadata is nonsense (C doesn't allow variadic functions where all arguments are variadic)
+        # This is metadata is nonsense (C doesn't allow variadic
+        # functions where all arguments are variadic)
         self.assertEqual(
             mod.describe_callable_metadata(
                 "array",
@@ -373,7 +382,9 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void array(out float arg0, in int arg1);\n\narg0: pass-by-reference out argument\narg1: pass-by-reference in argument",
+            "void array(out float arg0, in int arg1);\n\n"
+            "arg0: pass-by-reference out argument\n"
+            "arg1: pass-by-reference in argument",
         )
 
         self.assertEqual(
@@ -459,7 +470,10 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void function(<FUNCTION> arg0);\n\narg0: int callback(<FUNCTION> arg0);\n\n    arg0: double callback(in id arg0, id arg1);\n\n        arg0: array with length in arg1",
+            "void function(<FUNCTION> arg0);\n\n"
+            "arg0: int callback(<FUNCTION> arg0);\n\n"
+            "    arg0: double callback(in id arg0, id arg1);\n\n"
+            "        arg0: array with length in arg1",
         )
 
         # - block pointers (simple and nested)
@@ -552,7 +566,10 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void function(<BLOCK> arg0);\n\narg0: int callback(<BLOCK> arg0);\n\n    arg0: double callback(in id arg0, id arg1);\n\n        arg0: array with length in arg1",
+            "void function(<BLOCK> arg0);\n\n"
+            "arg0: int callback(<BLOCK> arg0);\n\n"
+            "    arg0: double callback(in id arg0, id arg1);\n\n"
+            "        arg0: array with length in arg1",
         )
 
         # - variadic arguments
@@ -580,7 +597,8 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void printf(id arg0, int arg1, ...);\n\nVariadic arguments form an array of C type int",
+            "void printf(id arg0, int arg1, ...);\n\n"
+            "Variadic arguments form an array of C type int",
         )
         self.assertEqual(
             mod.describe_callable_metadata(
@@ -593,7 +611,9 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void printf(in id arg0, int arg1, ...);\n\narg0: pass-by-reference in argument\nVariadic arguments form an array of C type int",
+            "void printf(in id arg0, int arg1, ...);\n\n"
+            "arg0: pass-by-reference in argument\n"
+            "Variadic arguments form an array of C type int",
         )
 
         # - printf_format
@@ -670,7 +690,8 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void array(in int arg0);\n\narg0: array with length on input in arg2, and output in arg3",
+            "void array(in int arg0);\n\n"
+            "arg0: array with length on input in arg2, and output in arg3",
         )
         self.assertEqual(
             mod.describe_callable_metadata(
@@ -687,7 +708,9 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void array(in int arg0);\n\narg0: array with length on input in arg2, and output in return value",
+            "void array(in int arg0);\n\n"
+            "arg0: array with length on input in arg2, and output "
+            "in return value",
         )
         self.assertEqual(
             mod.describe_callable_metadata(
@@ -764,7 +787,8 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=True,
             ),
-            "- (void)array:(in int)arg0;\n\narg0: array with length on input in arg0, and output in arg1",
+            "- (void)array:(in int)arg0;\n\n"
+            "arg0: array with length on input in arg0, and output in arg1",
         )
         self.assertEqual(
             mod.describe_callable_metadata(
@@ -784,14 +808,20 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=True,
             ),
-            "- (void)array:(in int)arg0;\n\narg0: array with length on input in arg0, and output in return value",
+            "- (void)array:(in int)arg0;\n\n"
+            "arg0: array with length on input in arg0, and "
+            "output in return value",
         )
 
         # - warnings
         self.assertEqual(
             mod.describe_callable_metadata(
                 "array",
-                {"arguments": [], "retval": {"type": b"v"}, "suggestion": "Please don't"},
+                {
+                    "arguments": [],
+                    "retval": {"type": b"v"},
+                    "suggestion": "Please don't",
+                },
                 ismethod=False,
             ),
             "void array(void);\n\nWARNING: Please don't",
@@ -806,7 +836,9 @@ class TestDescribeCallable(TestCase):
                 },
                 ismethod=False,
             ),
-            "void array(in int arg0);\n\nWARNING: Please don't\n\narg0: pass-by-reference in argument",
+            "void array(in int arg0);\n\n"
+            "WARNING: Please don't\n\n"
+            "arg0: pass-by-reference in argument",
         )
 
     def test_docattr(self):
@@ -827,98 +859,112 @@ class TestDescribeCallable(TestCase):
             mod.describe_callable(self.NSSearchPathForDirectoriesInDomains),
         )
 
+    def test_no_metadata(self):
+        class M:
+            __name__ = "foo"
 
-if sys.version_info[:2] >= (3, 3):
+            def __metadata__(self):
+                raise objc.internal_error
 
-    class TestCallableSignature(TestCase):
-        def test_function(self):
-            dct = {}
-            func = objc.loadBundleFunctions(
-                None,
-                dct,
-                [
-                    ("NSTemporaryDirectory", objc._C_ID),
-                    (
-                        "NSSearchPathForDirectoriesInDomains",
-                        objc._C_ID
-                        + objc._C_NSUInteger
-                        + objc._C_NSUInteger
-                        + objc._C_NSBOOL,
-                    ),
-                ],
-            )
-            NSTemporaryDirectory = dct["NSTemporaryDirectory"]
-            NSSearchPathForDirectoriesInDomains = dct[
-                "NSSearchPathForDirectoriesInDomains"
-            ]
-
-            self.assertEqual(
-                NSTemporaryDirectory.__signature__,
-                mod.callable_signature(NSTemporaryDirectory),
-            )
-            self.assertEqual(
-                NSSearchPathForDirectoriesInDomains.__signature__,
-                mod.callable_signature(NSSearchPathForDirectoriesInDomains),
-            )
-
-            sig = NSTemporaryDirectory.__signature__
-            self.assertIsInstance(sig, inspect.Signature)
-            self.assertEqual(len(sig.parameters), 0)
-
-            sig = NSSearchPathForDirectoriesInDomains.__signature__
-            self.assertIsInstance(sig, inspect.Signature)
-            self.assertEqual(len(sig.parameters), 3)
-            self.assertEqual(
-                sig.parameters["arg0"],
-                inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
-            )
-            self.assertEqual(
-                sig.parameters["arg1"],
-                inspect.Parameter("arg1", inspect.Parameter.POSITIONAL_ONLY),
-            )
-            self.assertEqual(
-                sig.parameters["arg2"],
-                inspect.Parameter("arg2", inspect.Parameter.POSITIONAL_ONLY),
-            )
-            self.assertEqual(list(sig.parameters), ["arg0", "arg1", "arg2"])
-
-        def test_selector(self):
-            m = NSArray.array
-            self.assertEqual(m.__signature__, mod.callable_signature(m))
-
-            sig = m.__signature__
-            self.assertIsInstance(sig, inspect.Signature)
-            self.assertEqual(len(sig.parameters), 0)
-
-            m = NSArray.arrayWithObjects_
-            sig = m.__signature__
-            self.assertIsInstance(sig, inspect.Signature)
-            self.assertEqual(len(sig.parameters), 1)
-            self.assertEqual(
-                sig.parameters["arg0"],
-                inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
-            )
-
-            m = NSArray.indexOfObject_inRange_
-            sig = m.__signature__
-            self.assertIsInstance(sig, inspect.Signature)
-            self.assertEqual(len(sig.parameters), 2)
-            self.assertEqual(
-                sig.parameters["arg0"],
-                inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
-            )
-            self.assertEqual(
-                sig.parameters["arg1"],
-                inspect.Parameter("arg1", inspect.Parameter.POSITIONAL_ONLY),
-            )
-            self.assertEqual(list(sig.parameters), ["arg0", "arg1"])
-
-        def test_not_for_regular_types(self):
-            self.assertRaises(AttributeError, mod.callable_signature, 42)
-            self.assertRaises(AttributeError, mod.callable_signature, int)
-            self.assertRaises(AttributeError, mod.callable_signature, dir)
-            self.assertRaises(AttributeError, mod.callable_signature, lambda x: x * 2)
+        m = M()
+        self.assertIs(mod.describe_callable(m), None)
 
 
-if __name__ == "__main__":
-    main()
+class TestCallableSignature(TestCase):
+    def test_function(self):
+        dct = {}
+        objc.loadBundleFunctions(
+            None,
+            dct,
+            [
+                ("NSTemporaryDirectory", objc._C_ID),
+                (
+                    "NSSearchPathForDirectoriesInDomains",
+                    objc._C_ID
+                    + objc._C_NSUInteger
+                    + objc._C_NSUInteger
+                    + objc._C_NSBOOL,
+                ),
+            ],
+        )
+        NSTemporaryDirectory = dct["NSTemporaryDirectory"]
+        NSSearchPathForDirectoriesInDomains = dct["NSSearchPathForDirectoriesInDomains"]
+
+        self.assertEqual(
+            NSTemporaryDirectory.__signature__,
+            mod.callable_signature(NSTemporaryDirectory),
+        )
+        self.assertEqual(
+            NSSearchPathForDirectoriesInDomains.__signature__,
+            mod.callable_signature(NSSearchPathForDirectoriesInDomains),
+        )
+
+        sig = NSTemporaryDirectory.__signature__
+        self.assertIsInstance(sig, inspect.Signature)
+        self.assertEqual(len(sig.parameters), 0)
+
+        sig = NSSearchPathForDirectoriesInDomains.__signature__
+        self.assertIsInstance(sig, inspect.Signature)
+        self.assertEqual(len(sig.parameters), 3)
+        self.assertEqual(
+            sig.parameters["arg0"],
+            inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
+        )
+        self.assertEqual(
+            sig.parameters["arg1"],
+            inspect.Parameter("arg1", inspect.Parameter.POSITIONAL_ONLY),
+        )
+        self.assertEqual(
+            sig.parameters["arg2"],
+            inspect.Parameter("arg2", inspect.Parameter.POSITIONAL_ONLY),
+        )
+        self.assertEqual(list(sig.parameters), ["arg0", "arg1", "arg2"])
+
+    def test_selector(self):
+        m = NSArray.array
+        self.assertEqual(m.__signature__, mod.callable_signature(m))
+
+        sig = m.__signature__
+        self.assertIsInstance(sig, inspect.Signature)
+        self.assertEqual(len(sig.parameters), 0)
+
+        m = NSArray.arrayWithObjects_
+        sig = m.__signature__
+        self.assertIsInstance(sig, inspect.Signature)
+        self.assertEqual(len(sig.parameters), 1)
+        self.assertEqual(
+            sig.parameters["arg0"],
+            inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
+        )
+
+        m = NSArray.indexOfObject_inRange_
+        sig = m.__signature__
+        self.assertIsInstance(sig, inspect.Signature)
+        self.assertEqual(len(sig.parameters), 2)
+        self.assertEqual(
+            sig.parameters["arg0"],
+            inspect.Parameter("arg0", inspect.Parameter.POSITIONAL_ONLY),
+        )
+        self.assertEqual(
+            sig.parameters["arg1"],
+            inspect.Parameter("arg1", inspect.Parameter.POSITIONAL_ONLY),
+        )
+        self.assertEqual(list(sig.parameters), ["arg0", "arg1"])
+
+    def test_not_for_regular_types(self):
+        self.assertRaises(AttributeError, mod.callable_signature, 42)
+        self.assertRaises(AttributeError, mod.callable_signature, int)
+        self.assertRaises(AttributeError, mod.callable_signature, dir)
+        self.assertRaises(AttributeError, mod.callable_signature, lambda x: x * 2)
+
+    def test_no_metadata(self):
+        class M:
+            __name__ = "foo"
+
+            def __metadata__(self):
+                raise objc.internal_error
+
+        m = M()
+        self.assertIs(mod.describe_callable(m), None)
+
+        self.assertIs(mod.callable_signature(m), None)
