@@ -1,12 +1,20 @@
-from PyObjCTools.TestSupport import *
 import objc
+import platform
+from PyObjCTools.TestSupport import TestCase
 
 
 class TestConstants(TestCase):
     def test_version_current(self):
-        self.assertIsInstance(objc.MAC_OS_X_VERSION_CURRENT, (int, long))
+        self.assertIsInstance(objc.MAC_OS_X_VERSION_CURRENT, int)
 
-        v = os_release().split(".")[:2]
+        # Use platform.mac_ver() to calculate the expected value,
+        # that matches the version seen by ObjC code. The test
+        # function os_release() returns the actual system version.
+        # The two will be different when running on macOS 11 or later
+        # with a binary compiled using the 10.15 SDK (or earlier).
+
+        # v = os_release().split(".")[:2]
+        v = platform.mac_ver()[0].split(".")[:2]
         v = "MAC_OS_X_VERSION_%s_%s" % tuple(v)
 
         self.assertGreaterEqual(objc.MAC_OS_X_VERSION_CURRENT, getattr(objc, v))
@@ -41,7 +49,3 @@ class TestConstants(TestCase):
         self.assertEqual(objc.MAC_OS_X_VERSION_10_13_6, 101_306)
         self.assertEqual(objc.MAC_OS_X_VERSION_10_14, 101_400)
         self.assertEqual(objc.MAC_OS_X_VERSION_10_14_1, 101_401)
-
-
-if __name__ == "__main__":
-    main()
