@@ -3,22 +3,40 @@ Some basic tests for converting values to and from Objective-C
 
 TODO: This only tests C values at the moment.
 """
-from __future__ import absolute_import
-from PyObjCTools.TestSupport import *
 
-from .testbndl import pyObjCPy, carrayMaker
-from .testbndl import UCHAR_MAX
-from .testbndl import CHAR_MAX, CHAR_MIN
-from .testbndl import SCHAR_MAX, SCHAR_MIN
-from .testbndl import USHRT_MAX, SHRT_MAX, SHRT_MIN
-from .testbndl import UINT_MAX, INT_MAX, INT_MIN
-from .testbndl import ULONG_MAX, LONG_MAX, LONG_MIN
-from .testbndl import ULLONG_MAX, LLONG_MAX, LLONG_MIN
-from .testbndl import DBL_MAX, DBL_MIN, DBL_EPSILON
-from .testbndl import FLT_MAX, FLT_MIN, FLT_EPSILON
-import objc
+import array
+import sys
 import warnings
-import array, sys
+
+import objc
+from PyObjCTools.TestSupport import TestCase
+
+from .testbndl import (
+    CHAR_MAX,
+    CHAR_MIN,
+    DBL_EPSILON,
+    DBL_MAX,
+    DBL_MIN,
+    FLT_EPSILON,
+    FLT_MAX,
+    FLT_MIN,
+    INT_MAX,
+    INT_MIN,
+    LLONG_MAX,
+    LLONG_MIN,
+    LONG_MAX,
+    LONG_MIN,
+    SCHAR_MIN,
+    SHRT_MAX,
+    SHRT_MIN,
+    UCHAR_MAX,
+    UINT_MAX,
+    ULLONG_MAX,
+    ULONG_MAX,
+    USHRT_MAX,
+    carrayMaker,
+    pyObjCPy,
+)
 
 
 class TestNumbers(TestCase):
@@ -30,25 +48,20 @@ class TestNumbers(TestCase):
         self.assertEqual(0, pyObjCPy(objc._C_UCHR, 0))
         self.assertEqual(0, pyObjCPy(objc._C_UCHR, b"\0"))
 
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_UCHR, long(0)))
         self.assertEqual(0, pyObjCPy(objc._C_UCHR, float(0)))
 
         self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, UCHAR_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, chr(UCHAR_MAX)))
-        else:
-            self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, bytes([UCHAR_MAX])))
-            self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, bytearray([UCHAR_MAX])))
+        self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, bytes([UCHAR_MAX])))
+        self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, bytearray([UCHAR_MAX])))
 
-        if sys.version_info[0] == 2:
-            self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, long(UCHAR_MAX)))
         self.assertEqual(UCHAR_MAX, pyObjCPy(objc._C_UCHR, float(UCHAR_MAX)))
 
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore", category=DeprecationWarning)
 
-            self.assertRaises((IndexError, ValueError), pyObjCPy, objc._C_UCHR, SCHAR_MIN)
+            self.assertRaises(
+                (IndexError, ValueError), pyObjCPy, objc._C_UCHR, SCHAR_MIN
+            )
             self.assertRaises(
                 (IndexError, ValueError), pyObjCPy, objc._C_UCHR, SCHAR_MIN - 1
             )
@@ -56,19 +69,11 @@ class TestNumbers(TestCase):
     def test_char(self):
         self.assertEqual(0, pyObjCPy(objc._C_CHR, 0))
         self.assertEqual(0, pyObjCPy(objc._C_CHR, b"\x00"))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_CHR, long(0)))
 
         self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, CHAR_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, chr(CHAR_MAX)))
-        else:
-            self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, bytes([CHAR_MAX])))
-            self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, bytearray([CHAR_MAX])))
+        self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, bytes([CHAR_MAX])))
+        self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, bytearray([CHAR_MAX])))
         self.assertEqual(CHAR_MIN, pyObjCPy(objc._C_CHR, CHAR_MIN))
-        if sys.version_info[0] == 2:
-            self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, long(CHAR_MAX)))
-            self.assertEqual(CHAR_MIN, pyObjCPy(objc._C_CHR, long(CHAR_MIN)))
         self.assertEqual(CHAR_MAX, pyObjCPy(objc._C_CHR, float(CHAR_MAX)))
         self.assertEqual(CHAR_MIN, pyObjCPy(objc._C_CHR, float(CHAR_MIN)))
 
@@ -82,9 +87,6 @@ class TestNumbers(TestCase):
     def test_unsigned_short(self):
         self.assertEqual(0, pyObjCPy(objc._C_USHT, 0))
         self.assertEqual(USHRT_MAX, pyObjCPy(objc._C_USHT, USHRT_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_USHT, long(0)))
-            self.assertEqual(USHRT_MAX, pyObjCPy(objc._C_USHT, long(USHRT_MAX)))
         self.assertEqual(0, pyObjCPy(objc._C_USHT, float(0)))
         self.assertEqual(USHRT_MAX, pyObjCPy(objc._C_USHT, float(USHRT_MAX)))
 
@@ -101,10 +103,6 @@ class TestNumbers(TestCase):
         self.assertEqual(0, pyObjCPy(objc._C_SHT, 0))
         self.assertEqual(SHRT_MAX, pyObjCPy(objc._C_SHT, SHRT_MAX))
         self.assertEqual(SHRT_MIN, pyObjCPy(objc._C_SHT, SHRT_MIN))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_SHT, long(0)))
-            self.assertEqual(SHRT_MAX, pyObjCPy(objc._C_SHT, long(SHRT_MAX)))
-            self.assertEqual(SHRT_MIN, pyObjCPy(objc._C_SHT, long(SHRT_MIN)))
         self.assertEqual(0, pyObjCPy(objc._C_SHT, float(0)))
         self.assertEqual(SHRT_MAX, pyObjCPy(objc._C_SHT, float(SHRT_MAX)))
         self.assertEqual(SHRT_MIN, pyObjCPy(objc._C_SHT, float(SHRT_MIN)))
@@ -118,9 +116,6 @@ class TestNumbers(TestCase):
     def test_unsigned_int(self):
         self.assertEqual(0, pyObjCPy(objc._C_UINT, 0))
         self.assertEqual(UINT_MAX, pyObjCPy(objc._C_UINT, UINT_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_UINT, long(0)))
-            self.assertEqual(UINT_MAX, pyObjCPy(objc._C_UINT, long(UINT_MAX)))
         self.assertEqual(0, pyObjCPy(objc._C_UINT, float(0)))
         self.assertEqual(UINT_MAX, pyObjCPy(objc._C_UINT, float(UINT_MAX)))
 
@@ -137,10 +132,6 @@ class TestNumbers(TestCase):
         self.assertEqual(0, pyObjCPy(objc._C_INT, 0))
         self.assertEqual(INT_MAX, pyObjCPy(objc._C_INT, INT_MAX))
         self.assertEqual(INT_MIN, pyObjCPy(objc._C_INT, INT_MIN))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_INT, long(0)))
-            self.assertEqual(INT_MAX, pyObjCPy(objc._C_INT, long(INT_MAX)))
-            self.assertEqual(INT_MIN, pyObjCPy(objc._C_INT, long(INT_MIN)))
         self.assertEqual(0, pyObjCPy(objc._C_INT, float(0)))
         self.assertEqual(INT_MAX, pyObjCPy(objc._C_INT, float(INT_MAX)))
         self.assertEqual(INT_MIN, pyObjCPy(objc._C_INT, float(INT_MIN)))
@@ -155,9 +146,6 @@ class TestNumbers(TestCase):
     def test_unsigned_long(self):
         self.assertEqual(0, pyObjCPy(objc._C_ULNG, 0))
         self.assertEqual(ULONG_MAX, pyObjCPy(objc._C_ULNG, ULONG_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_ULNG, long(0)))
-            self.assertEqual(ULONG_MAX, pyObjCPy(objc._C_ULNG, long(ULONG_MAX)))
         self.assertEqual(0, pyObjCPy(objc._C_ULNG, float(0)))
 
         with warnings.catch_warnings():
@@ -175,10 +163,6 @@ class TestNumbers(TestCase):
         self.assertEqual(0, pyObjCPy(objc._C_LNG, 0))
         self.assertEqual(LONG_MAX, pyObjCPy(objc._C_LNG, LONG_MAX))
         self.assertEqual(LONG_MIN, pyObjCPy(objc._C_LNG, LONG_MIN))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_LNG, long(0)))
-            self.assertEqual(LONG_MAX, pyObjCPy(objc._C_LNG, long(LONG_MAX)))
-            self.assertEqual(LONG_MIN, pyObjCPy(objc._C_LNG, long(LONG_MIN)))
         self.assertEqual(0, pyObjCPy(objc._C_LNG, float(0)))
         if sys.maxsize < 2 ** 32:
             self.assertEqual(LONG_MAX, pyObjCPy(objc._C_LNG, float(LONG_MAX)))
@@ -193,9 +177,6 @@ class TestNumbers(TestCase):
     def test_unsigned_long_long(self):
         self.assertEqual(0, pyObjCPy(objc._C_ULNG_LNG, 0))
         self.assertEqual(ULLONG_MAX, pyObjCPy(objc._C_ULNG_LNG, ULLONG_MAX))
-        if sys.version_info[0] == 2:
-            self.assertEqual(0, pyObjCPy(objc._C_ULNG_LNG, long(0)))
-            self.assertEqual(ULLONG_MAX, pyObjCPy(objc._C_ULNG_LNG, long(ULLONG_MAX)))
         self.assertEqual(0, pyObjCPy(objc._C_ULNG_LNG, float(0)))
 
         with warnings.catch_warnings(record=True) as w:
@@ -243,7 +224,9 @@ class TestNumbers(TestCase):
         self.assertEqual(DBL_EPSILON, pyObjCPy(objc._C_DBL, DBL_EPSILON))
         self.assertEqual(-DBL_EPSILON, pyObjCPy(objc._C_DBL, -DBL_EPSILON))
 
-        self.assertRaises((OverflowError, ValueError), pyObjCPy, objc._C_DBL, 1 << 10000)
+        self.assertRaises(
+            (OverflowError, ValueError), pyObjCPy, objc._C_DBL, 1 << 10000
+        )
 
         self.assertRaises(ValueError, pyObjCPy, objc._C_DBL, "1")
         self.assertRaises(ValueError, pyObjCPy, objc._C_DBL, b"1")
@@ -262,7 +245,9 @@ class TestNumbers(TestCase):
         # of float
         self.assertNotEqual(DBL_MAX, pyObjCPy(objc._C_FLT, DBL_MAX))
 
-        self.assertRaises((ValueError, OverflowError), pyObjCPy, objc._C_FLT, 1 << 10000)
+        self.assertRaises(
+            (ValueError, OverflowError), pyObjCPy, objc._C_FLT, 1 << 10000
+        )
 
         self.assertRaises(ValueError, pyObjCPy, objc._C_FLT, "1")
         self.assertRaises(ValueError, pyObjCPy, objc._C_FLT, b"1")
@@ -558,7 +543,3 @@ class PyOCTestTypeStr(TestCase):
         self.assertEqual(objc._C_IN, b"n")
         self.assertEqual(objc._C_OUT, b"o")
         self.assertEqual(objc._C_INOUT, b"N")
-
-
-if __name__ == "__main__":
-    main()
