@@ -6,9 +6,11 @@ This file provides a nice unittest wrapper around the functions in that file,
 the code in this file defines a class CTests that has the functions in the
 unitest.m file as its methods.
 """
-import sys, platform
-from PyObjCTools.TestSupport import *
+import platform
+import sys
+
 import objc
+from PyObjCTools.TestSupport import TestCase
 
 ctests = objc._ctests
 
@@ -16,9 +18,9 @@ names = list(ctests.keys())
 methods = {}
 
 
-def do_exec(value, locals, globals):
+def do_exec(value, locals_dict, globals_dict):
 
-    exec(value, locals, globals)
+    exec(value, locals_dict, globals_dict)
 
 
 def make_test(name):
@@ -58,13 +60,18 @@ def test_%s(self):
         result,
     )
 
-    return result["test_%s" % (name,)]
+    return result[f"test_{name}"]
+
+
+def test_AlwaysFails(self):
+    with self.assertRaisesRegex(AssertionError, r"Modules/objc/ctests.m:\d+ 0 == 1"):
+        ctests["AlwaysFails"]()
 
 
 for n in names:
-    methods["test_%s" % (n,)] = make_test(n)
+    if n == "AlwaysFails":
+        methods[f"test_{n}"] = test_AlwaysFails
+    else:
+        methods[f"test_{n}"] = make_test(n)
 
 CTests = type(TestCase)("CTests", (TestCase,), methods)
-
-if __name__ == "__main__":
-    main()
