@@ -1,52 +1,54 @@
-from PyObjCTools.TestSupport import *
-
-import objc
-import sys
 import warnings
 
-#try:
-#    from  Foundation import NSRange
-#
-#    _C_NSRange = NSRange.__typestr__
-#
-#except ImportError:
-if 1:
-    if sys.maxsize > 2 ** 32:
-        _C_NSRange = b"{_NSRange=QQ}"
-    else:
-        _C_NSRange = b"{_NSRange=II}"
+import objc
+from PyObjCTools.TestSupport import TestCase
+
+_C_NSRange = b"{_NSRange=QQ}"
 
 
 NSObject = objc.lookUpClass("NSObject")
 
-class TestBasicDescriptors (TestCase):
+
+class TestBasicDescriptors(TestCase):
 
     # IBOutlet is tested in test_ivar
 
-    def test_ibaction(self):
+    def test_IB_DESIGNABLE(self):
+        @objc.IB_DESIGNABLE
+        class Foo:
+            pass
 
+        self.assertIsInstance(Foo, type)
+
+        v = 99999
+        self.assertIs(objc.IB_DESIGNABLE(v), v)
+
+    def test_IBInspectable(self):
+        v = 99999
+        self.assertIs(objc.IBInspectable(v), v)
+
+    def test_ibaction(self):
         @objc.IBAction
         def myAction_(self, sender):
             return 1
 
         self.assertIsInstance(myAction_, objc.selector)
-        self.assertEqual(myAction_.signature, b'v@:@')
-        self.assertEqual(myAction_.selector, b'myAction:')
+        self.assertEqual(myAction_.signature, b"v@:@")
+        self.assertEqual(myAction_.selector, b"myAction:")
         self.assertFalse(myAction_.isClassMethod)
 
-
         self.assertRaises(TypeError, objc.IBAction, None)
         self.assertRaises(TypeError, objc.IBAction, 42)
 
     def test_instancemethod(self):
-        class TestDescriptorsClass1 (NSObject):
+        class TestDescriptorsClass1(NSObject):
             @objc.instancemethod
             def new(self):
                 pass
 
         o = NSObject.alloc().init()
-        self.assertFalse(hasattr(o, 'new'))
-        self.assertTrue(hasattr(NSObject, 'new'))
+        self.assertFalse(hasattr(o, "new"))
+        self.assertTrue(hasattr(NSObject, "new"))
         self.assertTrue(NSObject.new.isClassMethod)
 
         o = TestDescriptorsClass1.alloc().init()
@@ -54,12 +56,10 @@ class TestBasicDescriptors (TestCase):
         self.assertIsInstance(m, objc.selector)
         self.assertFalse(m.isClassMethod)
 
-
         self.assertRaises(TypeError, objc.instancemethod, None)
         self.assertRaises(TypeError, objc.instancemethod, 42)
 
     def test_typedSelector(self):
-
         @objc.typedSelector(b"I@:qq")
         def mySelector_arg_(self, a, b):
             return 4
@@ -72,7 +72,7 @@ class TestBasicDescriptors (TestCase):
         self.assertRaises(TypeError, objc.typedSelector(b"v@:i"), 42)
 
     def testNamedSelector(self):
-        @objc.namedSelector(b'foo:bar:')
+        @objc.namedSelector(b"foo:bar:")
         def mymethod(self, a, b):
             pass
 
@@ -83,7 +83,7 @@ class TestBasicDescriptors (TestCase):
         self.assertRaises(TypeError, objc.namedSelector(b"foo:bar:"), None)
         self.assertRaises(TypeError, objc.namedSelector(b"foo:bar:"), 42)
 
-        @objc.namedSelector(b'foo:bar:', signature=b"q@:qq")
+        @objc.namedSelector(b"foo:bar:", signature=b"q@:qq")
         def mymethod(self, a, b):
             pass
 
@@ -98,7 +98,7 @@ class TestBasicDescriptors (TestCase):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore", category=DeprecationWarning)
 
-            @objc.namedselector(b'foo:bar:')
+            @objc.namedselector(b"foo:bar:")
             def mymethod(self, a, b):
                 pass
 
@@ -109,7 +109,7 @@ class TestBasicDescriptors (TestCase):
             self.assertRaises(TypeError, objc.namedselector(b"foo:bar:"), None)
             self.assertRaises(TypeError, objc.namedselector(b"foo:bar:"), 42)
 
-            @objc.namedselector(b'foo:bar:', signature=b"q@:qq")
+            @objc.namedselector(b"foo:bar:", signature=b"q@:qq")
             def mymethod(self, a, b):
                 pass
 
@@ -117,7 +117,9 @@ class TestBasicDescriptors (TestCase):
             self.assertEqual(mymethod.signature, b"q@:qq")
             self.assertEqual(mymethod.selector, b"foo:bar:")
 
-            self.assertRaises(TypeError, objc.namedselector(b"foo:bar:", b"q@:qq"), None)
+            self.assertRaises(
+                TypeError, objc.namedselector(b"foo:bar:", b"q@:qq"), None
+            )
             self.assertRaises(TypeError, objc.namedselector(b"foo:bar:", b"q@:qq"), 42)
 
     # synthesize is tested in test_synthesize
@@ -147,7 +149,6 @@ class TestBasicDescriptors (TestCase):
         self.assertEqual(isColor.signature, b"@@:")
         self.assertEqual(setColor_.signature, b"v@:@")
 
-
         # Indexed accessors
 
         @objc.accessor
@@ -163,10 +164,9 @@ class TestBasicDescriptors (TestCase):
             return ["sour", "sweet"]
 
         @objc.accessor
-        def getFlavors_range_(self, buffer, range):
+        def getFlavors_range_(self, bfr, rng):
             return ["sour", "sweet"]
 
-
         self.assertIsInstance(countOfFlavors, objc.selector)
         self.assertIsInstance(objectInFlavorsAtIndex_, objc.selector)
         self.assertIsInstance(flavorsAtIndexes_, objc.selector)
@@ -205,7 +205,6 @@ class TestBasicDescriptors (TestCase):
         def replaceFlavorsAtIndexes_withFlavors_(self, indices, values):
             pass
 
-
         self.assertIsInstance(insertObject_inFlavorsAtIndex_, objc.selector)
         self.assertIsInstance(insertFlavors_atIndexes_, objc.selector)
         self.assertIsInstance(removeObjectFromFlavorsAtIndex_, objc.selector)
@@ -213,14 +212,20 @@ class TestBasicDescriptors (TestCase):
         self.assertIsInstance(replaceObjectInFlavorsAtIndex_withObject_, objc.selector)
         self.assertIsInstance(replaceFlavorsAtIndexes_withFlavors_, objc.selector)
 
-        self.assertEqual(insertObject_inFlavorsAtIndex_.signature, b"v@:@" + objc._C_NSUInteger)
+        self.assertEqual(
+            insertObject_inFlavorsAtIndex_.signature, b"v@:@" + objc._C_NSUInteger
+        )
         self.assertEqual(insertFlavors_atIndexes_.signature, b"v@:@@")
-        self.assertEqual(removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger)
+        self.assertEqual(
+            removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger
+        )
         self.assertEqual(removeFlavorsAtIndexes_.signature, b"v@:@")
-        self.assertEqual(replaceObjectInFlavorsAtIndex_withObject_.signature, b"v@:" + objc._C_NSUInteger + b"@")
+        self.assertEqual(
+            replaceObjectInFlavorsAtIndex_withObject_.signature,
+            b"v@:" + objc._C_NSUInteger + b"@",
+        )
         self.assertEqual(replaceFlavorsAtIndexes_withFlavors_.signature, b"v@:@@")
 
-
         # Getter Unordered Accessors
         @objc.accessor
         def countOfLanguages(self):
@@ -270,48 +275,53 @@ class TestBasicDescriptors (TestCase):
             return (False, None)
 
         self.assertIsInstance(validateColor_error_, objc.selector)
-        self.assertEqual(validateColor_error_.signature, objc._C_NSBOOL + b'@:N^@o^@')
-
-
+        self.assertEqual(validateColor_error_.signature, objc._C_NSBOOL + b"@:N^@o^@")
 
         # Keyword arguments (**kwds) and varargs (*args) are
         # not supported:
 
         def attrib(self, *args):
             pass
+
         self.assertRaises(TypeError, objc.accessor, attrib)
 
         def attrib(self, **kwds):
             pass
+
         self.assertRaises(TypeError, objc.accessor, attrib)
 
         # Not really an accessor
         def attrib_error_(self, a, b):
             pass
+
         self.assertRaises(TypeError, objc.accessor, attrib_error_)
 
         # Argument counts that don't match
         def validateObject_error_(self, a):
             pass
+
         self.assertRaises(TypeError, objc.accessor, validateObject_error_)
 
         def validateObject_error_(self, a, b, c):
             pass
+
         self.assertRaises(TypeError, objc.accessor, validateObject_error_)
 
         def validateObject_error_(self, a, b, c, d=1, e=2):
             pass
+
         self.assertRaises(TypeError, objc.accessor, validateObject_error_)
 
         @objc.accessor
         def validateObject_error_(self, a, b, c=1):
             pass
 
-
         def countOfFoo_withBar_withBaz_withNone_(self, foo, bar, baz, none):
             pass
-        self.assertRaises(TypeError, objc.accessor, countOfFoo_withBar_withBaz_withNone_)
 
+        self.assertRaises(
+            TypeError, objc.accessor, countOfFoo_withBar_withBaz_withNone_
+        )
 
     def test_typedAccessor(self):
         # NOTE: the optional type argument is tested through the typedAccessor function
@@ -340,7 +350,6 @@ class TestBasicDescriptors (TestCase):
         self.assertEqual(isColor.signature, mytype + b"@:")
         self.assertEqual(setColor_.signature, b"v@:" + mytype)
 
-
         # Indexed accessors
 
         @objc.typedAccessor(mytype)
@@ -356,18 +365,19 @@ class TestBasicDescriptors (TestCase):
             return ["sour", "sweet"]
 
         @objc.typedAccessor(mytype)
-        def getFlavors_range_(self, buffer, range):
+        def getFlavors_range_(self, bfr, rng):
             return ["sour", "sweet"]
 
-
         self.assertIsInstance(countOfFlavors, objc.selector)
         self.assertIsInstance(objectInFlavorsAtIndex_, objc.selector)
         self.assertIsInstance(flavorsAtIndexes_, objc.selector)
         self.assertIsInstance(getFlavors_range_, objc.selector)
 
         self.assertEqual(countOfFlavors.signature, objc._C_NSUInteger + b"@:")
-        self.assertEqual(objectInFlavorsAtIndex_.signature, mytype + b"@:" + objc._C_NSUInteger)
-        self.assertEqual(flavorsAtIndexes_.signature, b"@@:@") #XXX: is this correct?
+        self.assertEqual(
+            objectInFlavorsAtIndex_.signature, mytype + b"@:" + objc._C_NSUInteger
+        )
+        self.assertEqual(flavorsAtIndexes_.signature, b"@@:@")  # XXX: is this correct?
 
         # XXX: This needs even more work: also have to add custom metadata!
         self.assertEqual(getFlavors_range_.signature, b"v@:o^@" + _C_NSRange)
@@ -398,7 +408,6 @@ class TestBasicDescriptors (TestCase):
         def replaceFlavorsAtIndexes_withFlavors_(self, indices, values):
             pass
 
-
         self.assertIsInstance(insertObject_inFlavorsAtIndex_, objc.selector)
         self.assertIsInstance(insertFlavors_atIndexes_, objc.selector)
         self.assertIsInstance(removeObjectFromFlavorsAtIndex_, objc.selector)
@@ -406,13 +415,24 @@ class TestBasicDescriptors (TestCase):
         self.assertIsInstance(replaceObjectInFlavorsAtIndex_withObject_, objc.selector)
         self.assertIsInstance(replaceFlavorsAtIndexes_withFlavors_, objc.selector)
 
-        self.assertEqual(insertObject_inFlavorsAtIndex_.signature, b"v@:" + mytype + objc._C_NSUInteger)
-        self.assertEqual(insertFlavors_atIndexes_.signature, b"v@:@@") # XXX: is this correct?
-        self.assertEqual(removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger)
+        self.assertEqual(
+            insertObject_inFlavorsAtIndex_.signature,
+            b"v@:" + mytype + objc._C_NSUInteger,
+        )
+        self.assertEqual(
+            insertFlavors_atIndexes_.signature, b"v@:@@"
+        )  # XXX: is this correct?
+        self.assertEqual(
+            removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger
+        )
         self.assertEqual(removeFlavorsAtIndexes_.signature, b"v@:@")
-        self.assertEqual(replaceObjectInFlavorsAtIndex_withObject_.signature, b"v@:" + objc._C_NSUInteger + mytype)
-        self.assertEqual(replaceFlavorsAtIndexes_withFlavors_.signature, b"v@:@@") # XXX: is this correct?
-
+        self.assertEqual(
+            replaceObjectInFlavorsAtIndex_withObject_.signature,
+            b"v@:" + objc._C_NSUInteger + mytype,
+        )
+        self.assertEqual(
+            replaceFlavorsAtIndexes_withFlavors_.signature, b"v@:@@"
+        )  # XXX: is this correct?
 
         # Getter Unordered Accessors
         @objc.typedAccessor(mytype)
@@ -454,7 +474,7 @@ class TestBasicDescriptors (TestCase):
         self.assertIsInstance(intersectLanguagues_, objc.selector)
 
         self.assertEqual(addLanguagesObject_.signature, b"v@:" + mytype)
-        self.assertEqual(addLanguagues_.signature, b"v@:@") # XXX: is this correct?
+        self.assertEqual(addLanguagues_.signature, b"v@:@")  # XXX: is this correct?
         self.assertEqual(intersectLanguagues_.signature, b"v@:@")
 
         # Validation
@@ -463,7 +483,7 @@ class TestBasicDescriptors (TestCase):
             return (False, None)
 
         self.assertIsInstance(validateColor_error_, objc.selector)
-        self.assertEqual(validateColor_error_.signature, objc._C_NSBOOL + b'@:N^@o^@')
+        self.assertEqual(validateColor_error_.signature, objc._C_NSBOOL + b"@:N^@o^@")
 
     def test_Accessor(self):
         with warnings.catch_warnings():
@@ -492,7 +512,6 @@ class TestBasicDescriptors (TestCase):
             self.assertEqual(isColor.signature, b"@@:")
             self.assertEqual(setColor_.signature, b"v@:@")
 
-
             # Indexed accessors
 
             @objc.Accessor
@@ -508,17 +527,18 @@ class TestBasicDescriptors (TestCase):
                 return ["sour", "sweet"]
 
             @objc.Accessor
-            def getFlavors_range_(self, buffer, range):
+            def getFlavors_range_(self, buffer, rng):
                 return ["sour", "sweet"]
 
-
             self.assertIsInstance(countOfFlavors, objc.selector)
             self.assertIsInstance(objectInFlavorsAtIndex_, objc.selector)
             self.assertIsInstance(flavorsAtIndexes_, objc.selector)
             self.assertIsInstance(getFlavors_range_, objc.selector)
 
             self.assertEqual(countOfFlavors.signature, objc._C_NSUInteger + b"@:")
-            self.assertEqual(objectInFlavorsAtIndex_.signature, b"@@:" + objc._C_NSUInteger)
+            self.assertEqual(
+                objectInFlavorsAtIndex_.signature, b"@@:" + objc._C_NSUInteger
+            )
             self.assertEqual(flavorsAtIndexes_.signature, b"@@:@")
 
             # XXX: This needs even more work: also have to add custom metadata!
@@ -550,22 +570,29 @@ class TestBasicDescriptors (TestCase):
             def replaceFlavorsAtIndexes_withFlavors_(self, indices, values):
                 pass
 
-
             self.assertIsInstance(insertObject_inFlavorsAtIndex_, objc.selector)
             self.assertIsInstance(insertFlavors_atIndexes_, objc.selector)
             self.assertIsInstance(removeObjectFromFlavorsAtIndex_, objc.selector)
             self.assertIsInstance(removeFlavorsAtIndexes_, objc.selector)
-            self.assertIsInstance(replaceObjectInFlavorsAtIndex_withObject_, objc.selector)
+            self.assertIsInstance(
+                replaceObjectInFlavorsAtIndex_withObject_, objc.selector
+            )
             self.assertIsInstance(replaceFlavorsAtIndexes_withFlavors_, objc.selector)
 
-            self.assertEqual(insertObject_inFlavorsAtIndex_.signature, b"v@:@" + objc._C_NSUInteger)
+            self.assertEqual(
+                insertObject_inFlavorsAtIndex_.signature, b"v@:@" + objc._C_NSUInteger
+            )
             self.assertEqual(insertFlavors_atIndexes_.signature, b"v@:@@")
-            self.assertEqual(removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger)
+            self.assertEqual(
+                removeObjectFromFlavorsAtIndex_.signature, b"v@:" + objc._C_NSUInteger
+            )
             self.assertEqual(removeFlavorsAtIndexes_.signature, b"v@:@")
-            self.assertEqual(replaceObjectInFlavorsAtIndex_withObject_.signature, b"v@:" + objc._C_NSUInteger + b"@")
+            self.assertEqual(
+                replaceObjectInFlavorsAtIndex_withObject_.signature,
+                b"v@:" + objc._C_NSUInteger + b"@",
+            )
             self.assertEqual(replaceFlavorsAtIndexes_withFlavors_.signature, b"v@:@@")
 
-
             # Getter Unordered Accessors
             @objc.Accessor
             def countOfLanguages(self):
@@ -615,37 +642,43 @@ class TestBasicDescriptors (TestCase):
                 return (False, None)
 
             self.assertIsInstance(validateColor_error_, objc.selector)
-            self.assertEqual(validateColor_error_.signature, objc._C_NSBOOL + b'@:N^@o^@')
-
-
+            self.assertEqual(
+                validateColor_error_.signature, objc._C_NSBOOL + b"@:N^@o^@"
+            )
 
             # Keyword arguments (**kwds) and varargs (*args) are
             # not supported:
 
             def attrib(self, *args):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, attrib)
 
             def attrib(self, **kwds):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, attrib)
 
             # Not really an accessor
             def attrib_error_(self, a, b):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, attrib_error_)
 
             # Argument counts that don't match
             def validateObject_error_(self, a):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, validateObject_error_)
 
             def validateObject_error_(self, a, b, c):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, validateObject_error_)
 
             def validateObject_error_(self, a, b, c, d=1, e=2):
                 pass
+
             self.assertRaises(TypeError, objc.Accessor, validateObject_error_)
 
             @objc.Accessor
@@ -661,16 +694,13 @@ class TestBasicDescriptors (TestCase):
                 pass
 
             self.assertIsInstance(myMethod_arg_, objc.selector)
-            self.assertEqual(myMethod_arg_.signature, b'd@:ii')
-            self.assertEqual(myMethod_arg_.selector, b'myMethod:arg:')
+            self.assertEqual(myMethod_arg_.signature, b"d@:ii")
+            self.assertEqual(myMethod_arg_.selector, b"myMethod:arg:")
 
-            @objc.signature(b'q@:@q', selector=b'foo:bar:')
+            @objc.signature(b"q@:@q", selector=b"foo:bar:")
             def method(self, a, b):
                 pass
 
             self.assertIsInstance(method, objc.selector)
-            self.assertEqual(method.signature, b'q@:@q')
-            self.assertEqual(method.selector, b'foo:bar:')
-
-if __name__ == "__main__":
-    main()
+            self.assertEqual(method.signature, b"q@:@q")
+            self.assertEqual(method.selector, b"foo:bar:")
