@@ -10,13 +10,15 @@ TODO:
 - Do the same for python 2.x
 - Do the same for sets (NSSet) and lists (NSArray)
 """
-from PyObjCTools.TestSupport import *
+import collections.abc
 import operator
-import objc
 
 # Import some of the stdlib tests
 from test import mapping_tests
 
+import objc
+from PyObjCTools.TestSupport import TestCase
+
 NSDictionary = objc.lookUpClass("NSDictionary")
 NSMutableDictionary = objc.lookUpClass("NSMutableDictionary")
 
@@ -76,7 +78,9 @@ class TestNSDictionaryInterface(TestCase):
         s = {"c", "b"} ^ d.keys()
         self.assertEqual(s, {"a", "c"})
 
-        self.assertEqual(repr(self.createDictionary(a=1).keys()), "<nsdict_keys(['a'])>")
+        self.assertEqual(
+            repr(self.createDictionary(a=1).keys()), "<nsdict_keys(['a'])>"
+        )
 
     def testValues(self):
         d = self.createDictionary()
@@ -172,7 +176,7 @@ class TestNSDictionaryInterface(TestCase):
         class Exc(Exception):
             pass
 
-        class BadCmp(object):
+        class BadCmp:
             def __eq__(self, other):
                 raise Exc()
 
@@ -183,7 +187,7 @@ class TestNSDictionaryInterface(TestCase):
         d2 = {BadCmp(): 1}
 
         with self.assertRaises(Exc):
-            d1 == d2
+            d1 == d2  # noqa: B015
 
     def testKeysContained(self):
         self.helper_keys_contained(lambda x: x.keys())
@@ -243,19 +247,19 @@ class TestNSDictionaryInterface(TestCase):
         d2 = self.dictClass().dictionaryWithDictionary_({1: C()})
 
         with self.assertRaises(RuntimeError):
-            d1.items() == d2.items()
+            d1.items() == d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() != d2.items()
+            d1.items() != d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() <= d2.items()
+            d1.items() <= d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() >= d2.items()
+            d1.items() >= d2.items()  # noqa: B015
 
         d3 = self.dictClass().dictionaryWithDictionary_({1: C(), 2: C()})
         with self.assertRaises(RuntimeError):
-            d2.items() < d3.items()
+            d2.items() < d3.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d3.items() > d2.items()
+            d3.items() > d2.items()  # noqa: B015
 
         k1 = self.dictClass().dictionaryWithDictionary_({1: 1, 2: 2}).keys()
         k2 = self.dictClass().dictionaryWithDictionary_({1: 1, 2: 2, 3: 3}).keys()
@@ -288,15 +292,21 @@ class TestNSDictionaryInterface(TestCase):
 
     def testDictviewMixedSetOperations(self):
         # Just a few for .keys()
-        self.assertTrue(self.dictClass().dictionaryWithDictionary_({1: 1}).keys() == {1})
-        self.assertTrue({1} == self.dictClass().dictionaryWithDictionary_({1: 1}).keys())
+        self.assertTrue(
+            self.dictClass().dictionaryWithDictionary_({1: 1}).keys() == {1}
+        )
+        self.assertTrue(
+            {1} == self.dictClass().dictionaryWithDictionary_({1: 1}).keys()
+        )
         self.assertEqual(
             self.dictClass().dictionaryWithDictionary_({1: 1}).keys() | {2}, {1, 2}
         )
         self.assertEqual(
             {2} | self.dictClass().dictionaryWithDictionary_({1: 1}).keys(), {1, 2}
         )
-        self.assertFalse(self.dictClass().dictionaryWithDictionary_({1: 1}).keys() == [1])
+        self.assertFalse(
+            self.dictClass().dictionaryWithDictionary_({1: 1}).keys() == [1]
+        )
 
         # And a few for .items()
         self.assertTrue(
@@ -306,10 +316,12 @@ class TestNSDictionaryInterface(TestCase):
             {(1, 1)} == self.dictClass().dictionaryWithDictionary_({1: 1}).items()
         )
         self.assertEqual(
-            self.dictClass().dictionaryWithDictionary_({1: 1}).items() | {2}, {(1, 1), 2}
+            self.dictClass().dictionaryWithDictionary_({1: 1}).items() | {2},
+            {(1, 1), 2},
         )
         self.assertEqual(
-            {2} | self.dictClass().dictionaryWithDictionary_({1: 1}).items(), {(1, 1), 2}
+            {2} | self.dictClass().dictionaryWithDictionary_({1: 1}).items(),
+            {(1, 1), 2},
         )
         self.assertFalse(
             self.dictClass().dictionaryWithDictionary_({1: 1}).items() == [(1, 1)]
@@ -386,7 +398,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         class Exc(Exception):
             pass
 
-        class BadEq(object):
+        class BadEq:
             def __eq__(self, other):
                 raise Exc()
 
@@ -397,7 +409,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         d[BadEq()] = 42
         self.assertRaises(KeyError, d.__getitem__, 23)
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -504,7 +516,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
 
         self.assertRaises(Exc, d.update, FailingUserDict())
 
-        class badseq(object):
+        class badseq:
             def __iter__(self):
                 return self
 
@@ -528,7 +540,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         class Exc(Exception):
             pass
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -556,7 +568,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
                         b[repr(i)] = i
                 if copymode > 0:
                     b = a.mutableCopy()
-                for i in range(size):
+                for _ in range(size):
                     ka, va = ta = a.popitem()
                     self.assertEqual(va, int(ka))
                     kb, vb = tb = b.popitem()
@@ -588,7 +600,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         class Exc(Exception):
             pass
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -715,9 +727,6 @@ class TestDictUpdates(TestCase):
         self.do_test(NSMutableDictionary)
 
 
-import collections.abc
-
-
 class TestABC(TestCase):
     def testDictABC(self):
         self.assertTrue(issubclass(NSDictionary, collections.abc.Mapping))
@@ -797,7 +806,7 @@ class TestPyObjCDict(TestCase):
             self.assertEqual(v["b"], "world")
 
     def test_values(self):
-        py = dict(a=4, b=3)
+        py = {"a": 4, "b": 3}
         oc = NSDictionary(a=4, b=3)
 
         self.assertIn(4, py.values())
@@ -836,7 +845,3 @@ class TestPyObjCDict(TestCase):
         self.assertRaises(TypeError, operator.le, oc.keys(), ("a", "f"))
         self.assertRaises(TypeError, operator.gt, oc.keys(), ("a", "f"))
         self.assertRaises(TypeError, operator.ge, oc.keys(), ("a", "f"))
-
-
-if __name__ == "__main__":
-    main()
