@@ -6,7 +6,7 @@ NOTE: this file is very, very incomplete and just tests copying at the moment.
 
 import objc
 from PyObjCTest.pythonset import OC_TestSet
-from PyObjCTools.TestSupport import TestCase, main
+from PyObjCTools.TestSupport import TestCase
 
 OC_PythonDictionary = objc.lookUpClass("OC_PythonDictionary")
 OC_BuiltinPythonDictionary = objc.lookUpClass("OC_BuiltinPythonDictionary")
@@ -43,7 +43,3 @@ class TestDictionary(TestCase):
         self.assertEqual(dict(s), o)
         self.assertIsNot(s, o)
         self.assertIsInstance(o, dict)
-
-
-if __name__ == "__main__":
-    main()
