@@ -18,7 +18,7 @@ class TestInspectSignatures(TestCase):
                     value = None
 
                 if value is None:
-                    self.fail("No inspect.signature for %s" % (nm,))
+                    self.fail(f"No inspect.signature for {nm}")
 
     @min_python_release("3.4")
     def test_class_signature(self):
@@ -49,6 +49,7 @@ class TestInspectSignatures(TestCase):
                     "__abstractmethods__",
                     "__prepare__",
                     "__init_subclass__",
+                    "__annotations__",
                 ):
                     continue
                 obj = getattr(cls, nm)
@@ -64,4 +65,4 @@ class TestInspectSignatures(TestCase):
                             continue
 
                     if value is None:
-                        self.fail("No inspect.signature for %s.%s" % (cls.__name__, nm))
+                        self.fail(f"No inspect.signature for {cls.__name__}.{nm}")
