@@ -1,12 +1,5 @@
-# FIXME: This test suite seems to polute it's environment, other tests fail
-# when this test suite is active!
-from __future__ import unicode_literals
-from PyObjCTools.TestSupport import *
-import sys
-
-from PyObjCTools.TestSupport import onlyPython2
-
 import objc
+from PyObjCTools.TestSupport import TestCase
 
 NSObject = objc.lookUpClass("NSObject")
 
@@ -54,13 +47,13 @@ class TestFromObjCSuperToObjCClass(TestCase):
 
         orig_classAddMethods = mod.classAddMethods
         try:
-            l = []
+            lst = []
 
             def classAddMethods(cls, values):
                 self.assertIsInstance(cls, objc.objc_class)
                 for item in values:
                     self.assertIsInstance(item, objc.selector)
-                l.append((cls, values))
+                lst.append((cls, values))
 
             mod.classAddMethods = classAddMethods
 
@@ -68,10 +61,10 @@ class TestFromObjCSuperToObjCClass(TestCase):
                 return "foo the bar"
 
             objc.classAddMethod(NSObject, b"python_description", my_python_description)
-            self.assertEqual(len(l), 1)
-            self.assertIs(l[0][0], NSObject)
-            self.assertEqual(len(l[0][1]), 1)
-            m = l[0][1][0]
+            self.assertEqual(len(lst), 1)
+            self.assertIs(lst[0][0], NSObject)
+            self.assertEqual(len(lst[0][1]), 1)
+            m = lst[0][1][0]
             self.assertIsInstance(m, objc.selector)
             self.assertIs(m.callable, my_python_description)
             self.assertEqual(m.selector, b"python_description")
@@ -81,11 +74,11 @@ class TestFromObjCSuperToObjCClass(TestCase):
             def myAction(self):
                 return 1
 
-            l[:] = []
+            lst[:] = []
             objc.classAddMethod(NSObject, b"value", myAction)
-            self.assertIs(l[0][0], NSObject)
-            self.assertEqual(len(l[0][1]), 1)
-            m = l[0][1][0]
+            self.assertIs(lst[0][0], NSObject)
+            self.assertEqual(len(lst[0][1]), 1)
+            m = lst[0][1][0]
             self.assertIsInstance(m, objc.selector)
             self.assertIs(m.callable, myAction.callable)
             self.assertEqual(m.selector, b"value")
@@ -170,13 +163,13 @@ class TestFromObjCSuperToObjCClass(TestCase):
 
     def testAddedMethodType(self):
         def anotherNewClassMethod(cls):
-            "CLS DOC STRING"
+            """CLS DOC STRING"""
             return "BAR CLS"
 
         anotherNewClassMethod = classmethod(anotherNewClassMethod)
 
         def anotherNewMethod(self):
-            "INST DOC STRING"
+            """INST DOC STRING"""
             return "BAR SELF"
 
         self.assertTrue(
@@ -201,61 +194,14 @@ class TestFromObjCSuperToObjCClass(TestCase):
 
 
 class TestFromPythonClassToObjCClass(TestCase):
-    @onlyPython2
-    def testPythonSourcedMethods(self):
-        # 20031227, Ronald: Assigning the methods works alright, but actually
-        # using them won't because the new methods are actually still methods
-        # of a different class and will therefore complain about the type
-        # of 'self'.
-        objc.classAddMethods(
-            MEClass,
-            [PurePython.description, PurePython.newMethod, PurePython.purePythonMethod],
-        )
-
-        self.assertTrue(MEClass.instancesRespondToSelector_("description"))
-        self.assertTrue(MEClass.instancesRespondToSelector_("newMethod"))
-        self.assertTrue(MEClass.instancesRespondToSelector_("purePythonMethod"))
-
-        newInstance = MEClass.new()
-
-        # This is bogus, see above:
-        # self.assertEqual(newInstance.description(), "<pure>")
-        # self.assertEqual(newInstance.newMethod(), "<pure-new>")
-        # self.assertEqual(newInstance.purePythonMethod(), "<pure-py>")
-
-        # self.assertEqual(preEverythingInstance.description(), "<pure>")
-        # self.assertEqual(preEverythingInstance.newMethod(), "<pure-new>")
-        # self.assertEqual(preEverythingInstance.purePythonMethod(), "<pure-py>")
-
-        self.assertRaises(TypeError, newInstance.description)
-        self.assertRaises(TypeError, newInstance.newMethod)
-        self.assertRaises(TypeError, newInstance.purePythonMethod)
-        self.assertRaises(TypeError, preEverythingInstance.description)
-        self.assertRaises(TypeError, preEverythingInstance.newMethod)
-        self.assertRaises(TypeError, preEverythingInstance.purePythonMethod)
-
     def testPythonSourcedFunctions(self):
         # Same as testPythonSourcedMethods, but using function objects instead
         # of method objects.
 
-        if sys.version_info[0] == 2:
-            objc.classAddMethods(
-                MEClass,
-                [
-                    PurePython.description.im_func,
-                    PurePython.newMethod.im_func,
-                    PurePython.purePythonMethod.im_func,
-                ],
-            )
-        else:
-            objc.classAddMethods(
-                MEClass,
-                [
-                    PurePython.description,
-                    PurePython.newMethod,
-                    PurePython.purePythonMethod,
-                ],
-            )
+        objc.classAddMethods(
+            MEClass,
+            [PurePython.description, PurePython.newMethod, PurePython.purePythonMethod],
+        )
 
         self.assertTrue(MEClass.instancesRespondToSelector_("description"))
         self.assertTrue(MEClass.instancesRespondToSelector_("newMethod"))
@@ -371,12 +317,12 @@ class TestCategory(TestCase):
         self.assertTrue(not o.myCategoryMethod2())
 
     def testCategoryMultipleInheritance(self):
-
         NSObject = objc.lookUpClass("NSObject")
+        object_class = object
 
         try:
 
-            class NSObject(objc.Category(NSObject), object):
+            class NSObject(objc.Category(NSObject), object_class):
                 pass
 
             self.fail("Can use objc.Category with MI")
@@ -398,7 +344,7 @@ class TestCategory(TestCase):
         try:
             global list
 
-            class list(objc.Category(list)):
+            class list(objc.Category(list)):  # noqa: A001
                 pass
 
             self.fail("Category on list???")
@@ -465,7 +411,3 @@ class TestCategory(TestCase):
         self.assertHasAttr(NSObjectCat3, "classValue")
         o = NSObjectCat3.alloc().init()
         self.assertEqual(o.getClassValue(), "aap")
-
-
-if __name__ == "__main__":
-    main()
