@@ -1,29 +1,30 @@
-from __future__ import unicode_literals
-from PyObjCTools.TestSupport import *
-from PyObjCTest import structargs
-from PyObjCTest import testbndl
-from PyObjCTest import copying
-
-import objc, sys
 import functools
+import sys
+import gc
 import warnings
+
 import objc
-from PyObjCTest.fnd import NSObject, NSAutoreleasePool
+from PyObjCTest import copying, structargs
+from PyObjCTest import testbndl  # noqa: F401
+from PyObjCTest.fnd import NSAutoreleasePool, NSObject
+from PyObjCTest.testbndl import OC_TestClass1
+from PyObjCTest.properties import OCPropertyDefinitions
+from PyObjCTools.TestSupport import TestCase
 
 rct = structargs.StructArgClass.someRect.__metadata__()["retval"]["type"]
 
 
 class OCTestRegrWithGetItem(NSObject):
     def objectForKey_(self, k):
-        return "ofk: %s" % (k,)
+        return f"ofk: {k}"
 
     def __getitem__(self, k):
-        return "gi: %s" % (k,)
+        return f"gi: {k}"
 
 
 class OCTestRegrWithGetItem2(OCTestRegrWithGetItem):
     def objectForKey_(self, k):
-        return "ofk2: %s" % (k,)
+        return f"ofk2: {k}"
 
 
 class ReturnAStruct(NSObject):
@@ -47,8 +48,7 @@ class TestRegressions(TestCase):
         self.assertIsNot(oc, oc2)
         self.assertEqual(oc, py)
 
-        if sys.version_info[0] == 3:
-            self.assertEqual(py, oc)
+        self.assertEqual(py, oc)
 
     def testNSObjectPerforming(self):
         o = NSObject.performSelector_("new")
@@ -72,8 +72,6 @@ class TestRegressions(TestCase):
 
         # Check that we generate a warning for unitialized objects that
         # get deallocated
-        import sys
-        import gc
 
         with warnings.catch_warnings(record=True) as w:
             warnings.filterwarnings("always")
@@ -116,10 +114,10 @@ class TestRegressions(TestCase):
             def f(cls):
                 pass
 
-        object = NSObject.alloc().init()
+        anObject = NSObject.alloc().init()
 
-        self.assertRaises(TypeError, object.description, None)
-        self.assertRaises(TypeError, object.description, "twelf")
+        self.assertRaises(TypeError, anObject.description, None)
+        self.assertRaises(TypeError, anObject.description, "twelf")
         self.assertRaises(TypeError, NSObject.description, None)
         self.assertRaises(TypeError, ClsIsNone.f, None)
 
@@ -160,9 +158,9 @@ class TestRegressions(TestCase):
             def initialize(cls):
                 calls.append(repr(cls))
 
-        o = InitializeTestClass.new()
+        InitializeTestClass.new()
         self.assertEqual(len(calls), 1)
-        o = InitializeTestClass.new()
+        InitializeTestClass.new()
         self.assertEqual(len(calls), 1)
 
     def testStructReturnPy(self):
@@ -339,11 +337,40 @@ class TestDocTestProblem(TestCase):
         f = doctest.DocTestFinder(verbose=False)
         f.find(DocTestHelper)
 
-class TestNSDataCreationIssue (TestCase):
+
+class TestNSDataCreationIssue(TestCase):
     def test_issue271(self):
         cls = objc.lookUpClass("NSData")
 
         cls.alloc().initWithData_(b"hello")
 
-if __name__ == "__main__":
-    main()
+
+class TestTypedefedClass(TestCase):
+    # Issue #298, see description in Modules/objc/module.m
+    def test_typedefed(self):
+        v = OC_TestClass1.alloc().init()
+
+        o = v.returnObjectValue_(1)
+        self.assertIsInstance(o, objc.lookUpClass("NSObject"))
+
+        o = v.returnObjectValue_(2)
+        self.assertIsInstance(o, objc.lookUpClass("NSArray"))
+
+        o = v.returnObjectValue_(3)
+        self.assertIsInstance(o, objc.lookUpClass("NSDictionary"))
+
+    def test_property(self):
+        v = OCPropertyDefinitions.alloc().init()
+
+        o = v.parent()
+        self.assertIsInstance(o, objc.lookUpClass("NSArray"))
+
+
+class TestReboundMethod(TestCase):
+    # Issue #399
+    def test_rebound(self):
+        class Foo:
+            alloc = NSObject.alloc
+
+        f = Foo()
+        f.alloc().init()
