@@ -7,16 +7,21 @@ XXX: Add tests that check that the type actually works as expected:
 Add tests for nested structs as well (that is assert that NSRect.location is
 an NSPoint, but using our own types)
 """
-from PyObjCTools.TestSupport import *
-import objc
 import sys
-from PyObjCTest.structs import *
-from PyObjCTest.fnd import NSObject
+import warnings
 
-if sys.maxsize > 2 ** 32:
-    PTR_SIZE = 8
-else:
-    PTR_SIZE = 4
+import objc
+import pickle
+from PyObjCTest.fnd import NSObject
+from PyObjCTest.structs import OC_StructTest
+from PyObjCTools.TestSupport import TestCase, pyobjc_options, expectedFailure
+
+PTR_SIZE = 8
+
+# Used in the pickle test, and therefore needs a fully qualified name
+GlobalType = objc.createStructType(
+    "PyObjCTest.test_structs.GlobalType", b"{_GlobalType=ff}", ["a", "b"]
+)
 
 
 class TestStructs(TestCase):
@@ -48,7 +53,9 @@ class TestStructs(TestCase):
     def testNamedTupleAPI(self):
         Point = objc.createStructType("OCPoint", b"{_OCPoint=dd}", ["x", "y"])
         Line = objc.createStructType(
-            "OCLine", b"{_OCLine={_OCPoint=dd}{_OCPoint=dd}}d", ["start", "stop", "width"]
+            "OCLine",
+            b"{_OCLine={_OCPoint=dd}{_OCPoint=dd}}d",
+            ["start", "stop", "width"],
         )
 
         self.assertEqual(Point._fields, ("x", "y"))
@@ -60,35 +67,43 @@ class TestStructs(TestCase):
 
         self.assertEqual(p._asdict(), {"x": 3.0, "y": 4.0})
 
+        with self.assertRaisesRegex(
+            TypeError, "_replace called with positional arguments"
+        ):
+            p._replace(42)
+
+        with self.assertRaises(AttributeError):
+            p._replace(invalid=9)
+
         p2 = p._replace(y=5)
         self.assertEqual(p.x, 3.0)
         self.assertEqual(p.y, 4.0)
         self.assertEqual(p2.x, 3.0)
         self.assertEqual(p2.y, 5)
 
-        l = Line(Point(1, 2), Point(8, 9), 7)
-        self.assertEqual(l.start.x, 1.0)
-        self.assertEqual(l.start.y, 2.0)
-        self.assertEqual(l.stop.x, 8.0)
-        self.assertEqual(l.stop.y, 9.0)
-        self.assertEqual(l.width, 7.0)
+        ln = Line(Point(1, 2), Point(8, 9), 7)
+        self.assertEqual(ln.start.x, 1.0)
+        self.assertEqual(ln.start.y, 2.0)
+        self.assertEqual(ln.stop.x, 8.0)
+        self.assertEqual(ln.stop.y, 9.0)
+        self.assertEqual(ln.width, 7.0)
 
         self.assertEqual(
-            l._asdict(), {"start": Point(1, 2), "stop": Point(8, 9), "width": 7.0}
+            ln._asdict(), {"start": Point(1, 2), "stop": Point(8, 9), "width": 7.0}
         )
 
-        l2 = l._replace(stop=Point(3, 4), width=0.5)
-        self.assertEqual(l.start.x, 1.0)
-        self.assertEqual(l.start.y, 2.0)
-        self.assertEqual(l.stop.x, 8.0)
-        self.assertEqual(l.stop.y, 9.0)
-        self.assertEqual(l.width, 7.0)
+        ln2 = ln._replace(stop=Point(3, 4), width=0.5)
+        self.assertEqual(ln.start.x, 1.0)
+        self.assertEqual(ln.start.y, 2.0)
+        self.assertEqual(ln.stop.x, 8.0)
+        self.assertEqual(ln.stop.y, 9.0)
+        self.assertEqual(ln.width, 7.0)
 
-        self.assertEqual(l2.start.x, 1.0)
-        self.assertEqual(l2.start.y, 2.0)
-        self.assertEqual(l2.stop.x, 3.0)
-        self.assertEqual(l2.stop.y, 4.0)
-        self.assertEqual(l2.width, 0.5)
+        self.assertEqual(ln2.start.x, 1.0)
+        self.assertEqual(ln2.start.y, 2.0)
+        self.assertEqual(ln2.stop.x, 3.0)
+        self.assertEqual(ln2.stop.y, 4.0)
+        self.assertEqual(ln2.width, 0.5)
 
     def testCreateImplicit(self):
         tp = objc.createStructType("BarStruct", b'{_BarStruct="e"f"f"f"g"f"h"f}', None)
@@ -103,8 +118,12 @@ class TestStructs(TestCase):
 
         self.assertEqual(tp._fields, ("e", "f", "g", "h"))
 
-        self.assertRaises(ValueError, objc.createStructType, "Foo2", b'{_Foo=f"a"}', None)
-        self.assertRaises(ValueError, objc.createStructType, "Foo3", b'{_Foo="a"f', None)
+        self.assertRaises(
+            ValueError, objc.createStructType, "Foo2", b'{_Foo=f"a"}', None
+        )
+        self.assertRaises(
+            ValueError, objc.createStructType, "Foo3", b'{_Foo="a"f', None
+        )
         self.assertRaises(
             ValueError, objc.createStructType, "Foo4", b'^{_Foo="a"f}', None
         )
@@ -161,6 +180,7 @@ class TestStructs(TestCase):
                     )
                 ],
             )
+            self.assertIsInstance(StructArrayDelegate, objc.informal_protocol)
 
             class OC_PyStruct(NSObject):
                 def arrayOf4Structs_(self, value):
@@ -214,13 +234,19 @@ class TestStructs(TestCase):
         with pyobjc_options(structs_indexable=False, structs_writable=True):
             v = tp0(1, 2)
 
-            with self.assertRaises(TypeError):
+            with self.assertRaisesRegex(
+                TypeError, "Instances of 'FooStruct' are not sequences"
+            ):
                 v[0]
 
-            with self.assertRaises(TypeError):
+            with self.assertRaisesRegex(
+                TypeError, "Instances of 'FooStruct' are not sequences"
+            ):
                 v[0] = 4
 
-            with self.assertRaises(TypeError):
+            with self.assertRaisesRegex(
+                TypeError, "Instances of 'FooStruct' are not sequences"
+            ):
                 len(v)
 
             self.assertFalse(v == (1, 2))
@@ -265,6 +291,171 @@ class TestStructs(TestCase):
 
             self.assertRaises(TypeError, tp0, 4, 5, 6)
 
+    def test_struct_as_sequence(self):
+        tp0 = objc.createStructType(
+            "FooStruct2", b'{FooStruct2="first"i"second"i"third"i}', None
+        )
 
-if __name__ == "__main__":
-    main()
+        with pyobjc_options(structs_indexable=True, structs_writable=True):
+            v = tp0(1, 2, 3)
+
+            #  Read (item, slice)
+            self.assertEqual(v[0], 1)
+            self.assertEqual(v[-1], 3)
+            self.assertEqual(v[:], (1, 2, 3))
+
+            self.assertEqual(v[-4:8], (1, 2, 3))
+
+            self.assertEqual(v[1:2], (2,))
+            self.assertEqual(v[::2], (1, 3))
+
+            with self.assertRaisesRegex(IndexError, "FooStruct2 index out of range"):
+                v[5]
+
+            with self.assertRaisesRegex(IndexError, "FooStruct2 index out of range"):
+                v[-6]
+
+            with self.assertRaises(IndexError):
+                v[2 ** 68]
+
+            self.assertEqual(v[2 ** 68 :], ())
+            self.assertEqual(v[2 ** 68 : 2 ** 68 + 3], ())
+
+            # Write (item, slice)
+            v[0] = 4
+            self.assertEqual(v.first, 4)
+
+            v[0:2] = iter((9, 10))
+            self.assertEqual(v.first, 9)
+            self.assertEqual(v.second, 10)
+            self.assertEqual(v.third, 3)
+
+            v[::2] = (-1, -2)
+            self.assertEqual(v.first, -1)
+            self.assertEqual(v.second, 10)
+            self.assertEqual(v.third, -2)
+
+            with self.assertRaisesRegex(
+                TypeError, "Slice assignment would change size of FooStruct2 instance"
+            ):
+                v[:] = range(10)
+
+            with self.assertRaisesRegex(TypeError, "Must assign sequence to slice"):
+                v[:] = 42
+
+            with self.assertRaisesRegex(IndexError, "FooStruct2 index out of range"):
+                v[5] = 42
+
+            with self.assertRaisesRegex(IndexError, "FooStruct2 index out of range"):
+                v[-6] = 42
+
+            with self.assertRaises(IndexError):
+                v[2 ** 68] = 4
+
+            with self.assertRaises(TypeError):
+                v[2 ** 68 :] = (4,)
+
+            # Delete (item slice)
+            with self.assertRaisesRegex(
+                TypeError, "Cannot delete item '1' in a FooStruct2 instance"
+            ):
+                del v[1]
+
+            with self.assertRaisesRegex(
+                TypeError, "Cannot delete items in instances of FooStruct2"
+            ):
+                del v[1:3]
+
+            # Wrong type
+            with self.assertRaisesRegex(
+                TypeError, "Struct indices must be integers, not str"
+            ):
+                v["first"]
+
+            with self.assertRaisesRegex(
+                TypeError, "Struct indices must be integers, not str"
+            ):
+                v["first"] = 22
+
+            v = tp0(1, 2, 3)
+            self.assertIn(2, v)
+            self.assertNotIn(9, v)
+
+            with self.assertRaisesRegex(TypeError, "FooStruct2 objects are unhashable"):
+                hash(v)
+
+    def test_struct_as_attributes(self):
+        tp0 = objc.createStructType(
+            "FooStruct4", b'{FooStruct4="first"i"second"i"third"i}', None
+        )
+
+        with pyobjc_options(structs_indexable=False, structs_writable=True):
+            v = tp0(1, 2, 3)
+
+            #  Read
+            self.assertEqual(v.first, 1)
+
+            # Write
+            v.first = 4
+
+            # Delete
+            with self.assertRaisesRegex(
+                TypeError, "Cannot delete attributes of FooStruct4"
+            ):
+                del v.second
+
+    def test_no_sublcassing(self):
+        tp0 = objc.createStructType(
+            "FooStruct5", b'{FooStruct4="first"i"second"i"third"i}', None
+        )
+
+        with self.assertRaisesRegex(
+            TypeError, "type 'FooStruct5' is not an acceptable base type"
+        ):
+
+            class MyStruct(tp0):
+                pass
+
+    def test_struct_pickling(self):
+        orig = GlobalType(9, 10)
+        self.assertEqual(orig.a, 9.0)
+        self.assertEqual(orig.b, 10.0)
+
+        buf = pickle.dumps(orig)
+        self.assertIsInstance(buf, bytes)
+
+        copy = pickle.loads(buf)
+        self.assertIsInstance(copy, GlobalType)
+        self.assertEqual(copy.a, 9.0)
+        self.assertEqual(copy.b, 10.0)
+
+        self.assertEqual(copy, orig)
+
+    @expectedFailure
+    def test_struct_naming(self):
+        self.assertEqual(GlobalType.__name__, "GlobalType")
+        self.assertEqual(GlobalType.__module__, "PyObjCTest.test_structs")
+        self.assertEqual(GlobalType.__qualname__, "PyObjCTest.test_structs.GlobalType")
+
+    def test_sizeof(self):
+        self.assertIsInstance(GlobalType().__sizeof__(), int)
+
+        tp0 = objc.createStructType(
+            "FooStruct3", b'{FooStruct3="first"i"second"i"third"i}', None
+        )
+
+        self.assertGreaterThan(tp0().__sizeof__(), GlobalType().__sizeof__())
+
+    def todo_test_struct_defaults(self):
+        # Check default values for various types
+        # - char, unsigned char, char_as_int, char_as_text
+        # - short,  unsigned short
+        # - int,  unsigned int
+        # - long,  unsigned long
+        # - long long,  unsigned long long
+        # - float
+        # - double
+        # - bool, BOOL
+        # - nested structs
+        # - nested arrays
+        self.fail()
